/*!
$Id: camera.h 570 2009-11-27 07:09:29Z horino $
*/
#ifndef CAMERA_H
#define CAMERA_H

#ifdef _DEBUG
#include <iostream>
using namespace std;
#endif

#include "indi.h"
#include "graphics/graphics.h"
#include "vector3.h"
#include "matrix44.h"

namespace indi
{
	class Camera 
	{
	public:
		Camera(const Vector3 &p, const Vector3 &t, const Vector3 &u, const Matrix44 &prj) : _position(p), _target(t), _up(u), _projection(prj) {}
		Camera() : _position(Vector3::UnitZ() * 10), _target(Vector3::Zero()), _up(Vector3::Up()), _projection(Matrix44::Perspective(INDI_PI * 0.25f, 1.0f, 1.0f, 200.0f)) {}
		virtual ~Camera(){}

		const Vector3 &Position() const { return _position; }
		const Vector3 &Target() const { return _target; }
		const Vector3 &Up() const { return _up; }
		Vector3 &Position() { return _position; }
		Vector3 &Target() { return _target; }
		Vector3 &Up() { return _up; }

		const Vector3 &Forward() const { return _forward; }
		const Vector3 &Right() const { return _right; }
		const Vector3 &Left() const { return _left; }
		Vector3 &Forward() { return _forward; }
		Vector3 &Right() { return _right; }
		Vector3 &Left() { return _left; }

		const Matrix44 &View() const { return _view; }
		const Matrix44 &Projection() const { return _projection; }
		const Matrix44 &ViewProjection() const { return _viewProjection; }
		const Matrix44 &InverseTranspose() const { return _inverseTranspose; }
		Matrix44 &View() { return _view; }
		Matrix44 &Projection() { return _projection; }
		Matrix44 &ViewProjection() { return _viewProjection; }
		Matrix44 &InverseTranspose() { return _inverseTranspose; }

		virtual void Update(){
			_forward = _target - _position; 
			_forward.Normalize();

			_left = (Vector3::Up() ^ _forward); 
			_left.Normalize();

			_right = -_left;

			_up = (_forward ^ _left); 
			_up.Normalize();

			//_world = Matrix44::Translation(_position);
			_view = Matrix44::LookAt(_position, _target, _up);
			_viewProjection = _view * _projection;
			_inverseTranspose = _viewProjection.Inverse().Transpose();
		}

		virtual void Control() {}

	protected:
		Vector3 _position;
		Vector3 _target;
		Vector3 _up;

		Vector3 _forward;
		Vector3 _right;
		Vector3 _left;

		Matrix44 _view;
		Matrix44 _projection;
		Matrix44 _viewProjection;
		Matrix44 _inverseTranspose;

		//BoundingFrustum frustum;
	};
};

#endif // CAMERA_H