/*!
$Id: fpscamera.h 570 2009-11-27 07:09:29Z horino $
*/
#ifndef FPSCAMERA_H
#define FPSCAMERA_H

#ifdef _DEBUG
#include <iostream>
using namespace std;
#endif

#include "indi.h"
#include "vector2.h"
#include "vector3.h"
#include "quaternion.h"
#include "input/input.h"
#include "camera/camera.h"

namespace indi
{
	class FpsCamera : public Camera
	{
	public:
		FpsCamera() : Camera(), _angle(Vector2::Zero()) {}
		FpsCamera(const Vector3 &p, const Vector3 &t, const Vector3 &u, const Matrix44 &prj) : Camera(p, t, u, prj), _angle(Vector2::Zero()) {}
		virtual ~FpsCamera() {}

		void Control(const indi::Input::State &inpStat) {
			f32 rot_spped = 2.0f;
			f32 move_spped = 0.5f;

			Vector2 look(inpStat.RightStick() * rot_spped);
			f32 x(_angle.X() - look.X()); 
			while(x < 0.0f) x += 360.0f;
			while(x > 360.0f) x -= 360.0f;
			_angle.Set(x, math::Clamp(_angle.Y() + look.Y(), 150.0f, 240.0f));

			Vector3 dir(Vector3::UnitZ());
			dir = Quaternion(Vector3::UnitY(), math::ToRadian(_angle.X())).Rotate(dir); 
			Vector3 right = dir ^ Vector3::UnitY();
			dir = Quaternion(right, math::ToRadian(_angle.Y())).Rotate(dir);
			dir.Normalize();

			Vector3 d(Vector3(dir.X(), 0.0f, dir.Z()).Normalize());
			Vector3 r = d ^ Vector3::UnitY();
			_position += d * inpStat.LeftStickYf() * move_spped + r * inpStat.LeftStickXf() * move_spped;

			_target = _position + dir;
		}
	private:
		Vector2 _angle;
	};
};

#endif // FPSCAMERA_H