/*!
$Id: collision.h 652 2010-01-08 09:54:38Z horino $
*/
#ifndef COLLISION_H
#define COLLISION_H

#include "indi.h"
#include "vector3.h"
#include "vector4.h"

namespace indi
{
	/*!
	\brief RW
	\note AABBƋ̃RW̏ꍇASphereAABB()ł͂ȂAABBSphere()̂悤ɃAt@xbg(AABB)ɂȂĂ܂B
	*/
	namespace Collision
	{
		/*!
		\brief 
		*/
		class Distance
		{
		public:
			//< Plane*
			static f32 PlanePoint(const Vector3 &nPl, f32 dPl,
				const Vector3 p);

			/*!
			\brief Q
			*/
			class Squared
			{
			public:
				//< AABB*
				static f32 AABBPoint(const Vector3 &cAB, const Vector3 &rAB, 
					const Vector3 &p);
				//< OBB*
				static f32 OBBPoint(const Vector3 &cOB, const Vector3 &rOB, const Vector3 &xOB, const Vector3 &yOB, const Vector3 &zOB, 
					const Vector3 &p);
				//< Point*
				static f32 PointSegment(const Vector3 &p,
					const Vector3 &seg0, const Vector3 &seg1);
				//< Segment*
				static f32 SegmentSegment(const Vector3 &segA0, const Vector3 &segA1,
					const Vector3 &segB0, const Vector3 &segB1,
					f32 &t0, f32 &t1);
				static f32 SegmentSegment(const Vector3 &segA0, const Vector3 &segA1,
					const Vector3 &segB0, const Vector3 &segB1) { f32 t0, t1; return SegmentSegment(segA0, segA1, segB0, segB1, t0, t1); }
			};
		};

		/*!
		\brief ŋߐړ_
		*/
		class ClosestPoint
		{
		public:
			//< AABB*
			static Vector3 AABBPoint(const Vector3 &cAB, const Vector3 &rAB, 
				const Vector3 &p);
			//< OBB*
			static Vector3 OBBPoint(const Vector3 &cOB, const Vector3 &rOB, const Vector3 &xOB, const Vector3 &yOB, const Vector3 &zOB, 
				const Vector3 &p);
			//< Plane*
			static Vector3 PlanePoint(const Vector3 &nPl, f32 dPl,
				const Vector3 p);
			//< Point*
			static Vector3 PointSegment(const Vector3 &p,
				const Vector3 &seg0, const Vector3 &seg1,
				f32 &t);
			static Vector3 PointSegment(const Vector3 &p,
				const Vector3 &seg0, const Vector3 &seg1) { f32 t; return PointSegment(p, seg0, seg1, t); }
			//< Segment*
			static void SegmentSegment(const Vector3 &segA0, const Vector3 &segA1,
				const Vector3 &segB0, const Vector3 &segB1, 
				f32 &t0, f32 &t1);
			static void SegmentSegment(const Vector3 &segA0, const Vector3 &segA1,
				const Vector3 &segB0, const Vector3 &segB1, 
				Vector3 &r0, Vector3 &r1) { f32 t0, t1; SegmentSegment(segA0, segA1, segB0, segB1, t0, t1); r0 = segA0 + (segA1 - segA0) * t0; r1 = segB0 + (segB1 - segB0) * t1; }
		};

		/*!
		\brief Փˌo
		*/
		class Detection
		{
		public:
			//< AABB*
			static BOOL AABBAABB(const Vector3 &cAB0, const Vector3 &rAB0,
				const Vector3 &cAB1, const Vector3 &rAB1);
			static BOOL AABBAABB(const Vector3 &cAB0, const Vector3 &rAB0, const Vector3 &vAB0,
				const Vector3 &cAB1, const Vector3 &rAB1, const Vector3 &vAB1,
				f32 &t);
			static BOOL AABBAABB(const Vector3 &cAB0, const Vector3 &rAB0, const Vector3 &vAB0,
				const Vector3 &cAB1, const Vector3 &rAB1, const Vector3 &vAB1) { f32 t; return AABBAABB(cAB0, rAB0, vAB0, cAB1, rAB1, vAB1, t); }
			static BOOL AABBPlane(const Vector3 &cAB, const Vector3 &rAB, 
				const Vector3 &nPl, f32 dPl);
			static BOOL AABBSolidPlane(const Vector3 &cAB, const Vector3 &rAB, 
				const Vector3 &nPl, f32 dPl);
			static BOOL AABBPlane(const Vector3 &cAB, const Vector3 &rAB, const Vector3 &vAB,
				const Vector3 &nPl, float dPl);
			static BOOL AABBPlane(const Vector3 &cAB, const Vector3 &rAB, const Vector3 &vAB,
				const Vector3 &nPl, float dPl,
				f32 &t);
			static BOOL AABBSolidPlane(const Vector3 &cAB, const Vector3 &rAB, const Vector3 &vAB,
				const Vector3 &nPl, float dPl);
			static BOOL AABBSolidPlane(const Vector3 &cAB, const Vector3 &rAB, const Vector3 &vAB,
				const Vector3 &nPl, float dPl,
				f32 &t);
			static BOOL AABBPoint(const Vector3 &cAB, const Vector3 &rAB,
				const Vector3 &p);
			static BOOL AABBRay(const Vector3 &cAb, const Vector3 &rAb, 
				const Vector3 &sRay, const Vector3 &dRay,
				f32 &t);
			static BOOL AABBSegment(const Vector3 &cAb, const Vector3 &rAb, 
				const Vector3 &seg0, const Vector3 &seg1);
			static BOOL AABBSegment(const Vector3 &cAb, const Vector3 &rAb, 
				const Vector3 &seg0, const Vector3 &seg1,
				f32 &t);
			static BOOL AABBSphere(const Vector3 &cAB, const Vector3 &rAB,
				const Vector3 &cSp, f32 rSp);
			static BOOL AABBSphere(const Vector3 &cAB, const Vector3 &rAB,
				const Vector3 &cSp, f32 rSp, const Vector3 &vSp,
				f32 &t);
			static BOOL AABBSphere(const Vector3 &cAB, const Vector3 &rAB,
				const Vector3 &cSp, f32 rSp, const Vector3 &vSp) { f32 t; return AABBSphere(cAB, rAB, cSp, rSp, vSp, t); }
			//< Capsule*
			static BOOL CapsuleCapsule(const Vector3 &cpA0, const Vector3 &cpA1, f32 rCpA, 
				const Vector3 &cpB0, const Vector3 &cpB1, f32 rCpB);
			static BOOL CapsuleSegment(const Vector3 &cp0, const Vector3 &cp1, f32 rCp,
				const Vector3 &seg0, const Vector3 &seg1,
				f32 &t);
			static BOOL CapsuleSegment(const Vector3 &cp0, const Vector3 &cp1, f32 rCp,
				const Vector3 &seg0, const Vector3 &seg1) { f32 t; return CapsuleSegment(cp0, cp1, rCp, seg0, seg1, t); }
			static BOOL CapsuleSphere(const Vector3 &cp0, const Vector3 &cp1, f32 rCp, 
				const Vector3 &cSp, f32 rSp);
			//< Cylinder*
			static BOOL CylinderSegment(const Vector3 &cy0, const Vector3 &cy1, f32 rCy, 
				const Vector3 &seg0, const Vector3 &seg1,
				f32 &t);
			static BOOL CylinderSegment(const Vector3 &cy0, const Vector3 &cy1, f32 rCy, 
				const Vector3 &seg0, const Vector3 &seg1) { f32 t; return CylinderSegment(cy0, cy1, rCy, seg0, seg1, t); }
			//< OBB*
			static BOOL OBBOBB(const Vector3 &cOB0, const Vector3 &rOB0, const Vector3 &xOB0, const Vector3 &yOB0, const Vector3 &zOB0,
				const Vector3 &cOB1, const Vector3 &rOB1, const Vector3 &xOB1, const Vector3 &yOB1, const Vector3 &zOB1);
			static BOOL OBBPlane(const Vector3 &cOB, const Vector3 &rOB, const Vector3 &xOB, const Vector3 &yOB, const Vector3 &zOB,
				const Vector3 &nPl, f32 dPl);
			static BOOL OBBPlane(const Vector3 &cOB, const Vector3 &rOB, const Vector3 &xOB, const Vector3 &yOB, const Vector3 &zOB, const Vector3 &vOB,
				const Vector3 &nPl, f32 dPl);
			static BOOL OBBPlane(const Vector3 &cOB, const Vector3 &rOB, const Vector3 &xOB, const Vector3 &yOB, const Vector3 &zOB, const Vector3 &vOB,
				const Vector3 &nPl, f32 dPl,
				f32 &t);
			static BOOL OBBSphere(const Vector3 &cOB, const Vector3 &rOB, const Vector3 &xOB, const Vector3 &yOB, const Vector3 &zOB,
				const Vector3 &cSp, f32 rSp);
			//< Plane*
			static BOOL PlanePlane(const Vector3 &nPl0, f32 dPl0, 
				const Vector3 &nPl1, f32 dPl1, 
				Vector3 &sLn, Vector3 &dLn);
			static BOOL PlanePlane(const Vector3 &nPl0, f32 dPl0, 
				const Vector3 &nPl1, f32 dPl1){ Vector3 sLn, dLn; return PlanePlane(nPl0, dPl0, nPl1, dPl1, sLn, dLn); }
			static BOOL PlaneSegment(const Vector3 &nPl, f32 dPl, 
				const Vector3 &seg0, const Vector3 &seg1,
				f32 &t);
			static BOOL PlaneSegment(const Vector3 &nPl, f32 dPl, 
				const Vector3 &seg0, const Vector3 &seg1) { f32 t; return PlaneSegment(nPl, dPl, seg0, seg1, t); }
			static BOOL PlaneSphere(const Vector3 &nPl, f32 dPl,  
				const Vector3 &cSp, f32 rSp);
			static BOOL SolidPlaneSphere(const Vector3 &nPl, f32 dPl,  
				const Vector3 &cSp, f32 rSp);
			static BOOL PlaneSphere(const Vector3 &nPl, f32 dPl,
				const Vector3 &cSp, f32 rSp, const Vector3 &vSp, 
				f32 &t);
			static BOOL PlaneSphere(const Vector3 &nPl, f32 dPl,
				const Vector3 &cSp, f32 rSp, const Vector3 &vSp);
			//< Ray*
			static BOOL RaySphere(const Vector3 &sRay, const Vector3 &dRay,
				const Vector3 &cSp, f32 rSp,
				f32 &t);
			static BOOL RaySphere(const Vector3 &sRay, const Vector3 &dRay,
				const Vector3 &cSp, f32 rSp);
			//< Segment*
			static BOOL SegmentSphere(const Vector3 &seg0, const Vector3 &seg1,
				const Vector3 &cSp, f32 rSp,
				f32 &t);
			static BOOL SegmentSphere(const Vector3 &seg0, const Vector3 &seg1,
				const Vector3 &cSp, f32 rSp) { f32 t; return SegmentSphere(seg0, seg1, cSp, rSp, t); }
			//< Sphere*
			static BOOL SphereSphere(const Vector3 &cSp0, f32 rSp0, 
				const Vector3 &cSp1, f32 rSp1);
			static BOOL SphereSphere(const Vector3 &cSp0, f32 rSp0, const Vector3 &vSp0,
				const Vector3 &cSp1, f32 rSp1, const Vector3 &vSp1,
				f32 &t);
			static BOOL SphereSphere(const Vector3 &cSp0, f32 rSp0, const Vector3 &vSp0,
				const Vector3 &cSp1, f32 rSp1, const Vector3 &vSp1) { f32 t; return SphereSphere(cSp0, rSp0, vSp0, cSp1, rSp1, vSp1, t); }
		};

		class Util
		{
		public:
			static Vector4 Plane(const Vector3 &a, const Vector3 &b, const Vector3 &c) {
#ifdef PS3
				Vector3 n(((b - a) ^ (c - b)).Normalize());
				return Vector4(n, - n * a);
#else
				D3DXPLANE pl;   
				return Vector4(D3DXPlaneNormalize(&pl, D3DXPlaneFromPoints(&pl, a, b, c)));
#endif
			}
		};

		/*!
		\brief 
		*/
		class Frustum
		{
		public:
			Frustum() {}
			Frustum(u32 x, u32 y, u32 w, u32 h, f32 n, f32 f) { Create(x, y, w, h, n, f); }

			void Create(u32 x, u32 y, u32 w, u32 h, f32 n, f32 f);
			void Update(const Matrix44 &world, const Matrix44 &view, const Matrix44 &projection);

			static BOOL Contains(const Vector4 &t, const Vector4 &b, const Vector4 &l, const Vector4 &r, const Vector4 &n, const Vector4 &f,
				const Vector3 &center, f32 radius);

			/*!
			\brief i_jɊ܂܂邩
			\param ̒Si܂͓_j
			\param ̔ai_̏ꍇ0.0fɂĂj
			\return ܂܂ĂTRUE
			\note _𔻒肷ꍇradius0.0fw肷΂悢
			*/
			BOOL Contains(const Vector3 &center, f32 radius) { return Contains(_top, _bottom, _left, _right, _near, _far, center, radius); }

			static BOOL Contains(const Vector4 &t, const Vector4 &b, const Vector4 &l, const Vector4 &r, const Vector4 &n, const Vector4 &f,
				const Vector3 &center, const Vector3 &radius);

			/*!
			\brief AABBɊ܂܂邩
			\param AABB̒S
			\param AABB̔a
			\return ܂܂ĂTRUE
			*/
			BOOL Contains(const Vector3 &center, const Vector3 &radius) { return Contains(_top, _bottom, _left, _right, _near, _far, center, radius); }

		private:
			Vector3 _vertices[8];

			Vector4 _top;
			Vector4 _bottom;
			Vector4 _left;
			Vector4 _right;
			Vector4 _near;
			Vector4 _far;
		};
	};
};

#endif //< COLLISION_H