/*!
$Id: bitset.h 301 2009-08-21 03:02:27Z horino $
*/
#ifndef BITSET_H
#define BITSET_H

#ifdef _DEBUG
#include <iostream>
using namespace std;
#endif

#include "indi.h"

namespace indi
{
	namespace itl
	{
		/*!
		stlbitsetǂ
		*/
		class Bitset
		{
		public:
			Bitset() : _raw(0) {}
			Bitset(u32 v) : _raw(v) {}

			Bitset &operator&=(const Bitset &rhs) {	_raw &= rhs._raw; return *this; }
			Bitset &operator|=(const Bitset &rhs) {	_raw |= rhs._raw; return *this; }
			Bitset &operator^=(const Bitset &rhs) {	_raw ^= rhs._raw; return *this; }
			Bitset &operator<<=(u32 pos) { _raw <<= pos; return *this; }
			Bitset &operator>>=(u32 pos) { _raw >>= pos; return *this; }
			Bitset operator~() const { return Bitset(*this).Flip(); }
//			BOOL operator==(const Bitset &rhs) const { return (_raw == rhs._raw); }
//			BOOL operator!==(const Bitset &rhs) const { return (_raw != rhs._raw); }
			Bitset operator<<(u32 pos) const { return Bitset(*this) <<= pos; }
			Bitset operator>>(u32 pos) const { return Bitset(*this) >>= pos; }

			u32 To_ulong() const { return _raw; }

			Bitset &Set() { _raw = 0xffffffff; return *this; }
			Bitset &Set(u32 pos) { _raw |= (1 << pos); return *this; }

			Bitset &Reset() { _raw = 0; return *this; }
			Bitset &Reset(u32 pos) { _raw &= ~(1 << pos); return *this; }

			Bitset &Flip() { _raw = ~_raw; return *this; }
			Bitset &Flip(u32 pos) { _raw ^= (1 << pos); return *this; }

			BOOL Test(u32 pos) const { return (_raw & (1 << pos)); }
			BOOL Any() const { return !(_raw == 0); }
			BOOL None() const { return (_raw == 0); }

			u32 Size() const { return 32; }
			u32 Count() const { u32 n(0); for(u32 i = 0;i < Size();++i) if(Test(i)) ++n; return n; }

		private:
			u32 _raw;
		};
	};
};

#ifdef _DEBUG
#ifdef PS3
static ostream __attribute__((__used__)) &operator<<(ostream &lhs, const indi::itl::Bitset &rhs) 
{
	for(indi::u32 i = 0;i < rhs.Size();++i){
		if((i % 4) == 0) cout << "|";
		if(rhs.Test(i)) cout << "x";
		else cout << "-";
	}
	cout << "|" << endl;
	return lhs; 
}
#else
static ostream &operator<<(ostream &lhs, const indi::itl::Bitset &rhs) 
{
	for(indi::u32 i = 0;i < rhs.Size();++i){
		if((i % 4) == 0) cout << "|";
		if(rhs.Test(i)) cout << "x";
		else cout << "-";
	}
	cout << "|" << endl;
	return lhs; 
}
#endif
#endif

#endif // BITSET_H