/*!
$Id: functional.h 295 2009-08-20 05:16:37Z horino $
*/
#ifndef FUNCTIONAL_H
#define FUNCTIONAL_H

namespace indi
{
	template<typename A, typename R>
	struct Unary_function {
		typedef A argument_type;
		typedef R result_type;
	};
	template<typename A1, typename A2, typename R>
	struct Binary_function {
		typedef A1 first_argument_type;
		typedef A2 second_argument_type;
		typedef R result_type;
	};

	template<typename R, typename T>
	class Mem_fun_t : public Unary_function<T, R> {
	public:
		explicit Mem_fun_t(R(T::*f)()) : func(f) {}
		R operator()(T *p) const { return (p->*func)(); }
	private:
		R (T::*func)();
	};
	template<typename R, typename T>
	class Const_mem_fun_t : public Unary_function<T, R> {
	public:
		explicit Const_mem_fun_t(R(T::*f)() const) : func(f) {}
		R operator()(const T *p) const { return (p->*func)(); }
	private:
		R (T::*func)() const;
	};

	template<typename R, typename T>
	class Mem_fun_ref_t : public Unary_function<T, R> {
	public:
		explicit Mem_fun_ref_t(R(T::*f)()) : func(f) {}
		R operator()(T &r) const { return (r.*func)(); }
	private:
		R (T::*func)();
	};
	template<typename R, typename T>
	class Const_mem_fun_ref_t : public Unary_function<T, R> {
	public:
		explicit Const_mem_fun_ref_t(R(T::*f)() const) : func(f) {}
		R operator()(const T &r) const { return (r.*func)(); }
	private:
		R (T::*func)() const;
	};

	template<typename R, typename T>
	Mem_fun_t<R, T> Mem_fun(R (T::*f)()){ return Mem_fun_t<R, T>(f); }
	template<typename R, typename T>
	Const_mem_fun_t<R, T> Mem_fun(R (T::*f)() const){ return Const_mem_fun_t<R, T>(f); }

	template<typename R, typename T>
	Mem_fun_ref_t<R, T> Mem_fun_ref(R (T::*f)()){ return Mem_fun_ref_t<R, T>(f); }
	template<typename R, typename T>
	Const_mem_fun_ref_t<R, T> Mem_fun_ref(R (T::*f)() const){
		return Const_mem_fun_ref_t<R, T>(f); 
	}

	template<typename O>
	class Binder1st : public Unary_function<typename O::second_argument_type, typename O::resut_type>{
	protected:
		O op;
		typename O::first_argument_type val;
	public:
		Binder1st(const O &x, const typename O::first_argument_type &y) : op(x), val(y) {}
		typename O::result_type operator()(const typename O::second_argument_type &x) const {
			return op(val, x);
		}
	};
	template<typename O, typename T>
	Binder1st<O> Bind1st(const O &f, const T &x){
		return Binder1st<O>(f, typename O::first_argument_type(x));
	};

	template<typename O>
	class Binder2nd : public Unary_function<typename O::first_argument_type, typename O::result_type>{
	protected:
		O op;
		typename O::second_argument_type val;
	public:
		Binder2nd(const O &x, const typename O::second_argument_type &y) : op(x), val(y) {}
		typename O::result_type operator()(const typename O::first_argument_type &x) const {
			return op(x, val);
		}
	};
	template<typename O, typename T>
	Binder2nd<O> Bind2nd(const O &f, const T &x){
		return Binder2nd<O>(f, typename O::second_argument_type(x));
	}
};

#endif // FUNCTIONAL_H