/*!
$Id: aabb.h 629 2009-12-21 04:06:49Z horino $
*/
#ifndef AABB_H
#define AABB_H

#include "vector3.h"

namespace indi
{
	namespace mesh
	{
		/*!
		\brief AABBNX
		\note Axis Aligned Bounding Box
		*/
		class AABB
		{
		public:
			AABB() : _center(Vector3::Zero()), _radius(Vector3::Zero()) {}
			virtual ~AABB() {}

			Vector3 Min() const { return _center - _radius; }
			Vector3 Max() const { return _center + _radius; }

			AABB &Add(const Vector3 &rhs) { return (*this) += rhs; }
			AABB &Add(const AABB &rhs) { return (*this) += rhs; }
			AABB &operator+=(const Vector3 &rhs) {
				Vector3 mn(Min());
				Vector3 mx(Max());
				mn.Set(math::Min(rhs.X(), mn.X()), math::Min(rhs.Y(), mn.Y()), math::Min(rhs.Z(), mn.Z()));
				mx.Set(math::Max(rhs.X(), mx.X()), math::Max(rhs.Y(), mx.Y()), math::Max(rhs.Z(), mx.Z()));
				_radius = (mx - mn) * 0.5f;
				_center = (mx + mn) * 0.5f;
				return *this; 
			}
			AABB &operator+=(const AABB &rhs) {
				*this += rhs.Min();
				*this += rhs.Max();
				return *this; 
			}
			void Reset() { _center = Vector3::Zero(); _radius = Vector3::Zero(); }

			Vector3 &Center() { return _center; }
			Vector3 &Radius() { return _radius; }

		protected:
			Vector3 _center;
			Vector3 _radius;
		};
	}
}

#endif // AABB_H