/*!
$Id: animation.h 623 2009-12-17 04:07:13Z horino $
*/
#ifndef ANIMATION_H
#define ANIMATION_H

#ifdef _DEBUG
#include <iostream>
using namespace std;
#endif

#include "file/file.h"
#include "graphics/texture.h"
#include "vector2.h"

namespace indi
{
	namespace mesh
	{
        /*!
		\brief Aj[VRg[wb_NX
		\note Rg[̃Aj[V
		*/
        class AnimationControllerHeader 
		{
		public:
			AnimationControllerHeader() : _base_y(0), _time_num(0) {}
			virtual ~AnimationControllerHeader() { if(_times) { delete [] _times; } }

			virtual u32 Read(const u8 *data);

			/*!
			\brief eNX`x[XxWi[JjԂ܂
			\return eNX`x[XxWi[Jj
			*/
			u32 GetBaseY() const { return _base_y; }

			/*!
			\brief ݂̌Ԃ܂
			\return ݐ
			*/
			u32 GetTimeNum() const { return _time_num; }

			/*!
			\brief (b)Ԃ܂
			\param Y
			\return (b)
			*/
			f32 GetTime(u32 i) const { ASSERT(i < _time_num); return _times[i]; }

			/*!
			\brief Yl̍őlԂ܂
			\return ő̃eNX`xWi[Jj
			*/
			u32 GetMaxY() const { return _time_num - 1; }

			/*!
			\brief ɂeNX`xWi[JjԂ܂
			\param ibj
			\return eNX`xWi[Jj
			*/
			u32 GetY(f32 t) const { 
				for(u32 i=_time_num - 1;i>=0;--i)
					if(_times[i] <= t) 
						return i;
				return 0;
			}
 
			/*!
			\brief ɂeNX`xWi[JjԂ܂
			\note ő𒴂Ăꍇ̓[v܂
			\param ibj
			\return eNX`xWi[Jj
			*/
			u32 GetLoopY(f32 t) const {
				f32 d(GetDuration());
				while(t > d) t -= d;
				return GetY(t);
			}

			/*!
			\brief L[t[ۊǂۂ̒[lāAԃp[^[0.0f - 1.0f]ŕԂ܂
			\param ibj
			\return ԃp[^
			\note 0.0fɋ߂݂̃t[ɋ߂A1.0fɋ߂̃t[ɋ߂
			*/
			f32 GetRatio(f32 t) const {
				f32 next_time(_times[_time_num - 1]);
				for(u32 i=_time_num - 1;i>=0;--i){
					if(_times[i] <= t) {
						f32 dif(next_time - _times[i]);
						if(dif < LIM_EPSILON(f32))
							return 1.0f;
						else
							return 1.0f - (next_time - t) / dif;
					}
					next_time = _times[i];
				}
#ifdef _DEBUG
				ASSERT(0);
#endif
				return 0.0f;
			}

			/*!
			\brief L[t[ۊǂۂ̒[lāAԃp[^[0.0f - 1.0f]ŕԂ܂
			\note ő𒴂Ăꍇ̓[v܂
			\param ibj
			\return ԃp[^
			*/
			f32 GetLoopRatio(f32 t) const {
				f32 d(GetDuration());
				while(t > d) t -= d;
				return GetRatio(t);
			}

			/*!
			\brief Aj[V̍ő厞Ԃ܂
			\return Aj[V̍ő厞
			*/
			f32 GetDuration() const { return _times[_time_num - 1]; }

		protected:
			u32 _base_y;
			u32 _time_num;
			f32 *_times;
		};

        /*!
		\brief Aj[V}bvwb_NX
		\note NbṽAj[V
		*/
        class AnimationMapHeader
		{
		public:
			AnimationMapHeader() : _id(0), _base_y(0), _controller_num(0), _controllers(0), _current(0) {}
			virtual ~AnimationMapHeader() {
				if(_id)
					delete [] _id; 
				if(_controllers) {
					for(u32 i=0;i<_controller_num;++i)
						if(_controllers[i])
							delete [] _controllers[i];
					delete [] _controllers;
				}
			}

			virtual u32 Read(const u8 *data);

			virtual AnimationControllerHeader *AnimationControllerHeaderCreator() const { return new AnimationControllerHeader(); }

			const char *GetID() const { return _id; }
			char *GetID() { return const_cast<char *>(static_cast<const AnimationMapHeader *>(this)->GetID()); }

			u32 GetBaseY() const { return _base_y; }

			u32 GetAnimationControllerHeaderNum() const { return _controller_num; }

			const AnimationControllerHeader *GetAnimationControllerHeader(u32 i) const { ASSERT(i < _controller_num); return _controllers[i]; }
			AnimationControllerHeader *GetAnimationControllerHeader(u32 i) { return const_cast<AnimationControllerHeader *>(static_cast<const AnimationMapHeader *>(this)->GetAnimationControllerHeader(i)); }

			const AnimationControllerHeader *operator[](s32 i) const { return GetAnimationControllerHeader(i); }

			AnimationControllerHeader *SetController(u32 i) { return (_current = GetAnimationControllerHeader(i)); }
			u32 GetMaxY() const { ASSERT(_current); return GetBaseY() + _current->GetBaseY() + _current->GetMaxY(); }
			u32 GetY(f32 t) const { ASSERT(_current); return GetBaseY() + _current->GetBaseY() + _current->GetY(t); }
			u32 GetLoopY(f32 t) const { ASSERT(_current); return GetBaseY() + _current->GetBaseY() + _current->GetLoopY(t); }
			f32 GetRatio(f32 t) const { ASSERT(_current); return  _current->GetRatio(t); }
			f32 GetLoopRatio(f32 t) const { ASSERT(_current); return  _current->GetLoopRatio(t); }

		protected:
			char *_id;
			u32 _base_y;
			u32 _controller_num;
			AnimationControllerHeader **_controllers;
			AnimationControllerHeader *_current;
		};

        /*!
		\brief Aj[V}bvNX
		*/
		class AnimationMap
		{
		public:
			AnimationMap() : _header_num(0), _headers(0), _texture(0), _current(0), _texture_data_top(0) {}
			virtual ~AnimationMap() {
				if(_headers) {
					for(u32 i=0;i<_header_num;++i)
						if(_headers[i])
							delete [] _headers[i];
					delete [] _headers;
				}
				if(_texture) { delete _texture; _texture = 0; } 
			}

			virtual u32 Read(const u8 *data);

			virtual AnimationMapHeader *AnimationMapHeaderCreator() const { return new AnimationMapHeader(); }
			virtual graphics::AnimationTexture *AnimationTextureCreator() const { return new graphics::AnimationTexture(); }

			u32 GetHeaderNum() const { return _header_num; }

			const AnimationMapHeader *GetAnimationMapHeader(u32 i) const { ASSERT(i < _header_num); return _headers[i]; }
			AnimationMapHeader *GetAnimationMapHeader(u32 i) { return const_cast<AnimationMapHeader *>(static_cast<const AnimationMap *>(this)->GetAnimationMapHeader(i)); }

			const AnimationMapHeader *GetAnimationMapHeader(const char *id) const { 
				for(u32 i=0;i<_header_num;++i)
					if(!Strcmp(_headers[i]->GetID(), id)) 
						return _headers[i];
				return 0;
			}
			AnimationMapHeader *GetAnimationMapHeader(const char *id) { return const_cast<AnimationMapHeader *>(static_cast<const AnimationMap *>(this)->GetAnimationMapHeader(id)); }

			const AnimationMapHeader *operator[](s32 i) const { return GetAnimationMapHeader(i); }
			AnimationMapHeader *operator[](s32 i) { return const_cast<AnimationMapHeader *>(static_cast<const AnimationMap *>(this)->operator[](i)); }

			const AnimationMapHeader *operator[](const char *id) const { return GetAnimationMapHeader(id); }
			AnimationMapHeader *operator[](const char *id) { return const_cast<AnimationMapHeader *>(static_cast<const AnimationMap *>(this)->operator[](id)); }

			const graphics::AnimationTexture *GetAnimationTexture() const { return _texture; }
			graphics::AnimationTexture *GetAnimationTexture() { return const_cast<graphics::AnimationTexture *>(static_cast<const AnimationMap *>(this)->GetAnimationTexture()); }

			Vector2 GetTextureSize() const { return Vector2(static_cast<f32>(_texture->GetWidth()), static_cast<f32>(_texture->GetHeight())); }

			AnimationMapHeader *SetClip(u32 i) { return (_current = GetAnimationMapHeader(i)); }
			AnimationMapHeader *SetClip(const char *id) { return (_current = GetAnimationMapHeader(id)); }
			AnimationControllerHeader *SetController(u32 i) { ASSERT(_current); return _current->SetController(i); }
			u32 GetMaxY() const { ASSERT(_current); return _current->GetMaxY();	}
			u32 GetY(f32 t) const { ASSERT(_current); return _current->GetY(t);	}
			f32 GetRatio(f32 t) const { ASSERT(_current); return _current->GetRatio(t);	}
			f32 GetLoopRatio(f32 t) const { ASSERT(_current); return _current->GetLoopRatio(t);	}
			u32 GetLoopY(f32 t) const { ASSERT(_current); return _current->GetLoopY(t);	}
			u32 GetY(u32 clip, u32 ctrl, f32 t) { SetClip(clip); SetController(ctrl); return GetY(t); }
			u32 GetLoopY(u32 clip, u32 ctrl, f32 t) { SetClip(clip); SetController(ctrl); return GetLoopY(t);	}
			u32 GetY(const char *clip, u32 ctrl, f32 t) { SetClip(clip); SetController(ctrl); return GetY(t); }
			u32 GetLoopY(const char *clip, u32 ctrl, f32 t) { SetClip(clip); SetController(ctrl); return GetLoopY(t);	}

			QuaternionTransform GetTransform(u32 jnt, u32 y) const { ASSERT(_texture); return _texture->GetTransform(jnt << 1, y, _texture_data_top); }

		protected:
			u32 _header_num;
			AnimationMapHeader **_headers;
			graphics::AnimationTexture *_texture;

			AnimationMapHeader *_current;

			const u8 *_texture_data_top;
		};

        /*!
		\brief Aj[V}bvt@CNX
		*/
		class AnimationMapFile : public AnimationMap, public file::File
		{
		public:
			AnimationMapFile() : AnimationMap(), file::File() {}
			AnimationMapFile(const char *path) : AnimationMap(), file::File(path) {}
			virtual ~AnimationMapFile() {}

			virtual void OnLoaded() { ASSERT(Read(GetData()) == GetSize()); }
		};
	}
}

#endif // ANIMATION_H
