/*!
$Id: controller.h 630 2009-12-21 07:52:59Z horino $
*/
#ifndef CONTROLLER_H
#define CONTROLLER_H

#ifdef _DEBUG
#include <iostream>
using namespace std;
#endif

#include "quaternion_transform.h"
#include "matrix44.h"
#include "mesh/geometry.h"

namespace indi
{
	namespace mesh
	{
		/*!
		\brief WCgNX
		*/
		class Joint
		{
		public:
			Joint() : _id(0), _inverse(QuaternionTransform::Identity()), _transform(Matrix44::Identity()) {}
			virtual ~Joint() { if(_id) delete [] _id; }

			virtual u32 Read(const u8 *data);

			const char *GetId() const { return _id; }
			const QuaternionTransform &GetInverseTransform() const { return _inverse; }
			const Matrix44 &GetTransform() const { return _transform; }

		protected:
			char *_id;
			QuaternionTransform _inverse;
			Matrix44 _transform;
		};

		/*!
		\brief Rg[
		*/
		class Controller : public Geometry
		{
		public:
			Controller(Node *n) : Geometry(n), _shape(QuaternionTransform::Identity()), _skeleton_num(0), _joint_num(0), _skeletons(0), _joints(0), _index(0xffffffff) {}
			virtual ~Controller() {
				if(_skeletons) {
					for(u32 i=0;i<_skeleton_num;++i)
						if(_skeletons[i])
							delete [] _skeletons[i];
					delete [] _skeletons;
				}
				if(_joints) {
					for(u32 i=0;i<_joint_num;++i)
						if(_joints[i])
							delete _joints[i];
					delete [] _joints;
				}
			}

			virtual u32 Read(const u8 *data);

			BOOL IsController() const { return TRUE; }

			virtual Joint *JointCreator() const { return new Joint(); }

			QuaternionTransform &GetShape() { return _shape; }

			u32 GetSkeletonNum() const { return _skeleton_num; }
			const char *GetSkeleton(u32 i) const { ASSERT(i < _skeleton_num); return _skeletons[i]; }
			char *GetSkeleton(u32 i) { return const_cast<char *>(static_cast<const Controller *>(this)->GetSkeleton(i)); }

			u32 GetJointNum() const { return _joint_num; }
			const Joint *GetJoint(u32 i) const { ASSERT(i < _joint_num); return _joints[i]; }
			Joint *GetJoint(u32 i) { return const_cast<Joint *>(static_cast<const Controller *>(this)->GetJoint(i)); }
			u32 GetIndexOfJoint(const char *id) const {
				for(u32 i=0;i<_joint_num;++i)
					if(Strcmp(_joints[i]->GetId(), id) == 0) return i;
				return 0xffffffff;
			}

			u32 GetIndex() const { return _index; }
			void SetIndex(u32 i) { _index = i; }

		protected:
			QuaternionTransform _shape;
			u32 _skeleton_num;
			u32 _joint_num;
			char **_skeletons;
			Joint **_joints;

			u32 _index; //< V[ł̒ʂCfbNX
		};

		class ControllerAABB : public Controller, public AABB
		{
		public:
			ControllerAABB(Node *n) : Controller(n), AABB() {}
			virtual ~ControllerAABB() {}

			virtual u32 Read(const u8 *data);

			virtual Mesh *MeshCreator(Geometry *parent = 0) const { return new MeshAABB(parent); }
		};
	}
}

#ifdef _DEBUG
static ostream ATTR_USED &operator<<(ostream &lhs, const indi::mesh::Controller &rhs) {
	return lhs;
}
#endif

#endif // CONTROLLER_H