/*!
$Id: geometry.h 629 2009-12-21 04:06:49Z horino $
*/
#ifndef GEOMETRY_H
#define GEOMETRY_H

#ifdef _DEBUG
#include <iostream>
using namespace std;
#endif

#include "mesh/mesh.h"
#include "mesh/material.h"

namespace indi
{
	namespace mesh
	{
		class graphics::shader::Program;
		class Node;

		/*!
		\brief WIgNX
		\note }eAƃbV܂
		*/
		class Geometry
		{
		public:
			Geometry(Node *parent) : _parent(parent), _material_num(0), _materials(0), _mesh(0) {}
			virtual ~Geometry() {
				if(_materials) {
					for(u32 i=0;i<_material_num;++i)
						if(_materials[i])
							delete _materials[i];
					delete [] _materials;
				}
				if(_mesh)
					delete _mesh;
			}

			virtual u32 Read(const u8 *data);

			virtual BOOL IsController() const { return FALSE; }

			virtual Material *MaterialCreator() const { return new Material(); }
			virtual Mesh *MeshCreator(Geometry *parent = 0) const { return new Mesh(parent); }

			virtual void Draw() const { if(_mesh) _mesh->Draw(); }

			const Node *GetParent() const { return _parent; }

			u32 GetMaterialNum() const { return _material_num; }
			const Material *GetMaterial(u32 i) const { ASSERT(i < _material_num); return _materials[i]; }
			Material *GetMaterial(u32 i) { return const_cast<Material *>(static_cast<const Geometry *>(this)->GetMaterial(i)); }

			const Mesh *GetMesh() const { return _mesh; }
			Mesh *GetMesh() { return const_cast<Mesh *>(static_cast<const Geometry *>(this)->GetMesh()); }

			virtual void SetIndexWeight(graphics::shader::Program *prog, const char *index_name, const char *weight_name) { 
				Mesh *m(GetMesh());
				if(m)
					m->SetIndexWeight(prog, index_name, weight_name); 
			}

		protected:
			Node *_parent;
			u32 _material_num;
			Material **_materials;
			Mesh *_mesh;
		};

		class GeometryAABB : public Geometry, public AABB
		{
		public:
			GeometryAABB(Node *parent) : Geometry(parent), AABB() {}
			virtual ~GeometryAABB() {}

			virtual u32 Read(const u8 *data);

			virtual Mesh *MeshCreator(Geometry *parent = 0) const { return new MeshAABB(parent); }
		};
	}
}

#ifdef _DEBUG
static ostream ATTR_USED &operator<<(ostream &lhs, const indi::mesh::Geometry &rhs) {
	for(indi::u32 i=0;i<rhs.GetMaterialNum();++i)
		lhs << *rhs.GetMaterial(i);

	if(rhs.GetMesh())
		lhs << *rhs.GetMesh();

	return lhs;
}
#endif

#endif // GEOMETRY_H