/*!
$Id: material.h 418 2009-10-15 10:10:51Z horino $
*/
#ifndef MATERIAL_H
#define MATERIAL_H

#ifdef _DEBUG
#include <iostream>
using namespace std;
#endif

#include "vector4.h"
#include "mesh_define.h"
#include "colors.h"

namespace indi
{
	namespace mesh
	{
		/*!
		\brief J[܂̓eNX`NX
		*/
		class ColorOrTexture
		{
		public:
			ColorOrTexture() : _texture_flag(0), _texture_index(0), _color(color::White) {}
			virtual ~ColorOrTexture() {}

			virtual u32 Read(const u8 *data);

			BOOL IsTexture() const { return (_texture_flag & TextureValid); }

			u32 GetTextureIndex() const { return _texture_index; }
			const Vector4 &GetColor() const { return _color; }

		protected:
			u32 _texture_flag;
			u32 _texture_index;
			Vector4 _color;
		};

		/*!
		\brief VF[_x[XNX(ۃNX)
		*/
		class Shader
		{
		public:
			Shader() {}
			virtual ~Shader() {}

			virtual u32 Read(const u8 *data) = 0;
			virtual ShaderType GetShaderType() const = 0;
		};
		/*!
		\brief RX^gVF[_NX
		*/
		class Constant : public Shader
		{
		public:
			Constant() : Shader(), _emission(), _reflective(), _transparent(), _reflectivity(0), _transparency(0), _refraction(0) {}
			virtual ~Constant() {}

			virtual u32 Read(const u8 *data);
			virtual ShaderType GetShaderType() const { return ConstantShader; };

			const ColorOrTexture &GetEmission() const { return _emission; }
			const ColorOrTexture &GetReflective() const { return _reflective; }
			const ColorOrTexture &GetTransparent() const { return _transparent; }
			f32 GetReflectivity() const { return _reflectivity; }
			f32 GetTransparency() const { return _transparency; }
			f32 GetRefraction() const { return _refraction; }

		protected:
			ColorOrTexture _emission;
			ColorOrTexture _reflective;
			ColorOrTexture _transparent;
			f32 _reflectivity;
			f32 _transparency;
			f32 _refraction;
		};
		/*!
		\brief o[gVF[_NX
		*/
		class Lambert : public Constant
		{
		public:
			Lambert() : Constant(), _ambient(), _diffuse() {}
			virtual ~Lambert() {}

			virtual u32 Read(const u8 *data);
			virtual ShaderType GetShaderType() const { return LambertShader; };

			const ColorOrTexture &GetAmbient() const { return _ambient; }
			const ColorOrTexture &GetDiffuse() const { return _diffuse; }

		protected:
			ColorOrTexture _ambient;
			ColorOrTexture _diffuse;
		};
		/*!
		\brief tHVF[_NX
		*/
		class Phong : public Lambert
		{
		public:
			Phong() : Lambert(), _specular(), _shininess(0) {}
			virtual ~Phong() {}

			virtual u32 Read(const u8 *data);
			virtual ShaderType GetShaderType() const { return PhongShader; };

			const ColorOrTexture &GetSpecular() const { return _specular; }
			f32 GetShininess() const { return _shininess; }

		protected:
			ColorOrTexture _specular;
			f32 _shininess;
		};
		/*!
		\brief uVF[_NX
		*/
		class Blinn : public Phong
		{
		public:
			Blinn() : Phong() {}
			virtual ~Blinn() {}

			virtual u32 Read(const u8 *data) { return Phong::Read(data); }
			virtual ShaderType GetShaderType() const { return BlinnShader; };
		};

		/*!
		\brief vt@Cx[XNX(ۃNX)
		*/
		class Profile
		{
		public:
			Profile() {}
			virtual ~Profile() {}

			virtual u32 Read(const u8 *data) = 0;
			virtual ProfileType GetProfileType() const = 0;
		};
		/*!
		\brief vt@CCommonNX
		*/
		class Common : public Profile
		{
		public:
			Common() : Profile(), _shader(0) {}
			virtual ~Common() { if(_shader) delete _shader; }

			virtual u32 Read(const u8 *data);
			virtual ProfileType GetProfileType() const { return ProfileCommon; }

			virtual Constant *ConstantCreator() const { return new Constant(); }
			virtual Lambert *LambertCreator() const { return new Lambert(); }
			virtual Phong *PhongCreator() const { return new Phong(); }
			virtual Blinn *BlinnCreator() const { return new Blinn(); }

			const Shader *GetShader() const { return _shader; }

		protected:
			Shader *_shader;
		};

		/*!
		\brief }eANX
		*/
		class Material
		{
		public:
			Material() : _profile(0) {}
			virtual ~Material() { if(_profile) delete _profile; }

			virtual u32 Read(const u8 *data);

			virtual Common *CommonCreator() const { return new Common(); }

			const Profile *GetProfile() const { return _profile; }

		protected:
			Profile *_profile;
		};
	}
}

#ifdef _DEBUG
static ostream ATTR_USED &operator<<(ostream &lhs, const indi::mesh::ColorOrTexture &rhs) { 
	if(rhs.IsTexture()) lhs << rhs.GetTextureIndex();
	else                lhs << rhs.GetColor();
	return lhs;
}
static ostream ATTR_USED &operator<<(ostream &lhs, const indi::mesh::Constant &rhs) {
	lhs << "\t\tEmission : " << rhs.GetEmission() << endl;
	lhs << "\t\tReflective : " << rhs.GetReflective() << endl;
	lhs << "\t\tTransparent : " << rhs.GetTransparent() << endl;
	lhs << "\t\tReflectivity : " << rhs.GetReflectivity() << endl;
	lhs << "\t\tTransparency : " << rhs.GetTransparency() << endl;
	lhs << "\t\tRefraction : " << rhs.GetRefraction() << endl;
	return lhs;
}
static ostream ATTR_USED &operator<<(ostream &lhs, const indi::mesh::Lambert &rhs) {
	lhs << "\t\tEmission : " << rhs.GetEmission() << endl;
	lhs << "\t\tAmbient : " << rhs.GetAmbient() << endl;
	lhs << "\t\tDiffuse : " << rhs.GetDiffuse() << endl;
	lhs << "\t\tReflective : " << rhs.GetReflective() << endl;
	lhs << "\t\tTransparent : " << rhs.GetTransparent() << endl;
	lhs << "\t\tReflectivity : " << rhs.GetReflectivity() << endl;
	lhs << "\t\tTransparency : " << rhs.GetTransparency() << endl;
	lhs << "\t\tRefraction : " << rhs.GetRefraction() << endl;
	return lhs;
}
static ostream ATTR_USED &operator<<(ostream &lhs, const indi::mesh::Phong &rhs) {
	lhs << "\t\tEmission : " << rhs.GetEmission() << endl;
	lhs << "\t\tAmbient : " << rhs.GetAmbient() << endl;
	lhs << "\t\tDiffuse : " << rhs.GetDiffuse() << endl;
	lhs << "\t\tSpecular : " << rhs.GetSpecular() << endl;
	lhs << "\t\tReflective : " << rhs.GetReflective() << endl;
	lhs << "\t\tTransparent : " << rhs.GetTransparent() << endl;
	lhs << "\t\tReflectivity : " << rhs.GetReflectivity() << endl;
	lhs << "\t\tTransparency : " << rhs.GetTransparency() << endl;
	lhs << "\t\tRefraction : " << rhs.GetRefraction() << endl;
	lhs << "\t\tShininess : " << rhs.GetShininess() << endl;
	return lhs;
}
static ostream ATTR_USED &operator<<(ostream &lhs, const indi::mesh::Blinn &rhs) { return lhs << static_cast<const indi::mesh::Phong &>(rhs); }
static ostream ATTR_USED &operator<<(ostream &lhs, const indi::mesh::Common &rhs) {
	const indi::mesh::Shader *s(rhs.GetShader());
	if(s){
		switch(s->GetShaderType()){
			default: break;
			case indi::mesh::ConstantShader:
				lhs << "\tConstant" << endl << *static_cast<const indi::mesh::Constant *>(s); 
				break;
			case indi::mesh::LambertShader:
				lhs << "\tLambert" << endl << *static_cast<const indi::mesh::Lambert *>(s);  
				break;
			case indi::mesh::PhongShader:
				lhs << "\tPhong" << endl << *static_cast<const indi::mesh::Phong *>(s);    
				break;
			case indi::mesh::BlinnShader:
				lhs << "\tBlinn" << endl << *static_cast<const indi::mesh::Blinn *>(s);   
				break;
		}
	} 
	return lhs;
}
static ostream ATTR_USED &operator<<(ostream &lhs, const indi::mesh::Material &rhs) {
	const indi::mesh::Profile *p(rhs.GetProfile());
	if(p){
		switch(p->GetProfileType()){
            default:
				break;
			case indi::mesh::ProfileCommon:
				lhs << "Profile Common " << endl << *static_cast<const indi::mesh::Common *>(p);
				break;
		}
	} 
	return lhs;
}
#endif

#endif // MATERIAL_H