/*!
$Id: mesh.h 629 2009-12-21 04:06:49Z horino $
*/
#ifndef MESH_H
#define MESH_H

#ifdef _DEBUG
#include <iostream>
using namespace std;
#endif

#include "itl/algorithm.h"
#include "itl/functional.h"

#include "mesh/triangle.h"

namespace indi
{
	namespace mesh
	{
		class graphics::shader::Program;
		class Geometry;

		/*!
		\brief bVNX
		\note gCAO܂
		*/
		class Mesh
		{
		public:
			Mesh(Geometry *parent) : _parent(parent), _name(0), _triangle_num(0), _triangles(0) {}
			virtual ~Mesh() {
				if(_name) 
					delete [] _name;

				if(_triangles) {
					for(u32 i=0;i<_triangle_num;++i)
						if(_triangles[i])
							delete _triangles[i];
					delete [] _triangles;
				}
			}

			virtual u32 Read(const u8 *data);

			virtual Triangle *TriangleCreator() const { return new Triangle(); }

			virtual void Draw() const { itl::For_each(&_triangles[0], &_triangles[_triangle_num], Mem_fun(&Triangle::Draw)); }

			Geometry *GetParent() { return _parent; }

			const char *GetName() const { return _name; }

			u32 GetTriangleNum() const { return _triangle_num; }
			const Triangle *GetTriangle(u32 i) const { ASSERT(i < _triangle_num); return _triangles[i]; }
			Triangle *GetTriangle(u32 i) { return const_cast<Triangle *>(static_cast<const Mesh *>(this)->GetTriangle(i)); }

			virtual void SetIndexWeight(graphics::shader::Program *prog, const char *index_name, const char *weight_name) { 
				for(u32 i=0;i<GetTriangleNum();++i) {
					Triangle *t(GetTriangle(i));
					if(t)
						t->SetIndexWeight(prog, index_name, weight_name); 
				}
			}

		protected:
			Geometry *_parent;
			char *_name;
			u32 _triangle_num;
			Triangle **_triangles;
		};

		class MeshAABB : public Mesh, public AABB
		{
		public:
			MeshAABB(Geometry *parent) : Mesh(parent), AABB() {}
			virtual ~MeshAABB() {}

			virtual u32 Read(const u8 *data);

			virtual Triangle *TriangleCreator() const { return new TriangleAABB(); }
		};
	}
}

#ifdef _DEBUG
static ostream ATTR_USED &operator<<(ostream &lhs, const indi::mesh::Mesh &rhs) {
	lhs << "Mesh \"" << rhs.GetName() << "\"" << endl;
	if(rhs.GetTriangleNum()) lhs << "\t" << rhs.GetTriangleNum() << " triangles" << endl;
	return lhs;
}
#endif

#endif // MESH_H