/*!
$Id: node.h 629 2009-12-21 04:06:49Z horino $
*/
#ifndef NODE_H
#define NODE_H

#ifdef _DEBUG
#include <iostream>
using namespace std;
#endif

#include "mesh/aabb.h"
#include "mesh/controller.h"

namespace indi
{
	namespace mesh
	{
		class graphics::shader::Program;

		/*!
		\brief m[hNX
		\note WIgARg[܂
		*/
		class Node
		{
		public:
			Node(Node *parent) : _parent(parent), _name(0), _transform(Matrix44::Identity()), _geometry_num(0), _controller_num(0), _node_num(0), _geometries(0), _controllers(0), _nodes(0) {}
			virtual ~Node() {
				if(_name)
					delete [] _name;
				if(_geometries) {
					for(u32 i=0;i<_geometry_num;++i)
						if(_geometries[i])
							delete _geometries[i];
					delete [] _geometries;
				}
				if(_controllers) {
					for(u32 i=0;i<_controller_num;++i)
						if(_controllers[i])
							delete _controllers[i];
					delete [] _controllers;
				}
				if(_nodes) {
					for(u32 i=0;i<_node_num;++i)
						if(_nodes[i])
							delete _nodes[i];
					delete [] _nodes;
				}
			}

			virtual u32 Read(const u8 *data);

			virtual Geometry *GeometryCreator(Node *parent = 0) const { return new Geometry(parent); }
			virtual Controller *ControllerCreator(Node *parent = 0) const { return new Controller(parent); }
			virtual Node *NodeCreator(Node *parent = 0) const { return new Node(parent); }

			virtual void Draw() const {
				itl::For_each(&_geometries[0], &_geometries[_geometry_num], Mem_fun(&Geometry::Draw)); 
				itl::For_each(&_controllers[0], &_controllers[_controller_num], Mem_fun(&Controller::Draw)); 
				itl::For_each(&_nodes[0], &_nodes[_node_num], Mem_fun(&Node::Draw)); 
			}

			Node *GetParent() { return _parent; }
			const char *GetName() const { return _name; }
			const Matrix44 &GetTransform() const { return _transform; }

			u32 GetGeometryNum() const { return _geometry_num; }
			const Geometry *GetGeometry(u32 i) const { ASSERT(i < _geometry_num); return _geometries[i]; }
			Geometry *GetGeometry(u32 i) { return const_cast<Geometry *>(static_cast<const Node *>(this)->GetGeometry(i)); }

			u32 GetControllerNum() const { return _controller_num; }
			const Controller *GetController(u32 i) const { ASSERT(i < _controller_num); return _controllers[i]; }
			Controller *GetController(u32 i) { return const_cast<Controller *>(static_cast<const Node *>(this)->GetController(i)); }

			u32 GetNodeNum() const { return _node_num; }
			const Node *GetNode(u32 i) const { ASSERT(i < _node_num); return _nodes[i]; }
			Node *GetNode(u32 i) { return const_cast<Node *>(static_cast<const Node *>(this)->GetNode(i)); }

			/*!
			\brief ẽgXtH[ċAIɓKp܂
			*/
			void AccumulateTransform() {
				if(_parent)
					_transform *= _parent->_transform;
				itl::For_each(&_nodes[0], &_nodes[_node_num], Mem_fun(&Node::AccumulateTransform)); 
			} 

			virtual void SetIndexWeight(graphics::shader::Program *prog, const char *index_name, const char *weight_name) { 
				for(u32 i=0;i<GetGeometryNum();++i){
					Geometry *g(GetGeometry(i));
					if(g)
						g->SetIndexWeight(prog, index_name, weight_name); 
				}
				for(u32 i=0;i<GetControllerNum();++i){
					Controller *c(GetController(i));
					if(c)
						c->SetIndexWeight(prog, index_name, weight_name); 
				}
				for(u32 i=0;i<GetNodeNum();++i){
					Node *n(GetNode(i));
					if(n)
						n->SetIndexWeight(prog, index_name, weight_name); 
				}
			}

			/*!
			\brief Rg[̒ʂCfbNX쐬܂
			\param ݂̃CfbNXiċAIɒׂ邽߁j
			\return ŐṼCfbNXԂ܂
			*/
			u32 CreateControllerIndices(u32 idx) {
				u32 new_idx(idx);
				for(u32 i=0;i<GetControllerNum();++i){
					Controller *c(GetController(i));
					if(c)
						c->SetIndex(new_idx++);
				}
				for(u32 i=0;i<GetNodeNum();++i){
					Node *n(GetNode(i));
					if(n)
						new_idx = n->CreateControllerIndices(new_idx);
				}
				return new_idx;
			}

		protected:
			Node *_parent;
			char *_name;

			Matrix44 _transform; //< m[h̓XP[Ă邱Ƃ悭̂ŁiNH[^jIgXtH[łȂj}gNXŎ

			u32 _geometry_num;
			u32 _controller_num;
			u32 _node_num;
			Geometry **_geometries;
			Controller **_controllers;
			Node **_nodes;
		};

		class NodeAABB : public Node, public AABB
		{
		public:
			NodeAABB(Node *parent) : Node(parent), AABB() {}
			virtual ~NodeAABB() {}

			virtual u32 Read(const u8 *data);

			virtual Geometry *GeometryCreator(Node *parent = 0) const { return new GeometryAABB(parent); }
			virtual Controller *ControllerCreator(Node *parent = 0) const { return new ControllerAABB(parent); }
		};
	}
}

#ifdef _DEBUG
static ostream ATTR_USED &operator<<(ostream &lhs, const indi::mesh::Node &rhs) {
	lhs << "Node \"" << rhs.GetName() << "\"" << endl;
	//lhs << "\tTransform : " << endl << rhs.GetTransform() << endl;

	for(indi::u32 i=0;i<rhs.GetGeometryNum();++i)
		lhs << *rhs.GetGeometry(i);

	for(indi::u32 i=0;i<rhs.GetControllerNum();++i)
		lhs << *rhs.GetController(i);

	for(indi::u32 i=0;i<rhs.GetNodeNum();++i)
		lhs << *rhs.GetNode(i);

	return lhs;
}
#endif

#endif // NODE_H