/*!
$Id: scene.h 629 2009-12-21 04:06:49Z horino $
*/
#ifndef SCENE_H
#define SCENE_H

#ifdef _DEBUG
#include <iostream>
using namespace std;
#endif

#include "mesh/aabb.h"
#include "mesh/node.h"

namespace indi
{
	namespace mesh
	{
		class graphics::shader::Program;

		/*!
		\brief V[NX
		*/
		class Scene
		{
		public:
			Scene() : _node_num(0), _nodes(0) {}
			virtual ~Scene() {
				if(_nodes) {
					for(u32 i=0;i<_node_num;++i)
						if(_nodes[i])
							delete _nodes[i];
					delete [] _nodes;
				}
			}

			virtual u32 Read(const u8 *data);

			virtual Node *NodeCreator(Node *parent = 0) const { return new Node(parent); }

			virtual void Draw() const { itl::For_each(&_nodes[0], &_nodes[_node_num], Mem_fun(&Node::Draw)); 	}

			u32 GetNodeNum() const { return _node_num; }
			const Node *GetNode(u32 i) const { ASSERT(i < _node_num); return _nodes[i]; }
			Node *GetNode(u32 i) { return const_cast<Node *>(static_cast<const Scene *>(this)->GetNode(i)); }

			virtual void SetIndexWeight(graphics::shader::Program *prog, const char *index_name, const char *weight_name) { 
				for(u32 i=0;i<GetNodeNum();++i){
					Node *n(GetNode(i));
					if(n)
						n->SetIndexWeight(prog, index_name, weight_name); 
				}
			}

			/*!
			\brief Rg[̒ʂCfbNX쐬܂
			\param ݂̃CfbNXiċAIɒׂ邽߁j
			\return ŐṼCfbNXԂ܂
			*/
			u32 CreateControllerIndices(u32 idx) {
				u32 new_idx(idx);
				for(u32 i=0;i<GetNodeNum();++i){
					Node *n(GetNode(i));
					if(n)
						new_idx = n->CreateControllerIndices(new_idx);
				}
				return new_idx;
			}

		protected:
			u32 _node_num;
			Node **_nodes;
		};

		class SceneAABB : public Scene, public AABB
		{
		public:
			SceneAABB() : Scene(), AABB() {}
			virtual ~SceneAABB() {}

			virtual u32 Read(const u8 *data);

			virtual Node *NodeCreator(Node *parent = 0) const { return new NodeAABB(parent); }
		};
	}
}

#ifdef _DEBUG
static ostream ATTR_USED &operator<<(ostream &lhs, const indi::mesh::Scene &rhs) {
	for(indi::u32 i=0;i<rhs.GetNodeNum();++i)
		lhs << *rhs.GetNode(i);
	return lhs;
}
#endif

#endif // SCENE_H