/*!
$Id: scenes.h 629 2009-12-21 04:06:49Z horino $
*/
#ifndef SCENES_H
#define SCENES_H

#ifdef _DEBUG
#include <iostream>
using namespace std;
#endif

#include "mesh/scene.h"

#include "file/file.h"

namespace indi
{
	namespace mesh
	{
		class graphics::shader::Program;

		/*!
		\brief V[ǗNX
		*/
		class Scenes
		{
		public:
			Scenes() : _scene_num(0), _scenes(0) {}
			virtual ~Scenes() {
				if(_scenes) {
					for(u32 i=0;i<_scene_num;++i)
						if(_scenes[i])
							delete _scenes[i];
					delete [] _scenes;
				}
			}

			virtual u32 Read(const u8 *data);

			virtual Scene *SceneCreator() const { return new Scene(); }
		
			virtual void Draw() const { itl::For_each(&_scenes[0], &_scenes[_scene_num], Mem_fun(&mesh::Scene::Draw)); }

			u32 GetSceneNum() const { return _scene_num; }
			const Scene *GetScene(u32 i) const { ASSERT(i < _scene_num); return _scenes[i]; }

			Scene *GetScene(u32 i) { return const_cast<Scene *>(static_cast<const Scenes *>(this)->GetScene(i)); }

			virtual void SetIndexWeight(graphics::shader::Program *prog, const char *index_name = "In.Indices", const char *weight_name = "In.Weights")	{
				for(u32 i=0;i<GetSceneNum();++i){
					Scene *s(GetScene(i));
					if(s)
						s->SetIndexWeight(prog, index_name, weight_name); 
				}
			}

		protected:
			u32 _scene_num;
			mesh::Scene **_scenes;
		};

		class ScenesAABB : public Scenes, public AABB
		{
		public:
			ScenesAABB() : Scenes(), AABB() {}
			virtual ~ScenesAABB() {}

			virtual u32 Read(const u8 *data);

			virtual Scene *SceneCreator() const { return new SceneAABB(); }
		};

		/*!
		\brief V[ǗNX(񓯊ǂݍݔ)
		*/
		class ScenesFile : public Scenes, public file::File
		{
		public:
			ScenesFile() : Scenes(), file::File() {}
			ScenesFile(const char *path) : Scenes(), file::File(path) {}
			virtual ~ScenesFile() {}

			virtual void OnLoaded() { ASSERT(Read(GetData()) == GetSize()); }

			virtual void Draw() const { if(IsLoaded()) Scenes::Draw(); }
		};
		class ScenesAABBFile : public ScenesAABB, public file::File
		{
		public:
			ScenesAABBFile() : ScenesAABB(), file::File() {}
			ScenesAABBFile(const char *path) : ScenesAABB(), file::File(path) {}
			virtual ~ScenesAABBFile() {}

			virtual void OnLoaded() { ASSERT(Read(GetData()) == GetSize()); }

			virtual void Draw() const { if(IsLoaded()) Scenes::Draw(); }
		};
	}
}

#endif // SCENES_H