/*!
$Id: particle.h 642 2009-12-28 05:16:12Z horino $
*/
#ifndef PARTICLE_H
#define PARTICLE_H

#include "physics/entity.h"
#include "vector3.h"

namespace indi
{
	namespace physics
	{
		/*!
		\brief p[eBNNX
		*/
		class Particle : public Entity
		{
		public:
			friend class Spring;

			Particle() : Entity(), _mass(1.0f), _inv_mass(1.0f / _mass), _drag_coefficent(0.005f), _force(Vector3::Zero()), _acceleration(Vector3::Zero()), _velocity(Vector3::Zero()), _position(Vector3::Zero()), _prev_position(_position) {}
			Particle(f32 mass, const Vector3 &pos = Vector3::Zero(), f32 drag = 0.005f) : Entity(), _mass(mass), _inv_mass(1.0f / _mass), _drag_coefficent(drag), _force(Vector3::Zero()), _acceleration(Vector3::Zero()), _velocity(Vector3::Zero()), _position(pos), _prev_position(_position) {}
			virtual ~Particle() {}

			virtual BOOL IsParticle() const { return TRUE; }
			virtual BOOL IsRigidbody() const { return FALSE; }
			virtual BOOL IsSpring() const { return FALSE; }

			virtual void Update(f32 dt);
			virtual void Integrate(f32 dt);

			f32 GetMass() const { return _mass; }
			void SetMass(f32 m) { _mass = m; _inv_mass = 1.0f / _mass; }
			f32 GetInvMass() const { return _inv_mass; }

			f32 GetDragCoefficent() const { return _drag_coefficent; }
			void SetDragCoefficent(f32 d) { _drag_coefficent = d; }

			const Vector3 &GetForce() const { return _force; }
			void SetForce(const Vector3 &f) { _force = f; }

			const Vector3 &GetAcceleration() const { return _acceleration; }
			void SetAcceleration(const Vector3 &a) { _acceleration = a; }

			const Vector3 &GetVelocity() const { return _velocity; }
			void SetVelocity(const Vector3 &f) { _velocity = f; }

			const Vector3 &GetPosition() const { return _position; }
			void SetPosition(const Vector3 &p) { _position = p; _prev_position = _position; }

		protected:
			f32 _mass;
			f32 _inv_mass;
			f32 _drag_coefficent;
			Vector3 _force;
			Vector3 _acceleration;			
			Vector3 _velocity;
			Vector3 _position;
			Vector3 _prev_position;
		};
	}
}

#endif //< PARTICLE_H