/*!
$Id: spring.h 642 2009-12-28 05:16:12Z horino $
*/
#ifndef SPRING_H
#define SPRING_H

#include "physics/entity.h"
#include "physics/particle.h"

namespace indi
{
	namespace physics
	{
		/*!
		\brief XvONX
		*/
		class Spring : public Entity
		{
		public:
			Spring() : Entity(), _stiffness(8.0f), _dumping(0.1f), _a(0), _b(0), _length(1.0f) {}
			Spring(f32 s, f32 d, Particle *a = 0, Particle *b = 0) : Entity(), _stiffness(s), _dumping(d), _a(a), _b(b), _length((_a->GetPosition() - _b->GetPosition()).Length()) {}
			Spring(Particle *a, Particle *b) : Entity(), _stiffness(2.0f), _dumping(0.02f), _a(a), _b(b), _length((_a->GetPosition() - _b->GetPosition()).Length()) {}
			virtual ~Spring() {}

			virtual BOOL IsParticle() const { return FALSE; }
			virtual BOOL IsRigidbody() const { return FALSE; }
			virtual BOOL IsSpring() const { return TRUE; }

			virtual void ApplyForce();
			virtual void Constraint();

			f32 GetStiffness() const { return _stiffness; }
			void SetStiffness(f32 s) { _stiffness = s; }

			f32 GetDumping() const { return _dumping; }
			void SetDumping(f32 d) { _dumping = d; }

			f32 GetLength() const { return _length; }
			void SetLength(f32 l) { _length = l; }

			Particle *GetA() const { return _a; }
			void SetA(Particle *e) { _a = e; }

			Particle *GetB() const { return _b; }
			void SetB(Particle *e) { _b = e; }

		protected:
			f32 _stiffness;
			f32 _dumping;
			Particle *_a;
			Particle *_b;
			f32 _length;
		};
	}
}

#endif //< SPRING_H