/*!
$Id: world.h 642 2009-12-28 05:16:12Z horino $
*/
#ifndef WORLD_H
#define WORLD_H

#include "vector3.h"

namespace indi
{
	namespace physics
	{
		class Entity;
		class Particle;
		class Rigidbody;
		class Spring;
		class Cloth;

		class World 
		{
		public:
			World(u32 np = 128, u32 nr = 128, u32 ns = 128, u32 ci = 5) : _gravity(Vector3(0.0f, -9.8f, 0.0f)), _nparticle(np), _particles(new Particle * [_nparticle]), _nrigidbody(nr), _rigidbodies(new Rigidbody * [_nrigidbody]), _nspring(ns), _springs(new Spring * [_nspring]), constrain_iterate(ci) {
				Memzero(_particles, sizeof(_particles[0]) * _nparticle);
				Memzero(_rigidbodies, sizeof(_rigidbodies[0]) * _nrigidbody);
				Memzero(_springs, sizeof(_springs[0]) * _nspring);
			}
			virtual ~World();

			virtual void Update(f32 dt);

			void Add(Entity *e);
			void Add(Cloth *c);

			const Vector3 &GetGravity() const { return _gravity; }
			void SetGravity(const Vector3 &g) { _gravity = g; }

			static World *Create() { if(!_world) _world = new World(); return _world; }
			static World *Get() { return _world; }
			static void Delete() { if(_world) { delete _world; _world = 0; } }

		protected:
			static World *_world;
			Vector3 _gravity;
			u32 _nparticle;
			Particle **_particles;
			u32 _nrigidbody;
			Rigidbody **_rigidbodies;
			u32 _nspring;
			Spring **_springs;

			u32 constrain_iterate;
		};
	}
}

#endif //< WORLD_H