/*!
$Id: primitive.h 645 2010-01-05 09:30:49Z horino $
*/
#ifndef PRIMITIVE_H
#define PRIMITIVE_H

// #ifdef _DEBUG ɂׂH

#include "indi.h"
#include "graphics/vertexbuffer.h"

namespace indi
{
	namespace primitive
	{
		/*!
		\brief 
		*/
		class SegmentBase
		{
		public:
			SegmentBase(){}
			SegmentBase(graphics::VertexBuffer<graphics::VertexPosition> *v) : vertex(v) {}
			virtual ~SegmentBase() { if(vertex) { delete vertex; vertex = 0; } }
			operator graphics::VertexBuffer<graphics::VertexPosition>&() { return *vertex; }
			void Begin() { if(vertex) vertex->Begin(); }
			void Draw() { if(vertex) vertex->Draw(); }
			void End() { if(vertex) vertex->End(); }
		protected:
			graphics::VertexBuffer<graphics::VertexPosition> *vertex;
		};
		class SegmentX : public SegmentBase {
		public:
			SegmentX();
		};
		class SegmentY : public SegmentBase {
		public:
			SegmentY();
		};
		class SegmentZ : public SegmentBase {
		public:
			SegmentZ();
		};

		/*!
		\brief 
		*/
		class ArrowBase
		{
		public:
			ArrowBase(){}
			virtual ~ArrowBase() { if(vertex) { delete vertex; vertex = 0; } }
			operator graphics::VertexBuffer<graphics::VertexPosition>&() { return *vertex; }
			void Begin() { if(vertex) vertex->Begin(); }
			void Draw() { if(vertex) vertex->Draw(); }
			void End() { if(vertex) vertex->End(); }
		protected:
			graphics::VertexBuffer<graphics::VertexPosition> *vertex;
		};
		class ArrowX : public ArrowBase {
		public:
			ArrowX();
		};
		class ArrowY : public ArrowBase {
		public:
			ArrowY();
		};
		class ArrowZ : public ArrowBase {
		public:
			ArrowZ();
		};
		class ArrowXYZ  : public ArrowBase {
		public:
			ArrowXYZ();
		};

		/*!
		\brief 
		*/
		class PlaneBase
		{
		public:
			PlaneBase(){}
			virtual ~PlaneBase() { if(vertex) { delete vertex; vertex = 0; } }
			operator graphics::VertexBuffer<graphics::VertexPositionNormal>&() { return *vertex; }
			void Begin() { if(vertex) vertex->Begin(); }
			void Draw() { if(vertex) vertex->Draw(); }
			void End() { if(vertex) vertex->End(); }
		protected:
			graphics::VertexBuffer<graphics::VertexPositionNormal> *vertex;
		};
		class PlaneXY  : public PlaneBase {
		public:
			PlaneXY();
		};
		class PlaneYZ  : public PlaneBase {
		public:
			PlaneYZ();
		};
		class PlaneZX  : public PlaneBase {
		public:
			PlaneZX();
		};
		class WirePlaneBase
		{
		public:
			WirePlaneBase(){}
			virtual ~WirePlaneBase() { if(vertex) { delete vertex; vertex = 0; } }
			operator graphics::VertexBuffer<graphics::VertexPosition>&() { return *vertex; }
			void Begin() { if(vertex) vertex->Begin(); }
			void Draw() { if(vertex) vertex->Draw(); }
			void End() { if(vertex) vertex->End(); }
		protected:
			graphics::VertexBuffer<graphics::VertexPosition> *vertex;
		};
		class WirePlaneXY  : public WirePlaneBase {
		public:
			WirePlaneXY();
		};
		class WirePlaneYZ  : public WirePlaneBase {
		public:
			WirePlaneYZ();
		};
		class WirePlaneZX  : public WirePlaneBase {
		public:
			WirePlaneZX();
		};

		/*!
		\brief tXN[`p̕
		*/
		class ScreenPlane
		{
		public:
			ScreenPlane();
			~ScreenPlane(){ if(vertex) { delete vertex; vertex = 0; } }
			operator graphics::VertexBuffer<graphics::VertexPositionTexture>&() { return *vertex; }
			void Begin() const { if(vertex) vertex->Begin(); }
			void Draw() const { if(vertex) vertex->Draw(); }
			void End() const { if(vertex) vertex->End(); }
		protected:
			graphics::VertexBuffer<graphics::VertexPositionTexture> *vertex;
		};

		/*!
		\brief r{[h`p̕
		*/
		class BillboardPlane
		{
		public:
			BillboardPlane();
			~BillboardPlane(){ if(vertex) { delete vertex; vertex = 0; } }
			operator graphics::VertexBuffer<graphics::VertexPositionTexture>&() { return *vertex; }
			void Begin() const { if(vertex) vertex->Begin(); }
			void Draw() const { if(vertex) vertex->Draw(); }
			void End() const { if(vertex) vertex->End(); }
		protected:
			graphics::VertexBuffer<graphics::VertexPositionTexture> *vertex;
		};

		/*!
		\brief 
		*/
		class Sphere
		{
		public:
			Sphere();
			~Sphere(){ if(vertex) { delete vertex; vertex = 0; } }
			operator graphics::VertexBuffer<graphics::VertexPositionNormal>&() { return *vertex; }
			void Begin() { if(vertex) vertex->Begin(); }
			void Draw() { if(vertex) vertex->Draw(); }
			void End() { if(vertex) vertex->End(); }
		protected:
			graphics::VertexBuffer<graphics::VertexPositionNormal> *vertex;
		};
		class WireSphere
		{
		public:
			WireSphere();
			~WireSphere(){ if(vertex) { delete vertex; vertex = 0; } }
			operator graphics::VertexBuffer<graphics::VertexPosition>&() { return *vertex; }
			void Begin() { if(vertex) vertex->Begin(); }
			void Draw() { if(vertex) vertex->Draw(); }
			void End() { if(vertex) vertex->End(); }
		protected:
			graphics::VertexBuffer<graphics::VertexPosition> *vertex;
		};

		/*!
		\brief JvZ
		*/
		class Capsule
		{
		public:
			Capsule();
			~Capsule(){ if(vertex) { delete vertex; vertex = 0; } }
			operator graphics::VertexBuffer<graphics::VertexPositionNormal>&() { return *vertex; }
			void Begin() { if(vertex) vertex->Begin(); }
			void Draw() { if(vertex) vertex->Draw(); }
			void End() { if(vertex) vertex->End(); }
		protected:
			graphics::VertexBuffer<graphics::VertexPositionNormal> *vertex;
		};
		class WireCapsule
		{
		public:
			WireCapsule();
			~WireCapsule(){ if(vertex) { delete vertex; vertex = 0; } }
			operator graphics::VertexBuffer<graphics::VertexPosition>&() { return *vertex; }
			void Begin() { if(vertex) vertex->Begin(); }
			void Draw() { if(vertex) vertex->Draw(); }
			void End() { if(vertex) vertex->End(); }
		protected:
			graphics::VertexBuffer<graphics::VertexPosition> *vertex;
		};

		/*!
		\brief V_
		*/
		class Cylinder
		{
		public:
			Cylinder();
			~Cylinder(){ if(vertex) { delete vertex; vertex = 0; } }
			operator graphics::VertexBuffer<graphics::VertexPositionNormal>&() { return *vertex; }
			void Begin() { if(vertex) vertex->Begin(); }
			void Draw() { if(vertex) vertex->Draw(); }
			void End() { if(vertex) vertex->End(); }
		protected:
			graphics::VertexBuffer<graphics::VertexPositionNormal> *vertex;
		};
		class WireCylinder
		{
		public:
			WireCylinder();
			~WireCylinder(){ if(vertex) { delete vertex; vertex = 0; } }
			operator graphics::VertexBuffer<graphics::VertexPosition>&() { return *vertex; }
			void Begin() { if(vertex) vertex->Begin(); }
			void Draw() { if(vertex) vertex->Draw(); }
			void End() { if(vertex) vertex->End(); }
		protected:
			graphics::VertexBuffer<graphics::VertexPosition> *vertex;
		};

		/*!
		\brief {bNX
		*/
		class Box
		{
		public:
			Box();
			~Box(){ if(vertex) { delete vertex; vertex = 0; } }
			operator graphics::VertexBuffer<graphics::VertexPositionNormal>&() { return *vertex; }
			void Begin() { if(vertex) vertex->Begin(); }
			void Draw() { if(vertex) vertex->Draw(); }
			void End() { if(vertex) vertex->End(); }
		protected:
			graphics::VertexBuffer<graphics::VertexPositionNormal> *vertex;
		};
		class WireBox
		{
		public:
			WireBox();
			~WireBox(){ if(vertex) { delete vertex; vertex = 0; } }
			operator graphics::VertexBuffer<graphics::VertexPosition>&() { return *vertex; }
			void Begin() { if(vertex) vertex->Begin(); }
			void Draw() { if(vertex) vertex->Draw(); }
			void End() { if(vertex) vertex->End(); }
		protected:
			graphics::VertexBuffer<graphics::VertexPosition> *vertex;
		};

		/*!
		\brief Ci[{bNX(XJC{bNXȂǂŎgpj
		*/
		class InnerBox
		{
		public:
			InnerBox();
			~InnerBox(){ if(vertex) { delete vertex; vertex = 0; } }
			operator graphics::VertexBuffer<graphics::VertexPosition>&() { return *vertex; }
			void Begin() { if(vertex) vertex->Begin(); }
			void Draw() { if(vertex) vertex->Draw(); }
			void End() { if(vertex) vertex->End(); }
		protected:
			graphics::VertexBuffer<graphics::VertexPosition> *vertex;
		};
	};
};

#endif //< PRIMITIVE_H
