/*!
$Id: utilities.h 630 2009-12-21 07:52:59Z horino $
*/
#ifndef UTILITIES_H
#define UTILITIES_H

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#include <string.h>
#include <cmath>
#include "indi.h"

namespace indi
{
	namespace math
	{
		//< Ƃ肠W֐bvłiXւjB
		template<typename T> 
		T Sin(T rad) { return std::sin(rad); }
		template<typename T> 
		T Cos(T rad) { return std::cos(rad); }
		template<typename T> 
		T Asin(T rad) { return std::asin(rad); }
		template<typename T> 
		T Acos(T rad) { return std::acos(rad); }
		template<typename T> 
		T Sqrt(T rad) { return std::sqrt(rad); }

		template<typename T> 
		T ToRadian(T deg) { return deg * static_cast<T>(INDI_PI) / static_cast<T>(180.0f); }
		template<typename T> 
		T ToDegree(T rad) { return rad * static_cast<T>(180.0f) / static_cast<T>(INDI_PI); }

		template<typename T>
		const T &Min(const T &a, const T &b) { return b < a ? b : a; }
		template<typename T>
		const T &Min(const T &a, const T &b, const T &c) { return Min(Min(a, b), c); }

		template<typename T>
		const T &Max(const T &a, const T &b) { return a < b ? b : a; }
		template<typename T>
		const T &Max(const T &a, const T &b, const T &c) { return Max(Max(a, b), c); }

		template<typename T>
		const T &Clamp(const T &a, const T &mn, const T &mx) { return Min(Max(a, mn), mx); }
		template<typename T>
		const T Saturate(const T &a) { return Clamp(a, T(0), T(1)); }

		template<typename T>
		const T Abs(const T &a) { return a > 0 ? a : -a; }

		template<typename T>
		const T Pow(const T &a) { return a * a; }

		template<typename T>
		const T Pow(T a, T b) { return std::pow(a, b); }
	};

	template<typename T>
	void Swap(T &a, T &b) { T tmp(a); a = b; b = tmp; }

	static void * ATTR_USED  Memset(void *dst, u8 val, u32 size) { return memset(dst, val, size); }
	static void * ATTR_USED Memcpy(void *dst, const void *src, u32 size) { return memcpy(dst, src, size); }
#ifdef PS3
	static void ATTR_USED Memzero(void *dst, u32 size) { Memset(dst, 0, size); }
#else
	static void Memzero(void *dst, u32 size) { ZeroMemory(dst, size); }
#endif
	static char * ATTR_USED Strcpy(char *dst, const char *src) { return strcpy(dst, src); }
	static char * ATTR_USED Strncpy(char *dst, const char *src, u32 n) { return strncpy(dst, src, n); }
	static char * ATTR_USED Strcat(char *dst, const char *src) { return strcat(dst, src); }
	static s32 ATTR_USED Strcmp(const char *a, const char *b) { return strcmp(a, b); }
	static s32 ATTR_USED Strncmp(const char *a, const char *b, u32 n) { return strncmp(a, b, n); }
	static u32 ATTR_USED Strlen(const char *s) { return strlen(s); }
#ifdef PS3
	static void ATTR_USED Vsprintf(char *dst, const char *fmt, va_list lst) { vsprintf(dst, fmt, lst); }
#else
	static void ATTR_USED Vsprintf(char *dst, const char *fmt, va_list lst) { wvsprintf(dst, fmt, lst); }
#endif
	static s32 ATTR_USED Isspace(s32 c) { return isspace(c); }
	static s32 ATTR_USED Isdigit(s32 c) { return isdigit(c); }
	static s32 ATTR_USED Isalpha(s32 c) { return isalpha(c); }
	static s32 ATTR_USED Isalnum(s32 c) { return isalnum(c); }
	static s32 ATTR_USED Iscntrl(s32 c) { return iscntrl(c); }
	static s32 ATTR_USED Islower(s32 c) { return islower(c); }
	static s32 ATTR_USED Isupper(s32 c) { return isupper(c); }

	namespace string
	{
		static u32 ATTR_USED AllocStrcpy(char **dst, const char *src) {
			u32 len(Strlen(src) + 1);
			*dst = new char[len];
			Strncpy(*dst, src, len);
			return len;
		}
	}

	/*!
	\brief GfBAXbv܂
	*/
	static u32 ATTR_USED SwapEndian(u32 v)
	{
		return ((v >> 24) & 0x000000ff) | ((v >> 8) & 0x0000ff00) | ((v << 8) & 0x00ff0000) | ((v << 24) & 0xff000000); 
	}
	static void ATTR_USED SwapEndian(u32 *p, u32 n)
	{
		for(u32 i=0;i<n;++i, ++p)
			*p = SwapEndian(*p);
	}

	/*!
	\brief u32\f32\֕ϊ܂
	*/
	static f32 ATTR_USED ToF32(u32 v) {
		u32 sign_mask(v & 0x80000000);             //  : 1000|0000|0000|0000|0000|0000|0000|0000
		u32 exponent_mask((v & 0x7f800000) >> 23); // w : 0111|1111|1000|0000|0000|0000|0000|0000
		u32 fraction_mask(v & 0x007fffff);         //  : 0000|0000|0111|1111|1111|1111|1111|1111

		// exponent_maskAfraction_mask0̏ꍇ0
		if(exponent_mask == 0 && fraction_mask == 0)    
			return 0.0f;

		// exponent_mask0xff̏ꍇ́ANan, -, 
		ASSERT(exponent_mask != 0xff); 

		//< ̌vZ
		f32 fraction = 0.0f;
		for(u32 i=0;i<23;++i)
			if((1 << i) & fraction_mask)
				fraction += math::Pow(2.0f, static_cast<f32>(i) - 23.0f);

		return (sign_mask ? -1.0f : 1.0f) * math::Pow(2.0f, static_cast<f32>(exponent_mask) - 127.0f) * (1.0f + fraction);
	}
	static f16 ATTR_USED ToF16(f32 v){
		u32 u(*reinterpret_cast<u32 *>(&v));
		if(!u) return 0;
		s32	e(math::Clamp(static_cast<s32>(((u & 0x7f800000) >> 23) - 127 + 15), 0, 31));
		return (((u & 0x80000000) >> 16) & 0x8000) | ((e << 10) & 0x7c00) | (((u & 0x007fffff) >> 13) & 0x03ff);
	}
	static f32 ATTR_USED ToF32(f16 v){
		if(!v) return 0.0f;
		s32 e(((v & 0x7c00) >> 10) - 15 + 127);
		u32 u(((v & 0x8000) << 16) | ((e << 23) & 0x7f800000) | ((v & 0x03ff) << 13));
		return *reinterpret_cast<f32 *>(&u);
	}

	struct Delete {
		template<typename T>
		void operator()(const T *ptr) const { if(ptr) { delete ptr; ptr = 0; } }
	};

	struct DeleteArray {
		template<typename T>
		void operator()(const T *ptr) const { if(ptr) { delete [] ptr; ptr = 0; } }
	};
};


#endif // UTILITIES_H
