/*!
$Id: closest_point.cpp 652 2010-01-08 09:54:38Z horino $
*/
#include "stdafx.h"

#define CLOSEST_POINT_CPP

#include "collision/collision.h"
using namespace indi;

/*!
\brief AABBƓ_̍ŋߐړ_(삵Ă)
\param AABB̒S
\param AABB̔a
\param _
\return ŋߐړ_
*/
Vector3 Collision::ClosestPoint::AABBPoint(const Vector3 &cAB, const Vector3 &rAB, 
										   const Vector3 &p)
{
	Vector3 mn(cAB - rAB);
	Vector3 mx(cAB + rAB);
	return Vector3(math::Clamp(p.X(), mn.X(), mx.X()), math::Clamp(p.Y(), mn.Y(), mx.Y()), math::Clamp(p.Z(), mn.Z(), mx.Z()));
}

/*!
\brief OBBƓ_̍ŋߐړ_(삵Ă)
\param OBB̒S
\param OBB̔a
\param OBBX
\param OBBY
\param OBBZ
\param _
\return ŋߐړ_
*/
Vector3 Collision::ClosestPoint::OBBPoint(const Vector3 &cOB, const Vector3 &rOB, const Vector3 &xOB, const Vector3 &yOB, const Vector3 &zOB, 
										  const Vector3 &p)
{
	Vector3 d(p - cOB);
	Vector3 res(cOB);
	res += math::Clamp(d * xOB, -rOB.X(), rOB.X()) * xOB;
	res += math::Clamp(d * yOB, -rOB.Y(), rOB.Y()) * yOB;
	res += math::Clamp(d * zOB, -rOB.Z(), rOB.Z()) * zOB;
	return res;
}

/*!
\brief ʂƓ_̍ŋߐړ_(삵Ă)
\param ʂ̖@(vK)
\param ʂ̌_̋
\param _
\return ŋߐړ_
*/
Vector3 Collision::ClosestPoint::PlanePoint(const Vector3 &nPl, f32 dPl,
											const Vector3 p)
{
	return p - (nPl * p - dPl) * nPl;
}

/*!
\brief _Ɛ̍ŋߐړ_(삵Ă)
\param _
\param ̒[_
\param ̒[_
\param ŋߐړ_p[^Ԃ܂
\return ŋߐړ_
*/
Vector3 Collision::ClosestPoint::PointSegment(const Vector3 &p,
											  const Vector3 &seg0, const Vector3 &seg1,
											  f32 &t)
{
	Vector3 d(seg1 - seg0);
	t = (p - seg0) * d;
	if (t <= 0.0f) {
		t = 0.0f;
		return seg0;
	}
	else {
		f32 denom(d.LengthSquared());
		if (t >= denom) {
			t = 1.0f;
			return seg1;
		}
		return seg0 + d * t / denom;
	}
}

/*!
\brief Ɛ̍ŋߐړ_(삵Ă)
\param 0̒[_
\param 0̒[_
\param 1̒[_
\param 1̒[_
\param 0̃p[^Ԃ܂
\param 1̃p[^Ԃ܂
*/
void Collision::ClosestPoint::SegmentSegment(const Vector3 &segA0, const Vector3 &segA1,
											 const Vector3 &segB0, const Vector3 &segB1, 
											 f32 &t0, f32 &t1)
{
	Vector3 d0(segA1 - segA0);
	Vector3 d1(segB1 - segB0);
	f32 a(d0.LengthSquared());
	f32 e(d1.LengthSquared());

	//< Q̐Ƃ_ɏkނĂ
	if(a <= LIM_EPSILON(f32) && e <= LIM_EPSILON(f32)) { t0 = 0.0f; t1 = 0.0f; return; }

	Vector3 r(segA0 - segB0);
	f32 f(d1 * r);

	f32 s, t;
	//< A_ɏkނĂ
	if(a <= LIM_EPSILON(f32)) { s = 0.0f; t = math::Clamp(f / e, 0.0f, 1.0f); }
	else {
		f32 c(d0 * r);
		//< B_ɏkނĂ
		if(e <= LIM_EPSILON(f32)) { s = math::Clamp(-c / a, 0.0f, 1.0f); t = 0.0f; }
		else {
			f32 b(d0 * d1);
			f32 denom(a * e - math::Pow(b));
			//< sȏꍇ
			if(math::Abs(denom) <= LIM_EPSILON(f32)) s = 0.0f;
			else s = math::Clamp((b * f - c * e) / denom, 0.0f, 1.0f);

			f32 tnom(b * s + f);
			if(tnom < 0.0f) { t = 0.0f; s = math::Clamp(-c / a, 0.0f, 1.0f); }
			else if(tnom > e) { t = 1.0f; s = math::Clamp((b - c) / a, 0.0f, 1.0f); }
			else t = tnom / e;
		}
	}

	t0 = s;
	t1 = t;
}
