/*!
$Id: detection.cpp 652 2010-01-08 09:54:38Z horino $
*/
#include "stdafx.h"

#define DETECTION_CPP

#include "utilities.h"
#include "vector3.h"
#include "collision/collision.h"
using namespace indi;

/*!
\brief AABBm̃RW(삵Ă)
\param AABB0̒S
\param AABB0̔a
\param AABB1̒S
\param AABB1̔a
\return Փ˂ĂȂTRUE
*/
BOOL Collision::Detection::AABBAABB(const Vector3 &cAB0, const Vector3 &rAB0, 
									const Vector3 &cAB1, const Vector3 &rAB1)
{
	if (math::Abs(cAB0.X() - cAB1.X()) > rAB0.X() + rAB1.X()) return FALSE;
	if (math::Abs(cAB0.Y() - cAB1.Y()) > rAB0.Y() + rAB1.Y()) return FALSE;
	if (math::Abs(cAB0.Z() - cAB1.Z()) > rAB0.Z() + rAB1.Z()) return FALSE;
	return TRUE;
}

/*!
\brief AABBm̃RW(삵Ă)
\param AABB0̒S
\param AABB0̔a
\param AABB0̑x
\param AABB1̒S
\param AABB1̔a
\param AABB1̑x
\param Փ˃o[^Ԃ܂
\return Փ˂ĂȂTRUE
*/
BOOL Collision::Detection::AABBAABB(const Vector3 &cAB0, const Vector3 &rAB0, const Vector3 &vAB0,
									const Vector3 &cAB1, const Vector3 &rAB1, const Vector3 &vAB1,
									f32 &t)
{
	if(AABBAABB(cAB0, rAB0, cAB1, rAB1)) {
		t = 0.0f;
		return TRUE;
	}

	Vector3 v(vAB1 - vAB0);
	if(math::Abs(v.X()) < LIM_EPSILON(f32) && math::Abs(v.Y()) < LIM_EPSILON(f32) && math::Abs(v.Z()) < LIM_EPSILON(f32)) 
		return FALSE;

	t = 0.0f;
	f32 t1(1.0f);

	Vector3 mnA(cAB0 - rAB0);
	Vector3 mxA(cAB0 + rAB0);
	Vector3 mnB(cAB1 - rAB1);
	Vector3 mxB(cAB1 + rAB1);

	if(v.X() < 0.0f) {
		if(mxB.X() < mnA.X())
			return FALSE;
		f32 inv(1.0f / v.X());
		if(mxA.X() < mnB.X()) t = math::Max((mxA.X() - mnB.X()) * inv, t);
		if(mxB.X() > mnA.X()) t1 = math::Min((mnA.X() - mxB.X()) * inv, t1);
	}
	if(v.X() > 0.0f) {
		if (mnB.X() > mxA.X())
			return FALSE;
		f32 inv(1.0f / v.X());
		if(mxB.X() < mnA.X()) t = math::Max((mnA.X() - mxB.X()) * inv, t);
		if(mxA.X() > mnB.X()) t1 = math::Min((mxA.X() - mnB.X()) * inv, t1);
	}

	if(v.Y() < 0.0f) {
		if(mxB.Y() < mnA.Y()) 
			return FALSE;
		f32 inv(1.0f / v.Y());
		if(mxA.Y() < mnB.Y()) t = math::Max((mxA.Y() - mnB.Y()) * inv, t);
		if(mxB.Y() > mnA.Y()) t1 = math::Min((mnA.Y() - mxB.Y()) * inv, t1);
	}
	if(v.Y() > 0.0f)	{
		if(mnB.Y() > mxA.Y()) 
			return FALSE;
		f32 inv(1.0f / v.Y());
		if(mxB.Y() < mnA.Y()) t = math::Max((mnA.Y() - mxB.Y()) * inv, t);
		if(mxA.Y() > mnB.Y()) t1 = math::Min((mxA.Y() - mnB.Y()) * inv, t1);
	}

	if(v.Z() < 0.0f) {
		if(mxB.Z() < mnA.Z())
			return FALSE;
		f32 inv(1.0f / v.Z());
		if(mxA.Z() < mnB.Z()) t = math::Max((mxA.Z() - mnB.Z()) * inv, t);
		if(mxB.Z() > mnA.Z()) t1 = math::Min((mnA.Z() - mxB.Z()) * inv, t1);
	}
	if(v.Z() > 0.0f) {
		if(mnB.Z() > mxA.Z())
			return FALSE;
		f32 inv(1.0f / v.Z());
		if(mxB.Z() < mnA.Z()) t = math::Max((mnA.Z() - mxB.Z()) * inv, t);
		if(mxA.Z() > mnB.Z()) t1 = math::Min((mxA.Z() - mnB.Z()) * inv, t1);
	}

	if(t > t1) 
		return FALSE;

	return TRUE;
}

/*!
\brief AABBƃyʂƂ̃RW()
\param AABB̒S
\param AABB̔a
\param ʂ̖@
\param ʂ̌_̋
\return Փ˂ĂȂTRUE
*/
BOOL Collision::Detection::AABBPlane(const Vector3 &cAB, const Vector3 &rAB, 
									 const Vector3 &nPl, f32 dPl)
{
	return math::Abs(nPl * cAB - dPl) <= rAB.X() * math::Abs(nPl.X()) + rAB.Y() * math::Abs(nPl.Y()) + rAB.Z() * math::Abs(nPl.Z());
}

/*!
\brief AABBƃ\bhʂƂ̃RW()
\param AABB̒S
\param AABB̔a
\param ʂ̖@
\param ʂ̌_̋
\return Փ˂ĂȂTRUE
*/
BOOL Collision::Detection::AABBSolidPlane(const Vector3 &cAB, const Vector3 &rAB, 
									 const Vector3 &nPl, f32 dPl)
{
	return nPl * cAB - dPl <= rAB.X() * math::Abs(nPl.X()) + rAB.Y() * math::Abs(nPl.Y()) + rAB.Z() * math::Abs(nPl.Z());
}

/*!
\brief AABBƃyʂƂ̃RW(삵Ă)
\param AABB̒S
\param AABB̔a
\param AABB̑x
\param ʂ̖@
\param ʂ̌_̋
\return Փ˂ĂȂTRUE
*/
BOOL Collision::Detection::AABBPlane(const Vector3 &cAB, const Vector3 &rAB, const Vector3 &vAB,
									 const Vector3 &nPl, float dPl)
{
	f32 r(rAB.X() * math::Abs(nPl.X()) + rAB.Y() * math::Abs(nPl.Y()) + rAB.Z() * math::Abs(nPl.Z()));
	if(math::Abs(nPl * cAB - dPl) <= r) 
		return TRUE;
	if(nPl * vAB >= 0.0f)
		return FALSE;
	return PlaneSphere(nPl, dPl, cAB, r, vAB);
}

/*!
\brief AABBƃyʂƂ̃RW(삵Ă)
\param AABB̒S
\param AABB̔a
\param AABB̑x
\param ʂ̖@
\param ʂ̌_̋
\param Փ˃o[^Ԃ܂
\return Փ˂ĂȂTRUE
*/
BOOL Collision::Detection::AABBPlane(const Vector3 &cAB, const Vector3 &rAB, const Vector3 &vAB,
									 const Vector3 &nPl, float dPl,
									 f32 &t)
{
	f32 r(rAB.X() * math::Abs(nPl.X()) + rAB.Y() * math::Abs(nPl.Y()) + rAB.Z() * math::Abs(nPl.Z()));
	if(math::Abs(nPl * cAB - dPl) <= r)	{
		t = 0.0f;
		return TRUE;
	}
	f32 nv(nPl * vAB);
	if(nv >= 0.0f) 
		return FALSE;
	t = (r + dPl - nPl * cAB) / nv;
	if (t >= 0.0f && t <= 1.0f) 
		return TRUE;
	return FALSE;
}

/*!
\brief AABBƃ\bhʂƂ̃RW(삵Ă)
\param AABB̒S
\param AABB̔a
\param AABB̑x
\param ʂ̖@
\param ʂ̌_̋
\return Փ˂ĂȂTRUE
*/
BOOL Collision::Detection::AABBSolidPlane(const Vector3 &cAB, const Vector3 &rAB, const Vector3 &vAB,
										  const Vector3 &nPl, float dPl)
{
	f32 r(rAB.X() * math::Abs(nPl.X()) + rAB.Y() * math::Abs(nPl.Y()) + rAB.Z() * math::Abs(nPl.Z()));
	if(nPl * cAB - dPl <= r) 
		return TRUE;
	if(nPl * vAB >= 0.0f)
		return FALSE;
	return PlaneSphere(nPl, dPl, cAB, r, vAB);
}

/*!
\brief AABBƃ\bhʂƂ̃RW(삵Ă)
\param AABB̒S
\param AABB̔a
\param AABB̑x
\param ʂ̖@
\param ʂ̌_̋
\param Փ˃o[^Ԃ܂
\return Փ˂ĂȂTRUE
*/
BOOL Collision::Detection::AABBSolidPlane(const Vector3 &cAB, const Vector3 &rAB, const Vector3 &vAB,
									 const Vector3 &nPl, float dPl,
									 f32 &t)
{
	f32 r(rAB.X() * math::Abs(nPl.X()) + rAB.Y() * math::Abs(nPl.Y()) + rAB.Z() * math::Abs(nPl.Z()));
	if(nPl * cAB - dPl <= r)	{
		t = 0.0f;
		return TRUE;
	}
	f32 nv(nPl * vAB);
	if(nv >= 0.0f) 
		return FALSE;
	t = (r + dPl - nPl * cAB) / nv;
	if (t >= 0.0f && t <= 1.0f) 
		return TRUE;
	return FALSE;
}

/*!
\brief AABBƓ_Ƃ̃RW(삵Ă)
\param AABB̒S
\param AABB̔a
\param _
\return Փ˂ĂȂTRUE
*/
BOOL Collision::Detection::AABBPoint(const Vector3 &cAB, const Vector3 &rAB,
									 const Vector3 &p)
{
	if(math::Abs(cAB.X() - p.X()) > rAB.X()) return FALSE;
	if(math::Abs(cAB.Y() - p.Y()) > rAB.Y()) return FALSE;
	if(math::Abs(cAB.Z() - p.Z()) > rAB.Z()) return FALSE;
	return TRUE;
}

/*!
\BRIEF AABBƔƂ̃RW(삵Ă)
\param AABB̒S
\param AABB̔a
\param ̎n_
\param ̕xNg
\param Փ˃o[^Ԃ܂
\return Փ˂ĂȂTRUE
\note p180
*/
BOOL Collision::Detection::AABBRay(const Vector3 &cAb, const Vector3 &rAb, 
								   const Vector3 &sRay, const Vector3 &dRay,
								   f32 &t)
{
	t = 0.0f;
	f32 tmax(LIM_MAX(f32));

	Vector3 amin(cAb - rAb);
	Vector3 amax(cAb + rAb);
	for(u32 i=0;i<3;++i){
		if(math::Abs(dRay[i]) < LIM_EPSILON(f32)) {
			if(sRay[i] < amin[i] || sRay[i] > amax[i]) return FALSE;
		} else {
			f32 ood(1.0f / dRay[i]);
			f32 t1((amin[i] - sRay[i]) * ood);
			f32 t2((amax[i] - sRay[i]) * ood);
			if(t1 > t2) 
				Swap(t1, t2);
			t = math::Max(t, t1);
			tmax = math::Min(tmax, t2);
			if(t > tmax) return FALSE;
		} 
	}
	return TRUE;
}

/*!
\brief AABBƐƂ̃RW(삵Ă)
\param AABB̒S
\param AABB̔a
\param ̒[_
\param ̒[_
\return Փ˂ĂȂTRUE
\note p183
*/
BOOL Collision::Detection::AABBSegment(const Vector3 &cAb, const Vector3 &rAb, 
						const Vector3 &seg0, const Vector3 &seg1)
{
	Vector3 e(rAb + rAb);
	Vector3 m(seg0 + seg1 - cAb - cAb);
	Vector3 d(seg1 - seg0);

	f32 adx(math::Abs(d.X()));
	if(math::Abs(m.X()) > e.X() + adx) return FALSE;
	f32 ady(math::Abs(d.Y()));
	if(math::Abs(m.Y()) > e.Y() + ady) return FALSE;
	f32 adz(math::Abs(d.Z()));
	if(math::Abs(m.Z()) > e.Z() + adz) return FALSE;

	adx += LIM_EPSILON(f32);
	ady += LIM_EPSILON(f32);
	adz += LIM_EPSILON(f32);
	if(math::Abs(m.Y() * d.Z() - m.Z() * d.Y()) > e.Y() * adz + e.Z() * ady) return FALSE;
	if(math::Abs(m.Z() * d.X() - m.X() * d.Z()) > e.X() * adz + e.Z() * adx) return FALSE;
	if(math::Abs(m.X() * d.Y() - m.Y() * d.X()) > e.X() * ady + e.Y() * adx) return FALSE;
	return TRUE;
}

/*!
\brief AABBƐƂ̃RW(삵Ă)
\param AABB̒S
\param AABB̔a
\param ̒[_
\param ̒[_
\param Փ˃o[^Ԃ܂
\return Փ˂ĂȂTRUE
*/
BOOL Collision::Detection::AABBSegment(const Vector3 &cAb, const Vector3 &rAb, 
									   const Vector3 &seg0, const Vector3 &seg1,
									   f32 &t)
{
	return AABBRay(cAb, rAb, seg0, (seg1 - seg0), t) && t <= 1.0f; 
}

/*!
\brief AABBƋƂ̃RW(삵Ă)
\param AABB̒S
\param AABB̔a
\param ̒S
\param ̔a
\return Փ˂ĂȂTRUE
*/
BOOL Collision::Detection::AABBSphere(const Vector3 &cAB, const Vector3 &rAB,
									  const Vector3 &cSp, f32 rSp)
{
	return Collision::Distance::Squared::AABBPoint(cAB, rAB, cSp) <= math::Pow(rSp);
}

/*!
\brief AABBƋƂ̃RW(삵Ă)
\param AABB̒S
\param AABB̔a
\param ̒S
\param ̔a
\param ̑x
\param Փ˃o[^Ԃ܂
\return Փ˂ĂȂTRUE
*/
static Vector3 corner(const Vector3 &mn, const Vector3 &mx, u32 n) { return Vector3((n & 1) ? mx.X() : mn.X(), (n & 2) ? mx.Y() : mn.Y(), (n & 4) ? mx.Z() : mn.Z()); }
BOOL Collision::Detection::AABBSphere(const Vector3 &cAB, const Vector3 &rAB,
									  const Vector3 &cSp, f32 rSp, const Vector3 &vSp,
									  f32 &t)
{
	Vector3 rab(rAB + Vector3(rSp));
	if(!AABBRay(cAB, rab, cSp, vSp, t) || t > 1.0f)
		return FALSE;
	Vector3 p(cSp + vSp * t);

	u32 u = 0, v = 0;
	Vector3 mn(cAB - rab);
	Vector3 mx(cAB + rab);
	if(p.X() < mn.X()) u |= 1;
	if(p.X() > mx.X()) v |= 1;
	if(p.Y() < mn.Y()) u |= 2;
	if(p.Y() > mx.Y()) v |= 2;
	if(p.Z() < mn.Z()) u |= 4;
	if(p.Z() > mx.Z()) v |= 4;

	u32 m(u + v); // == u | v

	//< rbgPZbgĂꍇAp͖ʗ̈ɂ
	//< (gAABBƂ̌͐ʂƂȂ)
	if ((m & (m - 1)) == 0) 
		return TRUE;

	Vector3 seg0(cSp);
	Vector3 seg1(cSp + vSp);

	//< SẴrbgZbgĂꍇAp͒_̈ɂ
	if (m == 7) {
		f32 tmin(LIM_MAX(f32));
		Vector3 c(corner(mn, mx, v));
		if(CapsuleSegment(c, corner(mn, mx, v ^ 1), rSp, seg0, seg1, t))
			tmin = math::Min(t, tmin);
		if(CapsuleSegment(c, corner(mn, mx, v ^ 2), rSp, seg0, seg1, t))
			tmin = math::Min(t, tmin);
		if(CapsuleSegment(c, corner(mn, mx, v ^ 4), rSp, seg0, seg1, t))
			tmin = math::Min(t, tmin);
		if(tmin == LIM_MAX(f32))
			return FALSE;
		t = tmin;
		return TRUE;
	}

	return CapsuleSegment(corner(mn, mx, u ^ 7), corner(mn, mx, v), rSp, seg0, seg1, t);
}

/*!
\brief JvZƃJvZƂ̃RW(삵Ă)
\param JvZ0̒[_
\param JvZ0̒[_
\param JvZ0̔a
\param JvZ1̒[_
\param JvZ1̒[_
\param JvZ1̔a
\return Փ˂ĂȂTRUE
*/
BOOL Collision::Detection::CapsuleCapsule(const Vector3 &cpA0, const Vector3 &cpA1, f32 rCpA, 
										 const Vector3 &cpB0, const Vector3 &cpB1, f32 rCpB)
{
	return Distance::Squared::SegmentSegment(cpA0, cpA1, cpB0, cpB1) <= math::Pow(rCpA + rCpB);
}

/*!
\brief JvZƐƂ̃RW(삵Ă)
\param JvZ̒[_
\param JvZ̒[_
\param JvZ̔a
\param ̒[_
\param ̒[_
\param Փ˃o[^Ԃ܂
\return Փ˂ĂȂTRUE
*/
BOOL Collision::Detection::CapsuleSegment(const Vector3 &cp0, const Vector3 &cp1, f32 rCp,
										  const Vector3 &seg0, const Vector3 &seg1,
										  f32 &t)
{
	return Distance::Squared::SegmentSegment(cp0, cp1, seg0, seg1) <= math::Pow(rCp);
}

/*!
\brief JvZƋƂ̃RW(삵Ă)
\param JvZ̒[_
\param JvZ̒[_
\param JvZ̔a
\param ̒S
\param ̔a
\return Փ˂ĂȂTRUE
*/
BOOL Collision::Detection::CapsuleSphere(const Vector3 &cp0, const Vector3 &cp1, f32 rCp, 
										const Vector3 &cSp, f32 rSp)
{
	return Distance::Squared::PointSegment(cSp, cp0, cp1) <= math::Pow(rCp + rSp);
}

/*!
\brief V_Ɛ̃RW(v)
\param V_̒[_
\param V_̒[_
\param V_̔a
\param ̒[_
\param ̒[_
\param Փ˃o[^Ԃ܂
\return Փ˂ĂȂTRUE
*/
BOOL Collision::Detection::CylinderSegment(const Vector3 &cy0, const Vector3 &cy1, f32 rCy, 
										   const Vector3 &seg0, const Vector3 &seg1,
										   f32 &t)
{
	Vector3 d(cy1 - cy0);
	Vector3 m(seg0 - cy0);
	Vector3 n(seg1 - seg0);

	f32 md(m * d);
	f32 nd(n * d);
	f32 dd(d.LengthSquared());

	f32 mdnd(md + nd);
	if(md < 0.0f && mdnd < 0.0f) return FALSE;
	if(md > dd && mdnd > 0.0f) return FALSE;

	f32 nn(n.LengthSquared());
	f32 mn(m * n);
	f32 a(dd * nn - math::Pow(nd));
	f32 k(m.LengthSquared() - math::Pow(rCy));
	f32 c(dd * k - math::Pow(md));

	if(math::Abs(a) < LIM_EPSILON(f32)){
		if(c > 0.0f) return FALSE;
		if(md < 0.0f) t = -mn / nn;
		else if(md > dd) t = (nd - mn) / nn;
		else t = 0.0f;
		return TRUE;
	}

	f32 b(dd * mn - nd * md);
	f32 disc(math::Pow(b) - a * c);
	if(disc < 0.0f) return FALSE;

	f32 t0(t = (-b - math::Sqrt(disc)) / a);
	f32 tmp(md + t * nd);
	if(tmp < 0.0f){
		if(nd <= 0.0f) return FALSE;
		t = -md / nd;
		if(k + t * (2.0f * mn + t * nn) <= 0.0f) return TRUE;
	} else if(tmp > dd) {
		if(nd >= 0.0f) return FALSE;
		t = (dd - md) / nd;
		if(k + dd - 2.0f * md + t * (2.0f * (mn - nd) + t * nn) <= 0.0f) return TRUE;
	}
	t = t0;
	return t >= 0.0f && t <= 1.0f;
}

/*!
\brief OBBm̃RW(삵Ă)
\param OBB0̒S
\param OBB0̔a
\param OBB0X
\param OBB0Y
\param OBB0Z
\param OBB1̒S
\param OBB1̔a
\param OBB1X
\param OBB1Y
\param OBB1Z
\return Փ˂ĂȂTRUE
*/
BOOL Collision::Detection::OBBOBB(const Vector3 &cOB0, const Vector3 &rOB0, const Vector3 &xOB0, const Vector3 &yOB0, const Vector3 &zOB0,
								  const Vector3 &cOB1, const Vector3 &rOB1, const Vector3 &xOB1, const Vector3 &yOB1, const Vector3 &zOB1)
{
	Vector3 r0(xOB0 * xOB1, xOB0 * yOB1, xOB0 * zOB1); 
	Vector3 r1(yOB0 * xOB1, yOB0 * yOB1, yOB0 * zOB1); 
	Vector3 r2(zOB0 * xOB1, zOB0 * yOB1, zOB0 * zOB1);

	Vector3 tmp(cOB1 - cOB0);
	Vector3 t(tmp * xOB0, tmp * yOB0, tmp * zOB0);

	Vector3 abs_r0(math::Abs(r0.X()) + LIM_EPSILON(f32), math::Abs(r0.Y()) + LIM_EPSILON(f32), math::Abs(r0.Z()) + LIM_EPSILON(f32));
	Vector3 abs_r1(math::Abs(r1.X()) + LIM_EPSILON(f32), math::Abs(r1.Y()) + LIM_EPSILON(f32), math::Abs(r1.Z()) + LIM_EPSILON(f32));
	Vector3 abs_r2(math::Abs(r2.X()) + LIM_EPSILON(f32), math::Abs(r2.Y()) + LIM_EPSILON(f32), math::Abs(r2.Z()) + LIM_EPSILON(f32));

	f32 ra(rOB0.X());
	f32 rb(rOB1.X() * abs_r0.X() + rOB1.Y() * abs_r0.Y()  + rOB1.Z() * abs_r0.Z());
	if(math::Abs(t.X()) > ra + rb)	return FALSE;
	ra = rOB0.Y();
	rb = rOB1.X() * abs_r1.X() + rOB1.Y() * abs_r1.Y()  + rOB1.Z() * abs_r1.Z();
	if(math::Abs(t.Y()) > ra + rb)	return FALSE;
	ra = rOB0.Z();
	rb = rOB1.X() * abs_r2.X() + rOB1.Y() * abs_r2.Y()  + rOB1.Z() * abs_r2.Z();
	if(math::Abs(t.Z()) > ra + rb)	return FALSE;

	ra = rOB0.X() * abs_r0.X() + rOB0.Y() * abs_r1.X() + rOB0.Z() * abs_r2.X();
	rb = rOB1.X();
	if(math::Abs(t.X() * r0.X() + t.Y() * r1.X() + t.Z() * r2.X()) > ra + rb) return FALSE;
	ra = rOB0.X() * abs_r0.Y() + rOB0.Y() * abs_r1.Y() + rOB0.Z() * abs_r2.Y();
	rb = rOB1.Y();
	if(math::Abs(t.X() * r0.Y() + t.Y() * r1.Y() + t.Z() * r2.Y()) > ra + rb) return FALSE;
	ra = rOB0.X() * abs_r0.Z() + rOB0.Y() * abs_r1.Z() + rOB0.Z() * abs_r2.Z();
	rb = rOB1.Z();
	if(math::Abs(t.X() * r0.Z() + t.Y() * r1.Z() + t.Z() * r2.Z()) > ra + rb) return FALSE;

	ra = rOB0.Y() * abs_r2.X() + rOB0.Z() * abs_r1.X();
	rb = rOB1.Y() * abs_r0.Z() + rOB1.Z() * abs_r0.Y();
	if(math::Abs(t.Z() * r1.X() - t.Y() * r2.X()) > ra + rb) return FALSE;

	ra = rOB0.Y() * abs_r2.Y() + rOB0.Z() * abs_r1.Y();
	rb = rOB1.X() * abs_r0.Z() + rOB1.Z() * abs_r0.X();
	if(math::Abs(t.Z() * r1.Y() - t.Y() * r2.Y()) > ra + rb) return FALSE;

	ra = rOB0.Y() * abs_r2.Z() + rOB0.Z() * abs_r1.Z();
	rb = rOB1.X() * abs_r0.Y() + rOB1.Y() * abs_r0.X();
	if(math::Abs(t.Z() * r1.Z() - t.Y() * r2.Z()) > ra + rb) return FALSE;

	ra = rOB0.X() * abs_r2.X() + rOB0.Z() * abs_r0.X();
	rb = rOB1.Y() * abs_r1.Z() + rOB1.Z() * abs_r1.Y();
	if(math::Abs(t.X() * r2.X() - t.Z() * r0.X()) > ra + rb) return FALSE;

	ra = rOB0.X() * abs_r2.Y() + rOB0.Z() * abs_r0.Y();
	rb = rOB1.X() * abs_r1.Z() + rOB1.Z() * abs_r1.X();
	if(math::Abs(t.X() * r2.Y() - t.Z() * r0.Y()) > ra + rb) return FALSE;

	ra = rOB0.X() * abs_r2.Z() + rOB0.Z() * abs_r0.Z();
	rb = rOB1.X() * abs_r1.Y() + rOB1.Y() * abs_r1.X();
	if(math::Abs(t.X() * r2.Z() - t.Z() * r0.Z()) > ra + rb) return FALSE;

	ra = rOB0.X() * abs_r1.X() + rOB0.Y() * abs_r0.X();
	rb = rOB1.Y() * abs_r2.Z() + rOB1.Z() * abs_r2.Y();
	if(math::Abs(t.Y() * r0.X() - t.X() * r1.X()) > ra + rb) return FALSE;

	ra = rOB0.X() * abs_r1.Y() + rOB0.Y() * abs_r0.Y();
	rb = rOB1.X() * abs_r2.Z() + rOB1.Z() * abs_r2.X();
	if(math::Abs(t.Y() * r0.Y() - t.X() * r1.Y()) > ra + rb) return FALSE;

	ra = rOB0.X() * abs_r1.Z() + rOB0.Y() * abs_r0.Z();
	rb = rOB1.X() * abs_r2.Y() + rOB1.Y() * abs_r2.X();
	if(math::Abs(t.Y() * r0.Z() - t.X() * r1.Z()) > ra + rb) return FALSE;

	return TRUE;
}

/*!
\brief OBBƕʂ̃RW(삵Ă)
\param OBB̒S
\param OBB̔a
\param OBBX
\param OBBY
\param OBBZ
\param ʂ̖@
\param ʂ̌_̋
\return Փ˂ĂȂTRUE
*/
BOOL Collision::Detection::OBBPlane(const Vector3 &cOB, const Vector3 &rOB, const Vector3 &xOB, const Vector3 &yOB, const Vector3 &zOB,
									const Vector3 &nPl, f32 dPl)
{
	return math::Abs(nPl * cOB - dPl) <= rOB.X() * math::Abs(nPl * xOB) + rOB.Y() * math::Abs(nPl * yOB) + rOB.Z() * math::Abs(nPl * zOB);
}

/*!
\brief OBBƕʂ̃RW(삵Ă)
\param OBB̒S
\param OBB̔a
\param OBBX
\param OBBY
\param OBBZ
\param OBB̑x
\param ʂ̖@
\param ʂ̌_̋
\return Փ˂ĂȂTRUE
*/
BOOL Collision::Detection::OBBPlane(const Vector3 &cOB, const Vector3 &rOB, const Vector3 &xOB, const Vector3 &yOB, const Vector3 &zOB, const Vector3 &vOB,
									const Vector3 &nPl, f32 dPl)
{
	f32 r(rOB.X() * math::Abs(xOB * nPl) + rOB.Y() * math::Abs(yOB * nPl) + rOB.Z() * math::Abs(zOB * nPl));
	//< ŏ̎_ŌĂ
	if(math::Abs(nPl * cOB - dPl) <= r)
		return TRUE;
	//< AABB͕ʂ痣(s܂)֓Ă
	if(nPl * vOB >= 0.0f) 
		return FALSE;
	return PlaneSphere(nPl, dPl, cOB, r, vOB);
}

/*!
\brief OBBƕʂ̃RW(삵Ă)
\param OBB̒S
\param OBB̔a
\param OBBX
\param OBBY
\param OBBZ
\param OBB̑x
\param ʂ̖@
\param ʂ̌_̋
\param Փ˃o[^Ԃ܂
\return Փ˂ĂȂTRUE
*/
BOOL Collision::Detection::OBBPlane(const Vector3 &cOB, const Vector3 &rOB, const Vector3 &xOB, const Vector3 &yOB, const Vector3 &zOB, const Vector3 &vOB,
									const Vector3 &nPl, f32 dPl,
									f32 &t)
{
	f32 r(rOB.X() * math::Abs(xOB * nPl) + rOB.Y() * math::Abs(yOB * nPl) + rOB.Z() * math::Abs(zOB * nPl));
	if(math::Abs(nPl * cOB - dPl) <= r) {
		t = 0.0f;
		return TRUE;
	}
	f32 nv(nPl * vOB);
	if (nv >= 0.0f) 
		return FALSE;
	return PlaneSphere(nPl, dPl, cOB, r, vOB, t);
}

/*!
\brief OBBƋ̃RW(삵Ă)
\param OBB̒S
\param OBB̔a
\param OBBX
\param OBBY
\param OBBZ
\param ̒S
\param ̔a
\return Փ˂ĂȂTRUE
*/
BOOL Collision::Detection::OBBSphere(const Vector3 &cOB, const Vector3 &rOB, const Vector3 &xOB, const Vector3 &yOB, const Vector3 &zOB,
									 const Vector3 &cSp, f32 rSp)
{
	return (Collision::ClosestPoint::OBBPoint(cOB, rOB, xOB, yOB, zOB, cSp) - cSp).LengthSquared() <= math::Pow(rSp);
}

/*!
\brief ʂƕʂ̃RW(삵Ă)
\param 0̖@
\param 0̌_̋
\param 1̖@
\param 1̌_̋
\param ̎n_Ԃ܂
\param ̕Ԃ܂
\return Փ˂ĂȂTRUE
*/
BOOL Collision::Detection::PlanePlane(const Vector3 &nPl0, f32 dPl0, 
									  const Vector3 &nPl1, f32 dPl1, 
									  Vector3 &sLn, Vector3 &dLn)
{
	dLn = nPl0 ^ nPl1;
	f32 denom(dLn.LengthSquared());
	if(denom < LIM_EPSILON(f32))
		return FALSE;
	sLn = ((dPl0 * nPl1 - dPl1 * nPl0) ^ dLn) / denom;
	return TRUE;
}

/*!
\brief iyjʂƐ̃RW(삵Ă)
\param ʂ̖@
\param ʂ̌_̋
\param ̒[_
\param ̒[_
\param Փ˃o[^Ԃ܂
\return Փ˂ĂȂTRUE
*/
BOOL Collision::Detection::PlaneSegment(const Vector3 &nPl, f32 dPl, 
										const Vector3 &seg0, const Vector3 &seg1,
										f32 &t)
{
#ifdef _DEBUG
	ASSERT((seg1 - seg0).LengthSquared() > LIM_EPSILON(f32)); //< kނĂ
#endif
	t = (dPl - nPl * seg0) / (nPl * (seg1 - seg0));
	return t >= 0.0f && t <= 1.0f;
}

/*!
\brief (y)ʂƋ̃RW(삵Ă)
\param ʂ̖@
\param ʂ̌_̋
\param ̒S
\param ̔a
\return Փ˂ĂȂTRUE
*/
BOOL Collision::Detection::PlaneSphere(const Vector3 &nPl, f32 dPl,  
									   const Vector3 &cSp, f32 rSp)
{
	return math::Abs(cSp * nPl - dPl) <= rSp;
}
/*!
\brief (\bh)ʂƋ̃RW(삵Ă)
\param ʂ̖@
\param ʂ̌_̋
\param ̒S
\param ̔a
\return Փ˂ĂȂTRUE
\note ʂ̖@̔Αl܂Ă
*/
BOOL Collision::Detection::SolidPlaneSphere(const Vector3 &nPl, f32 dPl,  
											const Vector3 &cSp, f32 rSp)
{
	return cSp * nPl - dPl <= rSp;
}

/*!
\brief (y)ʂƋ̃RW(삵Ă)
\param ʂ̖@
\param ʂ̌_̋
\param ̒S
\param ̔a
\param ̑x
\param Փ˃o[^Ԃ܂
\return Փ˂ĂȂTRUE
*/
BOOL Collision::Detection::PlaneSphere(const Vector3 &nPl, f32 dPl,
				 const Vector3 &cSp, f32 rSp, const Vector3 &vSp, 
				 f32 &t)
{
	f32 dist(nPl * cSp - dPl);
	if(math::Abs(dist) <= rSp){
		t = 0.0f;
		return TRUE;
	} else {
		f32 denom(nPl * vSp);
		if(denom + dist >= 0.0f)
			return FALSE;
		else {
			f32 r(dist > 0.0f ? rSp : -rSp);
			t = (r - dist) / denom;
			return t >= 0.0f && t <= 1.0f;
		}
	}
}

/*!
\brief (y)ʂƋ̃RW(삵Ă)
\param ʂ̖@
\param ʂ̌_̋
\param ̒S
\param ̔a
\param ̑x
\return Փ˂ĂȂTRUE
*/
BOOL Collision::Detection::PlaneSphere(const Vector3 &nPl, f32 dPl,
				 const Vector3 &cSp, f32 rSp, const Vector3 &vSp)
{
	f32 adist(cSp * nPl - dPl);
	f32 bdist((cSp + vSp) * nPl - dPl);
	if(adist * bdist < 0.0f) 
		return TRUE;
	if(math::Abs(adist) <= rSp || math::Abs(bdist) <= rSp)
		return TRUE;
	return FALSE;
}

/*!
\brief CƋ̃RW(삵Ă)
\param C̎n_
\param C̕
\param ̒S
\param ̔a
\param Փ˃o[^Ԃ܂
\return Փ˂ĂȂTRUE
*/
BOOL Collision::Detection::RaySphere(const Vector3 &sRay, const Vector3 &dRay,
									 const Vector3 &cSp, f32 rSp,
									 f32 &t)
{
	Vector3 m(sRay - cSp);
	f32 b(m * dRay);
	f32 c(m.LengthSquared() - math::Pow(rSp));
	if(c > 0.0f && b > 0.0f) 
		return FALSE;
	f32 d(math::Pow(b) - c);
	if(d < 0.0f) 
		return FALSE;
	t = math::Max(-b - math::Sqrt(d), 0.0f);
	return TRUE;
}

/*!
\brief CƋ̃RW(삵Ă)
\param C̎n_
\param C̕
\param ̒S
\param ̔a
\param Փ˃o[^Ԃ܂
\return Փ˂ĂȂTRUE
*/
BOOL Collision::Detection::RaySphere(const Vector3 &sRay, const Vector3 &dRay,
								   const Vector3 &cSp, f32 rSp)
{
	Vector3 m(sRay - cSp);
	f32 c(m.LengthSquared() - math::Pow(rSp));
	if(c <= 0.0f)
		return TRUE;
	f32 b(m * dRay);
	if(b > 0.0f) 
		return FALSE;
	f32 d(math::Pow(b) - c);
	if(d < 0.0f)
		return FALSE;
	return TRUE;
}

/*!
\brief Ƌ̃RW(삵Ă)
\param ̒[_
\param ̒[_
\param ̒S
\param ̔a
\param Փ˃p[^Ԃ܂
\return Փ˂ĂȂTRUE
*/
BOOL Collision::Detection::SegmentSphere(const Vector3 &seg0, const Vector3 &seg1,
										 const Vector3 &cSp, f32 rSp,
										 f32 &t)
{
	return RaySphere(seg0, seg1 - seg0, cSp, rSp, t) && t<= 1.0f; 
}

/*!
\brief Ƌ̃RW(삵Ă)
\param 0̒S
\param 0̔a
\param 1̒S
\param 1̔a
\return Փ˂ĂȂTRUE
*/
BOOL Collision::Detection::SphereSphere(const Vector3 &cSp0, f32 rSp0, 
										const Vector3 &cSp1, f32 rSp1)
{
	f32 r(rSp0 + rSp1);
	return (cSp0 - cSp1).LengthSquared() <= math::Pow(r);
}

/*!
\brief (Ă)Ƌ̃RW(삵Ă)
\param 0̒S
\param 0̔a
\param 0̑x
\param 1̒S
\param 1̔a
\param 1̑x
\param Փ˃p[^Ԃ܂
\return Փ˂ĂȂTRUE
*/
BOOL Collision::Detection::SphereSphere(const Vector3 &cSp0, f32 rSp0, const Vector3 &vSp0,
										const Vector3 &cSp1, f32 rSp1, const Vector3 &vSp1,
										f32 &t)
{
#if 1
	//< Ԃł̏Փ˂o
	Vector3 s(cSp1 - cSp0);
	Vector3 v(vSp1 - vSp0);
	f32 r(rSp0 + rSp1);
	f32 c(s.LengthSquared() - math::Pow(r, 2.0f));
	if(c < 0.0f) {
		t = 0.0f;
		return TRUE;
	}
	f32 a(v.LengthSquared());
	if(a < LIM_EPSILON(f32)) 
		return FALSE;
	f32 b(v * s);
	if(b >= 0.0f) 
		return FALSE;
	f32 d(math::Pow(b, 2.0f) - a * c);
	if(d < 0.0f) 
		return FALSE;
	t = (-b - math::Sqrt(d)) / a;
	return FALSE;
#else
	//< Ԃł͏Փ˂ĂȂO
	f32 r(rSp0 + rSp1);
	Vector3 v(vSp0 - vSp1);
	f32 len(v.Length());
	if (RaySphere(cSp0, v / len, cSp1, r, t)) 
		return t <= len;
	return FALSE;
#endif
}

/*!
\brief i_jɊ܂܂邩
\param 䕽ʁij
\param 䕽ʁij
\param 䕽ʁij
\param 䕽ʁiEj
\param 䕽ʁiAj
\param 䕽ʁit@[j
\param ̒Si܂͓_j
\param ̔ai_̏ꍇ0.0fɂĂj
\return ܂܂ĂTRUE
\note _𔻒肷ꍇradius0.0fw肷΂悢
*/
BOOL Collision::Frustum::Contains(const Vector4 &t, const Vector4 &b, const Vector4 &l, const Vector4 &r, const Vector4 &n, const Vector4 &f,
								  const Vector3 &center, f32 radius) 
{
	return Vector3(t) * center + t.W() <= radius 
		&& Vector3(b) * center + b.W() <= radius 
		&& Vector3(l) * center + l.W() <= radius 
		&& Vector3(r) * center + r.W() <= radius 
		&& Vector3(n) * center + n.W() <= radius 
		&& Vector3(f) * center + f.W() <= radius;
}

/*!
\brief AABBɊ܂܂邩
\param 䕽ʁij
\param 䕽ʁij
\param 䕽ʁij
\param 䕽ʁiEj
\param 䕽ʁiAj
\param 䕽ʁit@[j
\param AABB̒S
\param AABB̔a
\return ܂܂ĂTRUE
*/
BOOL Collision::Frustum::Contains(const Vector4 &t, const Vector4 &b, const Vector4 &l, const Vector4 &r, const Vector4 &n, const Vector4 &f,
								  const Vector3 &center, const Vector3 &radius) 
{
	return Contains(t, b, l, r, n, f, center + radius, 0.0f) 
		|| Contains(t, b, l, r, n, f, center + Vector3(radius.X(), radius.Y(), -radius.Z()), 0.0f)
		|| Contains(t, b, l, r, n, f, center + Vector3(radius.X(), -radius.Y(), radius.Z()), 0.0f)
		|| Contains(t, b, l, r, n, f, center + Vector3(radius.X(), -radius.Y(), -radius.Z()), 0.0f)
		|| Contains(t, b, l, r, n, f, center + Vector3(-radius.X(), radius.Y(), radius.Z()), 0.0f)
		|| Contains(t, b, l, r, n, f, center + Vector3(-radius.X(), radius.Y(), -radius.Z()), 0.0f)
		|| Contains(t, b, l, r, n, f, center + Vector3(-radius.X(), -radius.Y(), radius.Z()), 0.0f)
		|| Contains(t, b, l, r, n, f, center - radius, 0.0f);
}
