/*!
$Id: material.cpp 531 2009-11-19 03:33:59Z horino $
*/
#include "stdafx.h"

#define MATERIAL_CPP

#include "utilities.h"
#include "mesh/material.h"
using namespace indi;

/*!
\brief oCif[^쐬܂
*/
u32 mesh::ColorOrTexture::Read(const u8 *data)
{
	const u8 *p(data);

	//< eNX`tO
	_texture_flag = *reinterpret_cast<const u32 *>(p); p+= sizeof(_texture_flag);
	//< eNX`CfbNX or J[
	if(_texture_flag) {
		_texture_index = *reinterpret_cast<const u32 *>(p); p+= sizeof(_texture_index);
	} else 
		p += _color.Read(p);

	return p - data;
}

/*!
\brief oCif[^쐬܂
*/
u32 mesh::Constant::Read(const u8 *data)
{
	const u8 *p(data);

	p += _emission.Read(p);
	p += _reflective.Read(p);
	p += _transparent.Read(p);
	_reflectivity = ToF32(*reinterpret_cast<const u32 *>(p)); p += sizeof(_reflectivity);
	_transparency = ToF32(*reinterpret_cast<const u32 *>(p)); p += sizeof(_transparency);
	_refraction = ToF32(*reinterpret_cast<const u32 *>(p));   p += sizeof(_refraction);

	return p - data;
}

/*!
\brief oCif[^쐬܂
*/
u32 mesh::Lambert::Read(const u8 *data)
{
	const u8 *p(data);

	p += _emission.Read(p);
	p += _ambient.Read(p);
	p += _diffuse.Read(p);
	p += _reflective.Read(p);
	p += _transparent.Read(p);
	_reflectivity = ToF32(*reinterpret_cast<const u32 *>(p)); p += sizeof(_reflectivity);
	_transparency = ToF32(*reinterpret_cast<const u32 *>(p)); p += sizeof(_transparency);
	_refraction = ToF32(*reinterpret_cast<const u32 *>(p));   p += sizeof(_refraction);

	return p - data;
}

/*!
\brief oCif[^쐬܂
*/
u32 mesh::Phong::Read(const u8 *data)
{
	const u8 *p(data);

	p += _emission.Read(p);
	p += _ambient.Read(p);
	p += _diffuse.Read(p);
	p += _specular.Read(p);
	p += _reflective.Read(p);
	p += _transparent.Read(p);
	_reflectivity = ToF32(*reinterpret_cast<const u32 *>(p)); p += sizeof(_reflectivity);
	_transparency = ToF32(*reinterpret_cast<const u32 *>(p)); p += sizeof(_transparency);
	_refraction = ToF32(*reinterpret_cast<const u32 *>(p));   p += sizeof(_refraction);
	_shininess = ToF32(*reinterpret_cast<const u32 *>(p));    p += sizeof(_shininess);

	return p - data;
}

/*!
\brief oCif[^쐬܂
*/
u32 mesh::Common::Read(const u8 *data)
{
	const u8 *p(data);

	//< VF[_̎ނ擾
	u32 shader(*reinterpret_cast<const u32 *>(p)); p+= sizeof(shader);

	//< VF[_̍쐬iVF[_ʁj
	switch(shader) {
		case mesh::ConstantShader:
			_shader = ConstantCreator(); 
			break;
		case mesh::LambertShader: 
			_shader = LambertCreator(); 
			break;
		case mesh::PhongShader: 
			_shader = PhongCreator();
			break;
		case mesh::BlinnShader: 
			_shader = BlinnCreator(); 
			break;
		default:
#ifdef _DEBUG
			ASSERT(0); //< m̃VF[_
#endif
			break;
	}
	p += _shader->Read(p);

	return p - data;
}

/*!
\brief oCif[^쐬܂
*/
u32 mesh::Material::Read(const u8 *data)
{
	const u8 *p(data);

	//< vt@Cނ̎擾
	u32 prof(*reinterpret_cast<const u32 *>(p)); p+= sizeof(prof);

	//< vt@C̍쐬ivt@Cʁj
	switch(prof){
		case mesh::ProfileCommon:
			_profile = CommonCreator();
			p += _profile->Read(p);
			break;
		default:
#ifdef _DEBUG
			ASSERT(0); //< _łprofile_common݂̂̑Ή
#endif
			break;
	}

	return p - data;
}

