/*!
$Id: triangle.cpp 629 2009-12-21 04:06:49Z horino $
*/
#include "stdafx.h"

#define TRIANGLE_CPP

#include "utilities.h"
#include "mesh/triangle.h"
#include "mesh/mesh_define.h"
#include "graphics/vertexbuffer.h"
#include "utilities.h"
using namespace indi;

/*!
\brief fXgN^
*/
mesh::Triangle::~Triangle() 
{
	switch(_primitive_type) {
		case mesh::Position: ReleaseVertices<graphics::VertexPosition>(); break;

		case mesh::Position | mesh::Normal: ReleaseVertices<graphics::VertexPositionNormal>(); break;
		case mesh::Position | mesh::Color: ReleaseVertices<graphics::VertexPositionColor>(); break;
		case mesh::Position | mesh::Texture: ReleaseVertices<graphics::VertexPositionTexture>(); break;

		case mesh::Position | mesh::Normal | mesh::Color: ReleaseVertices<graphics::VertexPositionNormalColor>(); break;
		case mesh::Position | mesh::Normal | mesh::Texture: ReleaseVertices<graphics::VertexPositionNormalTexture>(); break;

		case mesh::Position | mesh::Normal | mesh::Color | mesh::Texture: ReleaseVertices<graphics::VertexPositionNormalColorTexture>(); break;
		case mesh::Position | mesh::Normal | mesh::Index | mesh::Weight: ReleaseVertices<graphics::VertexPositionNormalIndexWeight>(); break;

		case mesh::Position | mesh::Normal | mesh::Texture | mesh::Index | mesh::Weight: ReleaseVertices<graphics::VertexPositionNormalTextureIndexWeight>(); break;

		//case mesh::Position | mesh::Normal | mesh::Texture | mesh::Index | mesh::Weight | mesh::Index2 | mesh::Weight2: ReleaseVertices<graphics::VertexPositionNormalTextureIndexWeightIndex2Weight2>(); break;
	}
}

/*!
\brief o[ebNXf[^ƃo[ebNXobt@̍쐬
\note o[ebNXf[^͎̂ĂĂ܂ĂǂA͈ꉞĂĂiNXȂǂŕKvɂȂ邩Hj
*/
template<typename T> u32 mesh::Triangle::CreateVertices(const u8 *data)
{
	const u8 *p(data);

	T *v = new T [_vertex_num];

	u32 size(sizeof(*v) * _vertex_num);
	Memcpy(reinterpret_cast<void *>(v), p, size); p+= size;

	_vertex_buffer = new graphics::VertexBuffer<T>(graphics::TRIANGLES, v, _vertex_num);
	_vertices = reinterpret_cast<void *>(v);

	return p - data;
}

/*!
\brief o[ebNXf[^ƃo[ebNXobt@̊J
*/
template<typename T> void mesh::Triangle::ReleaseVertices()
{
	if(_vertices){
		delete [] reinterpret_cast<T *>(_vertices);
		_vertices = 0;
	}

	if(_vertex_buffer) 
		delete _vertex_buffer;
}


/*!
\brief oCif[^쐬܂
*/
u32 mesh::Triangle::Read(const u8 *data)
{
	const u8 *p(data);

	//< }eACfbNX擾
	_material_index = *reinterpret_cast<const u32 *>(p); p += sizeof(_material_index);

	//< v~eBu^Cv擾
	_primitive_type = *reinterpret_cast<const u32 *>(p); p += sizeof(_primitive_type);

	//< o[ebNX
	_vertex_num = *reinterpret_cast<const u32 *>(p); p += sizeof(_vertex_num);

	//< o[ebNX̎擾ƃo[ebNXobt@̍쐬
	if(_vertex_num){
		switch(_primitive_type) {
	    case mesh::Position: p += CreateVertices<graphics::VertexPosition>(p); break;

		case mesh::Position | mesh::Normal:	p += CreateVertices<graphics::VertexPositionNormal>(p);	break;
		case mesh::Position | mesh::Color: p += CreateVertices<graphics::VertexPositionColor>(p); break;
		case mesh::Position | mesh::Texture: p += CreateVertices<graphics::VertexPositionTexture>(p); break;

		case mesh::Position | mesh::Normal | mesh::Color: p += CreateVertices<graphics::VertexPositionNormalColor>(p); break;
		case mesh::Position | mesh::Normal | mesh::Texture: p += CreateVertices<graphics::VertexPositionNormalTexture>(p); break;

		case mesh::Position | mesh::Normal | mesh::Color | mesh::Texture: p += CreateVertices<graphics::VertexPositionNormalColorTexture>(p); break;
		case mesh::Position | mesh::Normal | mesh::Index | mesh::Weight: p += CreateVertices<graphics::VertexPositionNormalIndexWeight>(p); break;

		case mesh::Position | mesh::Normal | mesh::Texture | mesh::Index | mesh::Weight: p += CreateVertices<graphics::VertexPositionNormalTextureIndexWeight>(p); break;

		//case mesh::Position | mesh::Normal | mesh::Texture | mesh::Index | mesh::Weight | mesh::Index2 | mesh::Weight2: p += CreateVertices<graphics::VertexPositionNormalTextureIndexWeightIndex2Weight2>(p); break;

		default:
#ifdef _DEBUG
			ASSERT(0); //< ΉAǉĂB
#endif
			break;
		}
	}

	return p - data;
}

/*!
\brief oCif[^쐬܂
*/
u32 mesh::TriangleAABB::Read(const u8 *data)
{
	u32 ret(mesh::Triangle::Read(data));

	if(_vertex_num){
		switch(_primitive_type) {
	    case mesh::Position: 
			{
				graphics::VertexPosition *p(reinterpret_cast<graphics::VertexPosition *>(_vertices));
				for(u32 i=0;i<_vertex_num;++i, ++p) 
					*this += Vector3(p->position[0], p->position[1], p->position[2]);
			}
			break;

		case mesh::Position | mesh::Normal:	
			{
				graphics::VertexPositionNormal *p(reinterpret_cast<graphics::VertexPositionNormal *>(_vertices));
				for(u32 i=0;i<_vertex_num;++i, ++p) 
					*this += Vector3(p->position[0], p->position[1], p->position[2]);
			}
			break;
		case mesh::Position | mesh::Color: 
			{
				graphics::VertexPositionColor *p(reinterpret_cast<graphics::VertexPositionColor *>(_vertices));
				for(u32 i=0;i<_vertex_num;++i, ++p) 
					*this += Vector3(p->position[0], p->position[1], p->position[2]);
			}
			break;
		case mesh::Position | mesh::Texture:
			{
				graphics::VertexPositionTexture *p(reinterpret_cast<graphics::VertexPositionTexture *>(_vertices));
				for(u32 i=0;i<_vertex_num;++i, ++p) 
					*this += Vector3(p->position[0], p->position[1], p->position[2]);
			}
			break;

		case mesh::Position | mesh::Normal | mesh::Color:
			{
				graphics::VertexPositionNormalColor *p(reinterpret_cast<graphics::VertexPositionNormalColor *>(_vertices));
				for(u32 i=0;i<_vertex_num;++i, ++p) 
					*this += Vector3(p->position[0], p->position[1], p->position[2]);
			}
			break;
		case mesh::Position | mesh::Normal | mesh::Texture:
			{
				graphics::VertexPositionNormalTexture *p(reinterpret_cast<graphics::VertexPositionNormalTexture *>(_vertices));
				for(u32 i=0;i<_vertex_num;++i, ++p) 
					*this += Vector3(p->position[0], p->position[1], p->position[2]);
			}
			break;

		case mesh::Position | mesh::Normal | mesh::Color | mesh::Texture:
			{
				graphics::VertexPositionNormalColorTexture *p(reinterpret_cast<graphics::VertexPositionNormalColorTexture *>(_vertices));
				for(u32 i=0;i<_vertex_num;++i, ++p) 
					*this += Vector3(p->position[0], p->position[1], p->position[2]);
			}
			break;
		case mesh::Position | mesh::Normal | mesh::Index | mesh::Weight: 
			{
				graphics::VertexPositionNormalIndexWeight *p(reinterpret_cast<graphics::VertexPositionNormalIndexWeight *>(_vertices));
				for(u32 i=0;i<_vertex_num;++i, ++p) 
					*this += Vector3(p->position[0], p->position[1], p->position[2]);
			}
			break;

		case mesh::Position | mesh::Normal | mesh::Texture | mesh::Index | mesh::Weight: 
			{
				graphics::VertexPositionNormalTextureIndexWeight *p(reinterpret_cast<graphics::VertexPositionNormalTextureIndexWeight *>(_vertices));
				for(u32 i=0;i<_vertex_num;++i, ++p) 
					*this += Vector3(p->position[0], p->position[1], p->position[2]);
			}
			break;

		//case mesh::Position | mesh::Normal | mesh::Texture | mesh::Index | mesh::Weight | mesh::Index2 | mesh::Weight2: 
			//break;

		default:
#ifdef _DEBUG
			ASSERT(0); //< ΉAǉĂB
#endif
			break;
		}
	}

	return ret;
}