/*!
$Id: cloth.cpp 642 2009-12-28 05:16:12Z horino $
*/
#include "stdafx.h"

#define CLOTH_CPP

#include "physics/cloth.h"
#include "physics/particle.h"
#include "physics/spring.h"
using namespace indi;

/*!
\brief NX̃fXgN^
*/
physics::Cloth::~Cloth() 
{
	if(_particles) {
		for(u32 i=0;i<_nparticle;++i)
			if(_particles[i])
				delete _particles[i];
		delete [] _particles;
		_particles = 0;
	}
	if(_reset_positions) { delete [] _reset_positions; _reset_positions = 0; }
	if(_springs) {
		for(u32 i=0;i<_nspring;++i)
			if(_springs[i])
				delete _springs[i];
		delete [] _springs;
		_springs = 0;
	}
}

/*!
\brief _Ƃ΂˂mۂ܂
\param 
\param c
*/
void physics::Cloth::Allocate(u32 w, u32 h)
{
	_particles = new Particle * [(_nparticle = (_width = w) * (_height = h))];
	_reset_positions = new Vector3 [_nparticle];
	Memzero(_particles, sizeof(_particles[0]) * _nparticle);

	_springs = new Spring * [(_nspring = GetStructSpringNum() + GetShearSpringNum() + GetBendSpringNum())];
	Memzero(_springs, sizeof(_springs[0]) * _nspring);
}

/*!
\brief NX̃p[eBN쐬܂
\param NX̎
\param 
\param c
\param ep[eBNʒu̔z
*/
void physics::Cloth::CreateParticles(f32 mass, f32 dx, f32 dy)
{
	f32 m(mass / static_cast<f32>(_nparticle));
	u32 c(0);
	f32 ofsx(static_cast<f32>(_width) * dx * 0.5f);
	f32 ofsy(static_cast<f32>(_height) * dy * 0.5f);
	for(u32 j=0;j<_height;++j)
		for(u32 i=0;i<_width;++i) {
			Vector3 pos(static_cast<f32>(i) * dx - ofsx, ofsy - static_cast<f32>(j) * dy, 0.0f);
			_particles[c] = new Particle(m, pos);
			_reset_positions[c] = pos;
			++c;
		}

#ifdef _DEBUG
	cout << "cloth : " << _nparticle << " particles" << endl;
#endif
}

/*!
\brief NX̃p[eBN쐬܂
\param NX̎
\param 
\param c
\param ep[eBNʒu̔z
*/
void physics::Cloth::CreateParticles(f32 mass, const Vector3 *positions)
{
	f32 m(mass / static_cast<f32>(_nparticle));
	for(u32 i=0;i<_nparticle;++i) {
		_particles[i] = new Particle(m, positions[i]);
		_reset_positions[i] = positions[i];
	}

#ifdef _DEBUG
	cout << "cloth : " << _nparticle << " particles" << endl;
#endif
}

/*!
\brief NX̂΂˂쐬܂
\param ΂˂̋x
\param ΂˂̌
*/
void physics::Cloth::CreateSprings(f32 struct_stiffness, f32 struct_dumping, f32 shear_stiffness, f32 shear_dumping, f32 bend_stiffness, f32 bend_dumping)
{
	u32 w1(_width - 1);
	u32 h1(_height - 1);

	u32 c(0);

	//< \΂
	if(!_flags.Test(NoStructSpring)) {
		for(u32 j=0;j<_height;++j) {
			for(u32 i=0;i<w1;++i) {
				u32 tmp(j * _width + i);
				_springs[c++] = new Spring(struct_stiffness, struct_dumping, _particles[tmp], _particles[tmp + 1]);
			}
		}
		for(u32 i=0;i<_width;++i) {
			for(u32 j=0;j<h1;++j) {
				_springs[c++] = new Spring(struct_stiffness, struct_dumping, _particles[j * _width + i], _particles[(j + 1) * _width + i]);
			}
		}
	}

	//< f΂
	if(!_flags.Test(NoShearSpring)) {
		for(u32 j=0;j<h1;++j) {
			for(u32 i=0;i<w1;++i) {
				u32 tmp0(j * _width + i);
				u32 tmp1((j + 1) * _width + i);
				_springs[c++] = new Spring(shear_stiffness, shear_dumping, _particles[tmp0], _particles[tmp1 + 1]);
				_springs[c++] = new Spring(shear_stiffness, shear_dumping, _particles[tmp1], _particles[tmp0 + 1]);
			}
		}
	}

	//< pȂ΂
	if(!_flags.Test(NoBendSpring)) {
		u32 w2(_width - 2);
		u32 h2(_height - 2);	
		for(u32 j=0;j<_height;++j) {
			for(u32 i=0;i<w2;++i) {
				u32 tmp(j * _width + i);
				_springs[c++] = new Spring(bend_stiffness, bend_dumping, _particles[tmp], _particles[tmp + 2]);
			}
		}
		for(u32 i=0;i<_width;++i) {
			for(u32 j=0;j<h2;++j) {
				_springs[c++] = new Spring(bend_stiffness, bend_dumping, _particles[j * _width + i], _particles[(j + 2) * _width + i]);
			}
		}
	}

#ifdef _DEBUG
	cout << "cloth : " << _nspring << " springs" << endl;
#endif

	ASSERT(c == _nspring);
}

/*!
\brief NXZbg܂
*/
void physics::Cloth::Reset()
{
	for(u32 i=0;i<_nparticle;++i) {
		_particles[i]->SetPosition(_reset_positions[i]);
		_particles[i]->SetAcceleration(Vector3::Zero());
		_particles[i]->SetVelocity(Vector3::Zero());
	}
}