/*!
$Id: rendertarget.h 536 2009-11-20 02:57:07Z horino $
*/
#ifndef RENDERTARGET_H
#define RENDERTARGET_H

#include <iostream>
using namespace std;

#include <PSGL/psgl.h>
#include <PSGL/psglu.h>

#include "indi.h"
#include "graphics/texture.h"

namespace indi
{
	namespace graphics
	{
		/*!
		\brief _^[Qbgx[XNX
		*/
		class RenderTargetBase 
		{
		public:
			RenderTargetBase() {}
			virtual ~RenderTargetBase(){}

			virtual BOOL IsRenderTarget() const = 0;
			virtual BOOL IsDepthTarget() const = 0;
			virtual u32 GetWidth() const = 0;
			virtual u32 GetHeight() const = 0;
		};

		/*!
		\brief _[^[QbgNX
		*/
		class RenderTarget : public RenderTargetBase
		{
		public:
			RenderTarget(RenderTexture *r) : RenderTargetBase(), _resolve(r) {}
			virtual ~RenderTarget(){}

			//< RXgN^ł͎g܂
			virtual BOOL IsRenderTarget() const { return TRUE; }
			virtual BOOL IsDepthTarget() const { return FALSE; }
			virtual u32 GetWidth() const { ASSERT(_resolve); return _resolve->GetWidth(); }
			virtual u32 GetHeight() const { ASSERT(_resolve); return _resolve->GetHeight(); }

			RenderTexture *GetResolveTexture() { return _resolve; }

		private:
			RenderTexture *_resolve;
		};

		/*!
		\brief fvX_[^[QbgNX
		*/
		class DepthTarget : public RenderTargetBase
		{
		public:
			DepthTarget(u32 w, u32 h) : RenderTargetBase(), _depth_target(0), _width(w), _height(h) { CreateDepthTarget(); }
			virtual ~DepthTarget() { if(_depth_target) { glDeleteRenderbuffersOES(1, &_depth_target); _depth_target = 0; } }

			//< RXgN^ł͎g܂
			virtual BOOL IsRenderTarget() const { return FALSE; }
			virtual BOOL IsDepthTarget() const { return TRUE; }
			virtual u32 GetWidth() const { return _width; }
			virtual u32 GetHeight() const { return _height; }

			u32 GetDepthTarget() const { return _depth_target; }

		private:
			void CreateDepthTarget();
			u32 _depth_target;
			u32 _width, _height;
		};
	};
};

#endif // RENDERTARGET_H