/*!
$Id: shader.h 610 2009-12-14 07:23:32Z horino $
*/
#ifndef SHADER_H
#define SHADER_H

#include "indi.h"

#ifdef _DEBUG
#include <iostream>
using namespace std;
#endif

#include <PSGL/psgl.h>

#include "file/file.h"

#ifndef CG_CHECKER
#ifdef _DEBUG
#define CG_CHECKER() { CGerror e(cgGetError()); if(CG_NO_ERROR != e) { cout << __FILE__ << ":" << __LINE__ << " : " << cgGetErrorString(e) << endl; ASSERT(0); } }
#else
#define CG_CHECKER() 
#endif
#endif

namespace indi
{
	class Vector2;
	class Vector3;
	class Vector4;
	class Matrix44;

	namespace graphics
	{
		class TextureBase;

		namespace shader
		{
			/*!
			\brief VF[_VXeǗNX
			*/
			class System
			{
			private:
				System(){}

			public:
				static void Initialize() { _context = cgCreateContext(); CG_CHECKER() }
				static void Finalize() { cgDestroyContext(_context); CG_CHECKER() }
				static CGcontext &GetContext() { return _context; }

				/*!
				\brief XR[v𔲂ƂɎIɃt@CiCY֗NX
				*/
				class AutoInit 
				{
				public:
					AutoInit(){ Initialize(); }
					~AutoInit(){ Finalize(); }
				};

			private:
				static CGcontext _context;
			};

			/*!
			\brief VF[_ɓnp[^ǗNX
			*/
			class Parameter
			{
			public:
				Parameter() : _parameter(CGparameter()) {}
				Parameter(CGparameter p) : _parameter(p) {}
				~Parameter() {}

				operator CGparameter&() { return _parameter; }
				void operator=(const CGparameter &rhs) { _parameter = rhs;}
				void operator=(f32 rhs);
				void operator=(const Vector2 &rhs);
				void operator=(const Vector3 &rhs);
				void operator=(const Vector4 &rhs);
				void operator=(const Matrix44 &lhs);
				void operator=(const graphics::TextureBase &lhs) { *this = &lhs; }
				void operator=(const graphics::TextureBase *lhs);

				//< eNX`p[^𖳌ɂꍇɎg܂
				void Disable() { 
					cgGLSetTextureParameter(_parameter, 0); CG_CHECKER();
					cgGLDisableTextureParameter(_parameter); CG_CHECKER();
				} 

				/*!
				\brief XR[v𔲂ƂɎIDisable()R[֗NX
				*/
				class AutoBeginTexture
				{
				public:
					//AutoBeginTexture(Parameter &p, const TextureBase &t) : _parameter(p) { _parameter = t; }
					//AutoBeginTexture(Parameter &p, const TextureBase *t) : _parameter(p) { _parameter = t; }
					//AutoBeginTexture(Parameter &p, GLuint tid) : _parameter(p) { _parameter = tid; } //< PS3̂
					AutoBeginTexture(Parameter p, const TextureBase &t) : _parameter(p) { _parameter = t; }
					AutoBeginTexture(Parameter p, const TextureBase *t) : _parameter(p) { _parameter = t; }
					AutoBeginTexture(Parameter p, GLuint tid) : _parameter(p) { _parameter = tid; } //< PS3̂
					~AutoBeginTexture(){ _parameter.Disable(); }

				private:
					Parameter &_parameter;
				};

			private:
				CGparameter _parameter;
			};

			/*!
			\brief VF[_vOǗNX
			*/
			class Program
			{
			public:
				Program() {}
				virtual ~Program() { cgDestroyProgram(_program); CG_CHECKER() }

				operator CGprogram&() { return _program; }
				operator CGprofile&() { return _profile; }
				void operator=(const CGprogram &rhs) { _program = rhs;}
				void operator=(const CGprofile &rhs) { _profile = rhs;}

				void Begin(){
					cgGLBindProgram(_program); CG_CHECKER();
					IsVertexProgram() ? (_current_vs_program = this) : (_current_ps_program = this);
					cgGLEnableProfile(_profile); CG_CHECKER();
				}
				void End(){
					cgGLUnbindProgram(_profile); CG_CHECKER();
					cgGLDisableProfile(_profile); CG_CHECKER();
				}

				virtual Parameter GetParameter(const char *pname) { 
					Parameter p(cgGetNamedParameter(_program, pname)); 
					CG_CHECKER()
					return p;
				}

				virtual BOOL IsVertexProgram() const = 0;
				virtual BOOL IsPixelProgram() const = 0;

				/*!
				\brief XR[v𔲂ƂɎIEnd()R[֗NX
				*/
				class AutoBegin
				{
				public:
					AutoBegin(Program &p) : _program(p) { _program.Begin(); }
					~AutoBegin(){ _program.End(); }
				private:
					Program &_program;
				};

			protected:
				CGprogram _program;
				CGprofile _profile;

			private:
				static indi::graphics::shader::Program *_current_vs_program;
				static indi::graphics::shader::Program *_current_ps_program;
			public:
				static indi::graphics::shader::Program *GetCurrentVS() { return _current_vs_program; }
				static indi::graphics::shader::Program *GetCurrentPS() { return _current_ps_program; }
			};

			/*!
			\brief VF[_vO(t@C񓯊ǂݍ)
			*/
			class ProgramFile : public Program, public file::File
			{
			public:
				ProgramFile() : Program(), file::File() {}
				ProgramFile(const char *path) : Program(), file::File(path) {}
				virtual ~ProgramFile() {}
			};	

			/*!
			\brief o[ebNXVF[_
			*/
			class VertexProgram : public ProgramFile
			{
			public:
				VertexProgram() : ProgramFile() {}
				VertexProgram(const char *path) : ProgramFile(path) {}
				virtual ~VertexProgram() {}

				virtual void OnLoaded()	{
					_profile = cgGLGetLatestProfile(CG_GL_VERTEX); CG_CHECKER()
					_program = cgCreateProgram(System::GetContext(), CG_BINARY, reinterpret_cast<const char *>(GetData()), _profile, NULL, NULL); CG_CHECKER()
				}

				BOOL IsVertexProgram() const { return TRUE; }
				BOOL IsPixelProgram() const { return  FALSE; }
			};

			/*!
			\brief sNZVF[_
			*/
			class PixelProgram : public ProgramFile
			{
			public:
				PixelProgram() : ProgramFile() {}
				PixelProgram(const char *path) : ProgramFile(path) {}
				virtual ~PixelProgram() {}

				virtual void OnLoaded() {
					_profile = cgGLGetLatestProfile(CG_GL_FRAGMENT); CG_CHECKER()
					_program = cgCreateProgram(System::GetContext(), CG_BINARY, reinterpret_cast<const char *>(GetData()), _profile, NULL, NULL); CG_CHECKER()
				}

				BOOL IsVertexProgram() const { return FALSE; }
				BOOL IsPixelProgram() const { return  TRUE; }
			};
		}
	}
}

#endif // SHADER_H