/*!
$Id: texture.h 635 2009-12-22 08:23:59Z horino $
*/
#ifndef TEXTURE_H
#define TEXTURE_H

#ifdef _DEBUG
#include <iostream>
using namespace std;
#endif

#include <PSGL/psgl.h>
#include <PSGL/psglu.h>

#include "indi.h"
#include "utilities.h"
#include "graphics/vertexbuffer_specialization.h"
#include "itl/bitset.h"
#include "quaternion_transform.h"

#ifndef RENDER_TARGET_SIZE_CHECKER
#ifdef _DEBUG
#define RENDER_TARGET_SIZE_CHECKER(__w, __h) { GLint max_size; glGetIntegerv(GL_MAX_RENDERBUFFER_SIZE_OES, &max_size); ASSERT(static_cast<s32>(__w) <= max_size && static_cast<s32>(__h) <= max_size); }
#else
#define RENDER_TARGET_SIZE_CHECKER(__w, __h) 
#endif
#endif

namespace indi
{
	namespace graphics
	{
		/*!
		\brief eNX`x[XNX
		*/
		class TextureBase 
		{
		public:
			friend class FrameBuffer;

			enum Filter {
#define TEXFILTER(id, gl) id = gl,
#include "texture.h"
#undef TEXFILTER
			};
			enum Wrap {
#define TEXWRAP(id, gl) id = gl,
#include "texture.h"
#undef TEXWRAP
			};
			enum ColorFormat {
#define COLOR_FORMAT(id, gl) id = gl,
#include "texture.h"
#undef COLOR_FORMAT
			};

			TextureBase(u32 w =0, u32 h = 0, ColorFormat fmt = TextureFormat()) : _id(0), _width(w), _height(h), _format(fmt) { glGenTextures(1, &_id);	}
			TextureBase(ColorFormat fmt) : _id(0), _width(0), _height(0), _format(fmt) { glGenTextures(1, &_id); }
			virtual ~TextureBase() { if(_id) { glDeleteTextures(1, &_id); _id =0; } }

			u32 GetWidth() const { return _width; }
			u32 GetHeight() const { return _height; }
			u32 GetId() const { return _id; }

			virtual BOOL IsCubeTexture() const = 0;
			virtual BOOL IsRenderTexture() const = 0;
			virtual BOOL IsAnimationTexture() const = 0;

			virtual void AddMipmap(u32 sur, u32 mip, u32 w, u32 h, const u8 *data) = 0;
			virtual u32 Read(const u8 *data);
			virtual void MipmapSettings(BOOL b) {}

			ColorFormat Format() { return _format; }

			static ColorFormat TextureFormat() { return RGBA; }
			static ColorFormat RenderTextureFormat() { return ARGB; }
			static ColorFormat DepthTextureFormat() { return SINGLE; }
			static ColorFormat AnimationTextureFormat() { return SINGLE4; } //< PS3łhalf float_eNX`̓T|[gĂȂ

		protected:
			u32 _id;
			u32 _width;
			u32 _height;
			ColorFormat _format;
		};

		/*!
		\brief eNX`NX
		*/
		class Texture : public TextureBase
		{
		public:
			/*!
			\brief RXgN^
			*/
			Texture(u32 w = 0, u32 h = 0, ColorFormat fmt = TextureFormat()) : TextureBase(w, h, fmt) {				
				glBindTexture(GL_TEXTURE_2D, _id);{
					glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_ALLOCATION_HINT_SCE, GL_TEXTURE_TILED_GPU_SCE);
					glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
					glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);	
					glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
					glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
				} glBindTexture(GL_TEXTURE_2D, 0);
			}
			Texture(ColorFormat fmt) : TextureBase(fmt) { 
				glBindTexture(GL_TEXTURE_2D, _id);{
					glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_ALLOCATION_HINT_SCE, GL_TEXTURE_TILED_GPU_SCE);
					glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
					glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);	
					glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
					glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
				} glBindTexture(GL_TEXTURE_2D, 0);
			}
			virtual ~Texture() {}

			/*!
			\brief ~bv}bvǉ܂
			\param T[tBXԍ(ɂOw)
			\param ~bv}bvx
			\param 
			\param 
			\param f[^
			*/
			void AddMipmap(u32 sur, u32 mip, u32 w, u32 h, const u8 *data) {
#ifdef _DEBUG
				ASSERT(w); ASSERT(h);
#endif
				glBindTexture(GL_TEXTURE_2D, _id);{
					glTexImage2D(GL_TEXTURE_2D, mip, 4, w, h, 0, _format, GL_UNSIGNED_INT_8_8_8_8, data);
				} glBindTexture(GL_TEXTURE_2D, 0);
				GL_ERROR_CHECK;
			}

			void MipmapSettings(BOOL b) { 
				glBindTexture(GL_TEXTURE_2D, _id);{
					if(b)
						glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR); 
					else
						glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);  
				} glBindTexture(GL_TEXTURE_2D, 0);
				GL_ERROR_CHECK;
			}

			virtual BOOL IsCubeTexture() const { return FALSE; }
			virtual BOOL IsRenderTexture() const { return FALSE; }
			virtual BOOL IsAnimationTexture() const { return FALSE; }
		};

		/*!
		\brief _OpeNX`NX
		\note 
		GL_ARGB_SCE
		GL_BGR(for 16bpp)
		GL_BGRA(for 16bpp)
		GL_RGBA16F_ARB
		GL_RGBA32F_ARB
		GL_DEPTH_COMPONENT
		*/
		class RenderTexture : public Texture
		{
		public:
			RenderTexture(u32 w, u32 h, ColorFormat fmt = RenderTextureFormat()) : Texture(w, h, fmt) {
				RENDER_TARGET_SIZE_CHECKER(w, h);
				glBindTexture(GL_TEXTURE_2D, _id);{
					glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
					glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);	
					glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
					glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
					glTexImage2D(GL_TEXTURE_2D, 0, _format, GetWidth(), GetHeight(), 0, GL_RGBA, GL_UNSIGNED_INT_8_8_8_8, 0);
				} glBindTexture(GL_TEXTURE_2D, 0);
				GL_ERROR_CHECK;
			}
			virtual ~RenderTexture() {}

			virtual BOOL IsRenderTexture() const { return TRUE; }
		};

		/*!
		\brief fvX_OpeNX`NX
		*/
		class DepthTexture : public RenderTexture
		{
		public:
			DepthTexture(u32 w, u32 h, ColorFormat fmt = DepthTextureFormat()) : RenderTexture(w, h, fmt) {}
			virtual ~DepthTexture() {}
		};

		/*!
		\brief Aj[VeNX`NX
		*/
		class AnimationTexture : public Texture
		{
		public:
			AnimationTexture(ColorFormat fmt = AnimationTextureFormat()) : Texture(fmt) {
				glBindTexture(GL_TEXTURE_2D, _id);{
					glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
					glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);	
					glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
					glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
					glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_FROM_VERTEX_PROGRAM_SCE, GL_TRUE); //< _eNX`ł͂dvI
				} glBindTexture(GL_TEXTURE_2D, 0);
				GL_ERROR_CHECK;
			}
			virtual ~AnimationTexture() {}

			virtual u32 Read(const u8 *data);

			QuaternionTransform GetTransform(u32 x, u32 y, const u8 *data) const;

			/*!
			\brief ~bv}bvǉ܂
			\param T[tBXԍ
			\param ~bv}bvx
			\param 
			\param 
			\param f[^
			*/
			void AddMipmap(u32 sur, u32 mip, u32 w, u32 h, const u8 *data){
				glBindTexture(GL_TEXTURE_2D, _id);{					
					glTexImage2D(GL_TEXTURE_2D, mip, _format, w, h, 0, GL_RGBA, _format == SINGLE4 ? GL_FLOAT : GL_HALF_FLOAT_ARB, data);
				} glBindTexture(GL_TEXTURE_2D, 0);
				GL_ERROR_CHECK;
			}

			virtual BOOL IsAnimationTexture() const { return TRUE; }
		};

		/*!
		\brief L[ueNX`NX
		*/
		class CubeTexture : public TextureBase
		{
		public:
			/*!
			\brief RXgN^
			*/
			CubeTexture(u32 w = 0, u32 h = 0, ColorFormat fmt = TextureFormat()) : TextureBase(w, h, fmt) {
				glBindTexture(GL_TEXTURE_CUBE_MAP, _id);{
					glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_ALLOCATION_HINT_SCE,GL_TEXTURE_TILED_GPU_SCE);
					glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_MIN_FILTER,GL_LINEAR);
					glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_MAG_FILTER,GL_LINEAR);
					glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_S, GL_MIRRORED_REPEAT); 
					glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_T, GL_MIRRORED_REPEAT); 
				} glBindTexture(GL_TEXTURE_CUBE_MAP, 0);
				GL_ERROR_CHECK;
			}

			/*!
			\brief ~bv}bvǉ܂
			\param T[tBXԍ
			\param ~bv}bvx
			\param 
			\param 
			\param f[^
			*/
			void AddMipmap(u32 sur, u32 mip, u32 w, u32 h, const u8 *data){
				glBindTexture(GL_TEXTURE_CUBE_MAP, _id);{
					glTexImage2D(GL_TEXTURE_CUBE_MAP_POSITIVE_X + sur, mip, _format, w, h, 0, _format, GL_UNSIGNED_INT_8_8_8_8, data);
				} glBindTexture(GL_TEXTURE_CUBE_MAP, 0);
				GL_ERROR_CHECK;
			}
			virtual ~CubeTexture() {}

			void MipmapSettings(BOOL b) { 
				glBindTexture(GL_TEXTURE_CUBE_MAP, _id);{
					if(b)
						glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR); 
					else
						glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
				} glBindTexture(GL_TEXTURE_CUBE_MAP, 0);
				GL_ERROR_CHECK;
			}

			virtual BOOL IsCubeTexture() const { return TRUE; }
			virtual BOOL IsRenderTexture() const { return FALSE; }
			virtual BOOL IsAnimationTexture() const { return FALSE; }
		};
	};
};

#endif // TEXTURE_H

#ifdef TEXFILTER
TEXFILTER(NEAREST, GL_NEAREST)
TEXFILTER(LINEAR, GL_LINEAR)
#endif

#ifdef TEXWRAP
TEXWRAP(CLAMP, GL_CLAMP)
TEXWRAP(REPEAT, GL_REPEAT)
TEXWRAP(MIRROR, GL_MIRRORED_REPEAT)
#endif

#ifdef COLOR_FORMAT
#if 0 // TODO : fvXƃJ[ŃtH[}bgς
COLOR_FORMAT(ARGB, GL_ARGB_SCE)
COLOR_FORMAT(RGBA, GL_RGBA)
COLOR_FORMAT(SINGLE, GL_RGBA32F_ARB/*GL_ALPHA32F_ARB*/)
#else
COLOR_FORMAT(ARGB, GL_RGBA32F_ARB)
COLOR_FORMAT(RGBA, GL_RGBA)
COLOR_FORMAT(SINGLE, GL_RGBA32F_ARB)
COLOR_FORMAT(SINGLE4, GL_RGBA32F_ARB)
COLOR_FORMAT(HALF4, GL_RGBA16F_ARB) 
#endif
#endif