/*!
$Id: vertexbuffer.h 643 2009-12-28 05:59:38Z horino $
*/
#ifndef VERTEXBUFFER_H
#define VERTEXBUFFER_H

#include <iostream>
using namespace std;

#include <PSGL/psgl.h>
#include <PSGL/psglu.h>

#include "indi.h"
#include "utilities.h"
#include "graphics/vertexbuffer_specialization.h"
#include "graphics/shader/shader.h"

#ifndef GL_MATRIX_INDEX_ARRAY
#define GL_MATRIX_INDEX_ARRAY 0xffffffff
#endif

#ifndef GL_VERTEX_WEIGHT_ARRAY
#define GL_VERTEX_WEIGHT_ARRAY 0xfffffffe
#endif

namespace indi
{
	namespace graphics
	{
		enum PrimitiveType {
#define PRIMTYPE(id, d3d) id = d3d,
#include "vertexbuffer.h"
#undef PRIMTYPE
		};

		/*!
		\brief o[ebNXobt@x[XNX
		*/
		class VertexBufferBase 
		{
		public:
			VertexBufferBase() {}
			virtual ~VertexBufferBase() {}

			virtual void Begin() = 0;
			virtual void Draw() = 0;
			virtual void End() = 0;

			virtual void SetIndexWeight(shader::Program *prog, const char *index_name, const char *weight_name) = 0;

			/*!
			\brief XR[v𔲂ƂɎIEnd()R[֗NX
			*/
			class AutoBegin
			{
			public:
				AutoBegin(VertexBufferBase &vb) : _vb(vb){ _vb.Begin(); }
				~AutoBegin(){ _vb.End(); }
			private:
				VertexBufferBase &_vb;
			};
		};

		/*!
		\brief o[ebNXobt@NX
		\note ev[gNXA^̒`CreateDeclaration()֐̃ev[g邱Ƃ\łB
		({"graphics/vertexbuffer_specialization.h"̌^g܂j
		*/
		template<typename T>
		class VertexBuffer : public VertexBufferBase
		{
		public:
			struct Declaration
			{
				Declaration() : id(0), size(0), type(0), semantic(0), offset(0), weight_index_parameter(0), weight_index_data(0) {}
				Declaration(u32 siz, u32 typ, u32 sem, u32 ofs) : id(0), size(siz), type(typ), semantic(sem), offset(ofs), weight_index_parameter(0), weight_index_data(0) {}

				u32 id;
				u32 size;
				u32 type;
				u32 semantic;
				u32 offset;

				CGparameter weight_index_parameter;
				u8 *weight_index_data;
			};

			VertexBuffer(PrimitiveType prim=TRIANGLES) : VertexBufferBase(), _primtype(prim), _declaration(0), _ndeclaration(0), _nvertices(0) {}
			VertexBuffer(PrimitiveType prim, const T *data, u32 n) : VertexBufferBase(), _primtype(prim), _declaration(0), _ndeclaration(0), _nvertices(n) { 
				CreateDeclaration();
				CreateVertexBuffer(data, _nvertices);
			}
			~VertexBuffer(){
				for(u32 i = 0;i < _ndeclaration;++i) {
					if(_declaration[i].id) {
						glDeleteBuffers(1, &_declaration[i].id);
						_declaration[i].id = 0;
					}
					if(_declaration[i].weight_index_data) {
						delete [] _declaration[i].weight_index_data;
						_declaration[i].weight_index_data = 0;
					}
				}
				if(_declaration) { 
					delete [] _declaration;
					_declaration = 0;
					_ndeclaration = 0; 
				} 
			}

			void CreateDeclaration();

#if 0
			/*!
			\brief _f[^dstփ}bv
			\note ̌dst̃f[^āAς񂾂A}bvR[
			*/
			void begin(f32 *dst, u32 siz){
				glBindBuffer(GL_ARRAY_BUFFER, _declaration[0].id); {
					Memcpy(dst, static_cast<f32 *>(glMapBuffer(GL_ARRAY_BUFFER, GL_READ_WRITE)), siz);
					glUnmapBuffer(GL_ARRAY_BUFFER);
				} glBindBuffer(GL_ARRAY_BUFFER, 0);
			}
			/*!
			\brief A}bv
			*/
			void end(){
				f32 *pos[1024];
				glBindBuffer(GL_ARRAY_BUFFER, _declaration[0].id); {
					glUnmapBuffer(GL_ARRAY_BUFFER);
				} glBindBuffer(GL_ARRAY_BUFFER, 0); 
			}
#endif
			/*!
			\brief o[ebNXobt@쐬܂
			\param f[^ւ̃|C^
			\param f[^̌
			*/
			void CreateVertexBuffer(const T *data, u32 n){
				for(u32 i = 0;i < _ndeclaration;++i){
					switch(_declaration[i].semantic){
		            default:
						{
#ifdef _DEBUG
							ASSERT(_declaration[i].type == GL_FLOAT); //< Ƃ肠GL_FLOAT̑O
#endif
							u32 unit_size(sizeof(f32) * _declaration[i].size);

							u8 *p = new u8[unit_size  * n];
							for(u32 j = 0;j < n;++j) 
								Memcpy(p + unit_size * j, reinterpret_cast<const u8 *>(data) + _declaration[i].offset + sizeof(T) * j, unit_size);

							glGenBuffers(1, &_declaration[i].id);
							glBindBuffer(GL_ARRAY_BUFFER, _declaration[i].id); {
								glEnableClientState(_declaration[i].semantic); {
									glBufferData(GL_ARRAY_BUFFER, unit_size * n, p, GL_STATIC_DRAW);
								} glDisableClientState(_declaration[i].semantic);
							} glBindBuffer(GL_ARRAY_BUFFER, 0); 

							delete [] p;
						}
						break;
					case GL_MATRIX_INDEX_ARRAY:  
					case GL_VERTEX_WEIGHT_ARRAY:
						{
#ifdef _DEBUG
							ASSERT(_declaration[i].type == GL_FLOAT); //< Ƃ肠GL_FLOAT̑O
#endif
							u32 unit_size(sizeof(f32) * _declaration[i].size);

							_declaration[i].weight_index_data = new u8[unit_size  * n];
							for(u32 j = 0;j < n;++j) 
								Memcpy(_declaration[i].weight_index_data + unit_size * j, reinterpret_cast<const u8 *>(data) + _declaration[i].offset + sizeof(T) * j, unit_size);
						}
						break;
					}
				}
			}	
	 
		private:
			void SetShaderParameter(Declaration &d, shader::Program *prog, const char *name)
			{
				if(name == 0) 
					d.weight_index_parameter = 0;
				else if(!d.weight_index_parameter){
							
					d.weight_index_parameter = prog->GetParameter(name);
#ifdef _DEBUG
					ASSERT(d.weight_index_parameter); //< O܂
#endif
				}
			}

		public:
			/*!
			\brief o[ebNXobt@̃CfbNXAEGCgZ}eBNXVF[_֓n܂
			\param VF[_vO
			\param VF[_̃EGCg
			\param VF[_̃CfbNX
			*/
			void SetIndexWeight(shader::Program *prog, const char *index_name, const char *weight_name)
			{
				for(u32 i = 0;i < _ndeclaration;++i){
					switch(_declaration[i].semantic){
			        default: break;
					case GL_MATRIX_INDEX_ARRAY:
						SetShaderParameter(_declaration[i], prog, index_name);
						break;
					case GL_VERTEX_WEIGHT_ARRAY:
						SetShaderParameter(_declaration[i], prog, weight_name);
						break;
					}
				}								  	   				
			}

			/*!
			\brief `̑O
			*/
			void Begin(){
				for(u32 i = 0;i < _ndeclaration;++i){
					glBindBuffer(GL_ARRAY_BUFFER, _declaration[i].id);
					switch(_declaration[i].semantic){
		            case GL_VERTEX_ARRAY:        
						glEnableClientState(_declaration[i].semantic);
						glVertexPointer(_declaration[i].size, _declaration[i].type, 0, NULL);
						break;
					case GL_NORMAL_ARRAY:
						glEnableClientState(_declaration[i].semantic);
						glNormalPointer(_declaration[i].type, 0, NULL); 
						break;
					case GL_TEXTURE_COORD_ARRAY: 
						glEnableClientState(_declaration[i].semantic);
						glTexCoordPointer(_declaration[i].size, _declaration[i].type, 0, NULL); 
						break;
					case GL_COLOR_ARRAY:
						glEnableClientState(_declaration[i].semantic);
						glColorPointer(_declaration[i].size, _declaration[i].type, 0, NULL); 
						break;
					case GL_MATRIX_INDEX_ARRAY:  
						if(_declaration[i].weight_index_parameter){
							cgGLEnableClientState(_declaration[i].weight_index_parameter);
							cgGLSetParameterPointer(_declaration[i].weight_index_parameter, _declaration[i].size, _declaration[i].type, sizeof(f32) * _declaration[i].size, _declaration[i].weight_index_data);
						}
						//glMatrixIndexPointerARB(_declaration[i].size, _declaration[i].type, 0, NULL); // ʂGLƂȊɂȂ
						break;
					case GL_VERTEX_WEIGHT_ARRAY:
						if(_declaration[i].weight_index_parameter){
							cgGLEnableClientState(_declaration[i].weight_index_parameter);
							cgGLSetParameterPointer(_declaration[i].weight_index_parameter, _declaration[i].size, _declaration[i].type, sizeof(f32) * _declaration[i].size, _declaration[i].weight_index_data);
						}
						//glVertexWeightPointerEXT(_declaration[i].size, _declaration[i].type, 0, NULL); // ʂGLƂȊɂȂ
						break;
					}
				}								  	   				
			}

			/*!
			\brief `
			*/
			void Draw() { glDrawArrays(_primtype, 0, _nvertices); }

			/*!
			\brief `̌㏈
			*/
			void End() {
				glBindBuffer(GL_ARRAY_BUFFER, 0);
				for(u32 i = 0;i < _ndeclaration;++i) {
					switch(_declaration[i].semantic){
		            default:
						glDisableClientState(_declaration[i].semantic);
						break;
					case GL_MATRIX_INDEX_ARRAY:  
					case GL_VERTEX_WEIGHT_ARRAY:
						//cgGLDisableClientState();
						break;
					}
				}
			}

			/*!
			\brief XR[v𔲂ƂɎIEnd()R[֗NX
			*/
			class AutoBegin
			{
			public:
				AutoBegin(VertexBuffer &vb) : _vb(vb){ _vb.Begin(); }
				~AutoBegin(){ _vb.End(); }
			private:
				VertexBuffer &_vb;
			};

		private:
			PrimitiveType _primtype;
			Declaration *_declaration;
			u32 _ndeclaration;
			u32 _nvertices;
		};

#ifdef VERTEXBUFFER_SPECIALIZATION
#include "../../source/graphics/vertexbuffer_specialization.cpp"
#endif
	};
};

#endif // VERTEXBUFFER_H

#ifdef PRIMTYPE
PRIMTYPE(POINTS, GL_POINTS)
PRIMTYPE(LINES, GL_LINES)
//PRIMTYPE(LINE_LOOP, GL_LINE_LOOP)
PRIMTYPE(LINE_STRIP, GL_LINE_STRIP)
PRIMTYPE(TRIANGLES, GL_TRIANGLES)
PRIMTYPE(TRIANGLE_STRIP, GL_TRIANGLE_STRIP)
PRIMTYPE(TRIANGLE_FAN, GL_TRIANGLE_FAN)
#endif