/*!
$Id: input.h 301 2009-08-21 03:02:27Z horino $
*/
#ifndef INPUT_H
#define INPUT_H

#ifdef _DEBUG
#include <iostream>
using namespace std;
#endif

#include <cell/pad.h>

#include "indi.h"
#include "utilities.h"
#include "itl/bitset.h"
#include "vector2.h"

namespace indi
{
	/*!
	\brief pbh͂܂AƂ肠ƍƂɂȂȂ̂
	\note ̂ւ͌ŒNɔC
	*/
	class Input
	{
	public:
		class State
		{
		public:
			enum Kind {
#define PADASSIGN(id, input, index) id, 
#include "input.h"
#undef PADASSIGN
				KindNum
			};

			/*!
			\breif etO
			*/
			enum {
				LS_DEADZONE,
				RS_DEADZONE,
				LT_DEADZONE,
				RT_DEADZONE,
			};
			State() : _flag(0) { indi::Memzero(&_state, sizeof(_state)); }

			BOOL Update(u32 i);
			BOOL IsValid() const { return _state.len > 0; }

			u8 LeftTrigger() const { return _state.button[18]; }
			u8 RightTrigger() const { return _state.button[19]; }
			f32 LeftTriggerf() const { return _flag.Test(LT_DEADZONE) ? 0.0f : static_cast<f32>(LeftTrigger()) / 0xff; }
			f32 RightTriggerf() const { return _flag.Test(RT_DEADZONE) ? 0.0f : static_cast<f32>(RightTrigger()) / 0xff; }

			s16 LeftStickX() const { return _flag.Test(LS_DEADZONE) ? 0 : math::Clamp(static_cast<s16>(_state.button[6]) - 127, -127, 127) << 8; }
			s16 LeftStickY() const { return _flag.Test(LS_DEADZONE) ? 0 : math::Clamp(127 - static_cast<s16>(_state.button[7]), -127, 127) << 8; }
			f32 LeftStickXf() const { return _flag.Test(LS_DEADZONE) ? 0.0f : static_cast<f32>(LeftStickX()) / 32767.0f; }
			f32 LeftStickYf() const { return _flag.Test(LS_DEADZONE) ? 0.0f : static_cast<f32>(LeftStickY()) / 32767.0f; }
			Vector2 LeftStick() const { return _flag.Test(LS_DEADZONE) ? Vector2::Zero() : Vector2(LeftStickXf(), LeftStickYf()); }

			s16 RightStickX() const { return _flag.Test(RS_DEADZONE) ? 0 : math::Clamp(static_cast<s16>(_state.button[4]) - 127, -127, 127) << 8; }
			s16 RightStickY() const { return _flag.Test(RS_DEADZONE) ? 0 : math::Clamp(127 - static_cast<s16>(_state.button[5]), -127, 127) << 8; }
			f32 RightStickXf() const { return _flag.Test(RS_DEADZONE) ? 0.0f : static_cast<f32>(RightStickX()) / 32767.0f; }
			f32 RightStickYf() const { return _flag.Test(RS_DEADZONE) ? 0.0f : static_cast<f32>(RightStickY()) / 32767.0f; }
			Vector2 RightStick() const { return _flag.Test(RS_DEADZONE) ? Vector2::Zero() : Vector2(RightStickXf(), RightStickYf()); }

			BOOL Button(u32 k) const { return Button(static_cast<Kind>(k)); }
			BOOL Button(Kind k) const {	return InputMask(k) & _state.button[InputIndex(k)]; }

			BOOL Up() const { return Button(D_UP); }
			BOOL Down() const { return Button(D_DOWN); }
			BOOL Left() const { return Button(D_LEFT); }
			BOOL Right() const { return Button(D_RIGHT); }

			BOOL Start() const { return Button(START); }
			BOOL Back() const { return Button(BACK_SELECT); }
			BOOL Select() const { return Back(); }

			BOOL LeftStickPush() const { return Button(L_STICK_PUSH); }
			BOOL RightStickPush() const { return Button(R_STICK_PUSH); }

			BOOL LeftShoulder() const { return Button(L_SHOULDER); }
			BOOL RightShoulder() const { return Button(R_SHOULDER); }

			BOOL A() const { return Button(A_CROSS); }
			BOOL Cross() const { return A(); }

			BOOL B() const { return Button(B_CIRCLE); }
			BOOL Circle() const { return B(); }

			BOOL X() const { return Button(X_SQUARE); }
			BOOL Square() const { return X(); }

			BOOL Y() const { return Button(Y_TRIANGLE); }
			BOOL Triangle() const { return Y(); }

			BOOL Guide() const { return Button(GUIDE); }

//			friend BOOL operator==(const State &lhs, const State &rhs){ return lhs._state.dwPacketNumber == rhs._state.dwPacketNumber;	}
//			friend BOOL operator!=(const State &lhs, const State &rhs){ return lhs._state.dwPacketNumber != rhs._state.dwPacketNumber ; }

#ifdef _DEBUG
			static char *KindString(Kind k);
			static char *KindString(u32 k) { return KindString(static_cast<Kind>(k)); }
#endif
		private:
			static u32 InputMask(Kind k);
			static u32 InputIndex(Kind k);

		private:
			CellPadData _state;
			itl::Bitset _flag;
		};

	public:
		Input() { ASSERT(cellPadInit(_padMax) == CELL_OK); }
		~Input() { ASSERT(cellPadEnd() == CELL_OK); }

		void Update();

		const State &Previous(u32 i) const { ASSERT(i < 4); return _state[i][1]; }
		const State &Current(u32 i) const { ASSERT(i < 4); return _state[i][0]; }

	private:
		static const u32 _padMax = 4; //< pbh͂S܂ŁH
		static const u32 _historyMax = 2; //< iƂ肠jOt[̓͂܂ŗoĂ
		State _state[_padMax][_historyMax];

		CellPadInfo2 _info;
	};
};

#ifdef _DEBUG
#ifdef PS3
static ostream __attribute__((__used__)) &operator<<(ostream &lhs, const indi::Input::State &rhs) 
{
	for(indi::u32 i = 0;i < indi::Input::State::KindNum; ++i)
		if(rhs.Button(i)) cout << indi::Input::State::KindString(i) << endl;
	if(rhs.LeftStick() != indi::Vector2::Zero()) cout << "LS : " << rhs.LeftStick() << endl;
	if(rhs.RightStick() != indi::Vector2::Zero()) cout << "RS : " << rhs.RightStick() << endl;
	if(rhs.LeftTriggerf() != 0.0f) cout << "LT : " << rhs.LeftTriggerf() << endl;
	if(rhs.RightTriggerf() != 0.0f) cout << "RT : " << rhs.RightTriggerf() << endl;
	return lhs; 
}
#else
static ostream &operator<<(ostream &lhs, const indi::Input::State &rhs) 
{
	for(indi::u32 i = 0;i < indi::Input::State::KindNum; ++i)
		if(rhs.Button(i)) cout << indi::Input::State::KindString(i) << endl;
	if(rhs.LeftStick() != indi::Vector2::Zero()) cout << "LS : " << rhs.LeftStick() << endl;
	if(rhs.RightStick() != indi::Vector2::Zero()) cout << "RS : " << rhs.RightStick() << endl;
	if(rhs.LeftTriggerf() != 0.0f) cout << "LT : " << rhs.LeftTriggerf() << endl;
	if(rhs.RightTriggerf() != 0.0f) cout << "RT : " << rhs.RightTriggerf() << endl;
	return lhs; 
}
#endif
#endif

#endif // INPUT_H

/*!
\brief {^̎ނenuminputƂ̑Ή\
*/
#ifdef PADASSIGN
PADASSIGN(D_UP,         0x10, 2)
PADASSIGN(D_DOWN,       0x40, 2)
PADASSIGN(D_LEFT,       0x80, 2)
PADASSIGN(D_RIGHT,      0x20, 2)
PADASSIGN(START,        0x8,  2)
PADASSIGN(BACK_SELECT,  0x1,  2)
PADASSIGN(L_STICK_PUSH, 0x2,  2)
PADASSIGN(R_STICK_PUSH, 0x4,  2)

PADASSIGN(L_SHOULDER,   0x4,  3)
PADASSIGN(R_SHOULDER,   0x8,  3)
PADASSIGN(A_CROSS,      0x40, 3)
PADASSIGN(B_CIRCLE,     0x20, 3)
PADASSIGN(X_SQUARE,     0x80, 3)
PADASSIGN(Y_TRIANGLE,   0x10, 3)
PADASSIGN(GUIDE,        0,    3)
#endif

