/*! @file
 @brief Matrix StackisX^bNj
 @author
 @date
*/

#ifndef __MATRIXSTACK_H__
#define __MATRIXSTACK_H__

#include "indi.h"

#if defined (_XBOX) || defined (_WINDOWS)
#include <d3dx9.h>
#elif defined (PS3)
#include <GLES/gl.h>
#endif

namespace indi
{

class MatrixStack
{
public:
	enum {
		MM_VIEW,
		MM_PROJECTION,
		MM_TEXTURE,
		MM_MAX
	};

private:
	static s32 matrixmode;
#if defined (_XBOX) || defined (_WINDOWS)
	static LPD3DXMATRIXSTACK _ms[ MM_MAX ];
#elif defined (PS3)
	static s32 MatrixModeValues[ MM_MAX ];
#endif

public:
	static s32 Create( void );

	static void Release( void );

	static s32 PopMatrix( void );

	static s32 PushMatrix( void );

	static s32 LoadIdentity( void );

	static s32 LoadMatrix( CONST Matrix44& m );

	static s32 MultMatrix( CONST Matrix44& m );

//	static s32 MultMatrixLocal( CONST Matrix44& m );

	static s32 RotateAxis( CONST Vector3& v, f32 Angle );

	static s32 RotateAxis( f32 Angle, f32 x, f32 y, f32 z );

//	static s32 RotateAxisLocal( CONST Vector3& v, f32 Angle );

//	static s32 RotateYawPitchRoll( f32 Yaw, f32 Pitch, f32 Roll );

//	static s32 RotateYawPitchRollLocal( f32 Yaw, f32 Pitch, f32 Roll );

	static s32 Scale( f32 x, f32 y, f32 z );

//	static s32 ScaleLocal( f32 x, f32 y, f32 z );

	static s32 Translate( f32 x, f32 y, f32 z );

//	static s32 TranslateLocal( f32 x, f32 y, f32 z );

	static Matrix44 GetTopMatrix( void );

	static s32 MatrixMode( s32 mode );

};

#include "matrixstack.inl"

}	// indi


#endif	// __MATRIXSTACK_H__
