#ifndef __MATRIXSTACK_INL__
#define __MATRIXSTACK_INL__

INLINE s32 MatrixStack::Create( void )
{
#if defined (_XBOX) || defined (_WINDOWS)
	s32 rtn = S_OK;
	for ( s32 i = 0; i < MM_MAX; ++ i ){
		rtn = D3DXCreateMatrixStack( 0, &_ms[ i ] );
		if ( rtn != S_OK )
			Release();
	}
	return rtn;
#elif defined (PS3)
	return 0;
#endif
}

INLINE void MatrixStack::Release( void )
{
#if defined (_XBOX) || defined (_WINDOWS)
	for ( s32 i = 0; i < MM_MAX; ++ i ){
		SAFE_RELEASE( _ms[ i ] );
	}
#elif defined (PS3)
	return;
#endif
}

INLINE s32 MatrixStack::PopMatrix( void )
{
#if defined (_XBOX) || defined (_WINDOWS)
	return _ms[ matrixmode ]->Pop();
#elif defined (PS3)
	glPopMatrix();
	return glGetError();
#endif
}

INLINE s32 MatrixStack::PushMatrix( void )
{
#if defined (_XBOX) || defined (_WINDOWS)
	return _ms[ matrixmode ]->Push();
#elif defined (PS3)
	glPushMatrix();
	return glGetError();
#endif
}

INLINE s32 MatrixStack::LoadIdentity( void )
{
#if defined (_XBOX) || defined (_WINDOWS)
	return _ms[ matrixmode ]->LoadIdentity();
#elif defined (PS3)
	glLoadIdentity();
	return glGetError();
#endif
}

INLINE s32 MatrixStack::LoadMatrix( CONST Matrix44& m )
{
#if defined (_XBOX) || defined (_WINDOWS)
	return _ms[ matrixmode ]->LoadMatrix( m );
#elif defined (PS3)
	glLoadMatrixf( (f32*)&m );
	return glGetError();
#endif
}

INLINE s32 MatrixStack::MultMatrix( CONST Matrix44& m )
{
#if defined (_XBOX) || defined (_WINDOWS)
	return _ms[ matrixmode ]->MultMatrix( m );
#elif defined (PS3)
	glMultMatrixf( (f32*)&m );
	return glGetError();
#endif
}

//INLINE s32 MatrixStack::MultMatrixLocal( CONST Matrix44& m )
//{
//	return _ms[ matrixmode ]->MultMatrixLocal( m );
//}

INLINE s32 MatrixStack::RotateAxis( CONST Vector3& v, f32 Angle )
{
#if defined (_XBOX) || defined (_WINDOWS)
	return _ms[ matrixmode ]->RotateAxis( v, Angle );
#elif defined (PS3)
	return RotateAxis( v.X(), v.Y(), v.Z(), Angle );
#endif
}

INLINE s32 MatrixStack::RotateAxis( f32 Angle, f32 x, f32 y, f32 z )
{
#if defined (_XBOX) || defined (_WINDOWS)
	Vector3 v( x, y, z );
	return RotateAxis( v, Angle );
#elif defined (PS3)
	glRotatef( Angle, x, y, z );
	return glGetError();
#endif
}

//INLINE s32 MatrixStack::RotateAxisLocal( CONST Vector3& v, f32 Angle )
//{
//	return _ms[ matrixmode ]->RotateAxisLocal( v, Angle );
//}

//INLINE s32 MatrixStack::RotateYawPitchRoll( f32 Yaw, f32 Pitch, f32 Roll )
//{
//	return _ms[ matrixmode ]->RotateYawPitchRoll( Yaw, Pitch, Roll );
//}

//INLINE s32 MatrixStack::RotateYawPitchRollLocal( f32 Yaw, f32 Pitch, f32 Roll )
//{
//	return _ms[ matrixmode ]->RotateYawPitchRollLocal( Yaw, Pitch, Roll );
//}

INLINE s32 MatrixStack::Scale( f32 x, f32 y, f32 z )
{
#if defined (_XBOX) || defined (_WINDOWS)
	return _ms[ matrixmode ]->Scale( x, y, z );
#elif defined (PS3)
	glScalef( x, y, z );
	return glGetError();
#endif
}

//INLINE s32 MatrixStack::ScaleLocal( f32 x, f32 y, f32 z )
//{
//	return _ms[ matrixmode ]->ScaleLocal( x, y, z );
//}

INLINE s32 MatrixStack::Translate( f32 x, f32 y, f32 z )
{
#if defined (_XBOX) || defined (_WINDOWS)
	return _ms[ matrixmode ]->Translate( x, y, z );
#elif defined (PS3)
	glTranslatef( x, y, z );
	return glGetError();
#endif
}

//INLINE s32 MatrixStack::TranslateLocal( f32 x, f32 y, f32 z )
//{
//	return _ms[ matrixmode ]->TranslateLocal( x, y, z );
//}

INLINE Matrix44 MatrixStack::GetTopMatrix( void )
{
#if defined (_XBOX) || defined (_WINDOWS)
	return Matrix44( _ms[ matrixmode ]->GetTop() );
#elif defined (PS3)
	return Matrix44();
#endif
}

INLINE s32 MatrixStack::MatrixMode( s32 mode )
{
	matrixmode = mode;
#if defined (_WINDOWS)
	return 0;
#elif defined (_XBOX)
	return 0;
#elif defined (PS3)
	glMatrixMode( MatrixModeValues[ mode ] );
	return glGetError();
#endif
}

#endif	// __MATRIXSTACK_INL__
