/*!
$Id: quaternion_transform.h 624 2009-12-17 07:37:52Z horino $
*/
#ifndef QUATERNION_TRANSFORM_H
#define QUATERNION_TRANSFORM_H

#ifdef _DEBUG
#include <iostream>
using namespace std;
#endif

#include "indi.h"
#include "quaternion.h"
#include "vector3.h"
#include "matrix44.h"

namespace indi
{
	class QuaternionTransform
	{
	public:
		QuaternionTransform() : rotation(), translation() {}
		QuaternionTransform(const Quaternion &r, const Vector3 &t) : rotation(r), translation(t) {}
		QuaternionTransform(const Matrix44 &m);

		u32 Read(const u8 *data) { 
			const u8 *p(data);
			p += rotation.Read(p);
			p += translation.Read(p);
			return p - data;
		}

		QuaternionTransform &operator*=(const QuaternionTransform &rhs) { return (*this = *this * rhs); }
		friend QuaternionTransform operator*(const QuaternionTransform &lhs, const QuaternionTransform &rhs){
			return QuaternionTransform(rhs.rotation * lhs.rotation, Vector3(rhs.rotation.Rotate(lhs.translation)) + rhs.translation);
		}
		operator Matrix44() const { return Matrix44(rotation) * Matrix44::Translation(translation); }

		static QuaternionTransform Identity() { return QuaternionTransform(Quaternion::Identity(), Vector3::Zero()); }

		Quaternion rotation;
		Vector3 translation;

#ifdef _DEBUG
		void NanCheck() const { rotation.NanCheck(); translation.NanCheck(); }
#endif
	};
};

#ifdef _DEBUG
static ostream ATTR_USED &operator<<(ostream &lhs, const indi::QuaternionTransform &rhs) { return lhs << rhs.rotation << rhs.translation; }
#endif

#endif // QUATERNION_TRANSFORM_H