/*!
$Id: vector2.h 648 2010-01-07 03:39:59Z horino $
*/
#ifndef VECTOR2_H
#define VECTOR2_H

#ifdef _DEBUG
#include <iostream>
using namespace std;
#endif

#include "indi.h"
#include "utilities.h"

namespace indi
{
	/*!
	\brief SCE񋟂̂̂Vector2Ȃ̂ŎOi͗ǂȂj
	*/
	namespace aos
	{
		class Vector2
		{
		private:
			f32 _x, _y;
		public:
			Vector2(){}
			Vector2(f32 v) : _x(v), _y(v){}
			Vector2(f32 x, f32 y) : _x(x), _y(y){}
			Vector2(const f32 *p) : _x(p[0]), _y(p[1]){}

			u32 Read(const u8 *data) { 
				f32 m[2];
				const u32 *p(reinterpret_cast<const u32 *>(data));
				for(u32 i=0;i<NELEMS(m);++i) 
					m[i] = ToF32(*reinterpret_cast<const u32 *>(p++));
				*this = Vector2(m);
				return reinterpret_cast<const u8 *>(p) - data;
			}

			f32 getX() const { return _x; }
			f32 getY() const { return _y; }
			void setX(f32 x) { _x = x; }
			void setY(f32 y) { _y = y; }

			Vector2 operator-() const { return Vector2(-getX(), -getY()); }
			f32 operator[](s32 i) const { ASSERT(i >= 0 && i < 2); return (&_x)[i]; }
			Vector2 &operator+=(const Vector2 &rhs) { _x += rhs._x; _y += rhs._y; return *this; }
			Vector2 &operator-=(const Vector2 &rhs) { _x -= rhs._x; _y -= rhs._y; return *this; }
			Vector2 &operator*=(f32 rhs) { _x *= rhs; _y *= rhs; return *this; }
			Vector2 &operator/=(f32 rhs) { _x /= rhs; _y /= rhs; return *this; }

			friend Vector2 operator+(const Vector2 &lhs, const Vector2 &rhs){ return Vector2(lhs._x + rhs._x, lhs._y + rhs._y); }
			friend Vector2 operator-(const Vector2 &lhs, const Vector2 &rhs){ return Vector2(lhs._x - rhs._x, lhs._y - rhs._y); }
			friend Vector2 operator*(const Vector2 &lhs, f32 rhs) { return Vector2(lhs._x * rhs, lhs._y * rhs); }
			friend Vector2 operator*(f32 lhs, const Vector2 &rhs) { return Vector2(rhs._x * lhs, rhs._y * lhs); }
			friend Vector2 operator/(const Vector2 &lhs, f32 rhs) { return Vector2(lhs._x / rhs, lhs._y / rhs); }
			friend f32 operator*(const Vector2 &lhs, const Vector2 &rhs){ return lhs._x * rhs._x + lhs._y * rhs._y; }

			f32 lengthSqr() const { return (*this) * (*this); }
			f32 length() const { return math::Sqrt(lengthSqr()); }
			Vector2 &normalize() { (*this) /= length(); return *this; }
			static Vector2 xAxis() { return Vector2(1.0f, 0.0f); }
			static Vector2 yAxis() { return Vector2(1.0f, 0.0f); }
		};
		static f32 dot(const Vector2 &lhs, const Vector2 &rhs){ return lhs * rhs; }
		static f32 lengthSqr(const Vector2 &v){ return v.lengthSqr(); }
		static f32 length(const Vector2 &v){ return v.length(); }
		static Vector2 &normalize(const Vector2 &v) { return Vector2(v).normalize(); }
	};

	class Vector2
	{
	private:
		aos::Vector2 _e;

	public:
		Vector2() {}
		Vector2(const aos::Vector2 &v) : _e(v) {}
		Vector2(f32 v) : _e(aos::Vector2(v)) {}
		Vector2(f32 x, f32 y) : _e(aos::Vector2(x, y)) {}

		operator const aos::Vector2&() const { return _e; }
		operator aos::Vector2&() { return _e; }
		operator const aos::Vector2*() const { return &_e; }
		operator aos::Vector2*() { return &_e; }

		f32 X() const { return _e.getX(); }
		f32 Y() const { return _e.getY(); }
		f32 GetX() const { return X(); }
		f32 GetY() const { return Y(); }
		Vector2 &Set(f32 x, f32 y) { _e.setX(x); _e.setY(y); return *this; }
		Vector2 &SetX(f32 x) { _e.setX(x); return *this; }
		Vector2 &SetY(f32 y) { _e.setY(y); return *this; }

		Vector2 &operator=(const Vector2 &rhs) { _e = rhs._e; return *this; }
		f32 operator[](s32 i) const { return _e[i]; }
		Vector2 operator-() const { return Vector2(-_e); }
		Vector2 &operator+=(const Vector2 &rhs) { _e += rhs._e; return *this; }
		Vector2 &operator-=(const Vector2 &rhs) { _e -= rhs._e; return *this; }
		Vector2 &operator*=(f32 rhs) { _e *= rhs; return *this; }
		Vector2 &operator/=(f32 rhs) { _e /= rhs; return *this; }

		friend Vector2 operator+(const Vector2 &lhs, const Vector2 &rhs){ return Vector2(lhs._e + rhs._e); }
		friend Vector2 operator-(const Vector2 &lhs, const Vector2 &rhs){ return Vector2(lhs._e - rhs._e); }
		friend Vector2 operator*(const Vector2 &lhs, f32 rhs) { return Vector2(lhs._e * rhs); }
		friend Vector2 operator*(f32 lhs, const Vector2 &rhs) { return Vector2(rhs._e * lhs); }
		friend Vector2 operator/(const Vector2 &lhs, f32 rhs) { return Vector2(lhs._e / rhs); }
		friend BOOL operator==(const Vector2 &lhs, const Vector2 &rhs){ return lhs.X() == rhs.X() && lhs.Y() == rhs.Y();	}
		friend BOOL operator!=(const Vector2 &lhs, const Vector2 &rhs){ return !(lhs == rhs); }
		friend f32 operator*(const Vector2 &lhs, const Vector2 &rhs){ return aos::dot(lhs._e, rhs._e); }

		f32 Length() const { return aos::length(_e); }
		f32 LengthSquared() const { return aos::lengthSqr(_e); }
		Vector2 &Normalize() { _e = aos::normalize(_e); return *this; }
		Vector2 &Adjust(f32 len){ Normalize() *= len; return *this; }

		static f32 Dot(const Vector2 &lhs, const Vector2 &rhs){ return lhs * rhs; }
		static Vector2 Zero(){ return Vector2(0.0f); }
		static Vector2 One(){ return Vector2(1.0f); }
		static Vector2 Half(){ return Vector2(0.5f); }
		static Vector2 UnitX(){ return Vector2(aos::Vector2::xAxis()); }
		static Vector2 UnitY(){ return Vector2(aos::Vector2::yAxis()); }

#ifdef _DEBUG
		void NanCheck() const { NAN_CHECK(_e.getX()); NAN_CHECK(_e.getY()); }
#endif
	};
};

#ifdef _DEBUG
static ostream ATTR_USED &operator<<(ostream &lhs, const indi::Vector2 &rhs) { return lhs << "(" << rhs.X() << ", " << rhs.Y() << ")";  }
#endif

#endif // VECTOR2_H