/*!
$Id: framebuffer.cpp 541 2009-11-20 10:40:29Z horino $
*/
#define RENDERTARGET_CPP

#ifdef _DEBUG
#include <iostream>
using namespace std;
#endif

#include "graphics/graphics.h"
#include "graphics/rendertarget.h"
#include "graphics/texture.h"
#include "graphics/framebuffer.h"
using namespace indi;

/*!
\brief FrameBuffer̃RXgN^
\param Z}eBNX0̃_^[Qbg
\param Z}eBNX1̃_^[Qbg
\param Z}eBNX2̃_^[Qbg
\param Z}eBNX3̃_^[Qbg
\param Z}eBNX4̃_^[Qbg
*/
graphics::FrameBuffer::FrameBuffer(RenderTargetBase *rt0, RenderTargetBase *rt1, RenderTargetBase *rt2, RenderTargetBase *rt3, RenderTargetBase *rt4)
{
	glGenFramebuffersOES(1, &_frame_buffer);
	glBindFramebufferOES(GL_FRAMEBUFFER_OES, _frame_buffer); {
		if(rt0) Add(0, rt0); 
		if(rt1) Add(1, rt1); 
		if(rt2) Add(2, rt2); 
		if(rt3) Add(3, rt3); 
		if(rt4) Add(4, rt4); 
	} glBindFramebufferOES(GL_FRAMEBUFFER_OES, 0);

#ifdef _DEBUG
	//< t[obt@Xe[g̃`FbN
	ASSERT(GL_FRAMEBUFFER_COMPLETE_OES == glCheckFramebufferStatusOES(GL_FRAMEBUFFER_OES)); 
#endif
}

/*!
\brief t[obt@փ_[^[Qbgǉ܂
\param Z}eBNXԍ
\param _[^[Qbg
*/
void graphics::FrameBuffer::Add(u32 i, RenderTargetBase *rt)
{
	if(rt->IsDepthTarget()){
		//< fvX^[Qbg̏ꍇ
#ifdef _DEBUG
		ASSERT(static_cast<DepthTarget *>(rt)->GetDepthTarget());
#endif
		glFramebufferRenderbufferOES(GL_FRAMEBUFFER_OES, GL_DEPTH_ATTACHMENT_OES, GL_RENDERBUFFER_OES, static_cast<DepthTarget *>(rt)->GetDepthTarget());
		GL_ERROR_CHECK;
	} else if(rt->IsRenderTarget()){
		//< _^[Qbg̏ꍇ
		graphics::RenderTexture *res_tex(static_cast<RenderTarget *>(rt)->GetResolveTexture());
#ifdef _DEBUG
		ASSERT(res_tex && res_tex->_id);
#endif
		glFramebufferTexture2DOES(GL_FRAMEBUFFER_OES, GL_COLOR_ATTACHMENT0_EXT + i, GL_TEXTURE_2D, res_tex->_id, 0);
		GL_ERROR_CHECK;
	}

	_rendertargets.push_back(rt);
}

/*!
\brief t[obt@w̐FŃNA܂'
\param R
\param G
\param B
\param A
*/
void graphics::FrameBuffer::Clear(u8 r, u8 g, u8 b, u8 a)
{ 
	Begin();
	graphics::System::ClearColor(r, g, b, a);
	graphics::System::Clear();
	End();
}

/*!
\brief _[^[Qbgւ̕`̊JnOɃR[܂
*/
void graphics::FrameBuffer::Begin()
{
#ifdef _DEBUG
	ASSERT(!_rendertargets.empty()); //< A^b`ĂȂ
#endif

	//< _[^[Qbg̃TCY𓾂܂(Z}eBbNOƂ)
	u32 w(_rendertargets[0]->GetWidth());
	u32 h(_rendertargets[0]->GetHeight());
	//< r[|[gVU[eNX`̑傫ɍ킹Ē
	glViewport(0, 0, w, h);
	glScissor(0, 0, w, h);
	glBindFramebufferOES(GL_FRAMEBUFFER_OES, _frame_buffer);
	GL_ERROR_CHECK;
}

/*!
\brief _[^[Qbgւ̕`̏IɃR[܂
*/
void graphics::FrameBuffer::End()
{
	glBindFramebufferOES(GL_FRAMEBUFFER_OES, 0);
	GL_ERROR_CHECK;
	//< r[|[gVU[߂
	u32 w(graphics::System::GetScreenWidth());
	u32 h(graphics::System::GetScreenHeight());
	glViewport(0, 0, w, h);
	glScissor(0, 0, w, h);
}

/*!
\brief ̃t[obt@pāAw肳ꂽ_eNX`tXN[`悵܂
\param _eNX`
\note eNX`̃Rs[ȂǂɎg܂
*/
void graphics::FrameBuffer::FullScreenRender(const RenderTexture *t)
{
	Begin();{
		graphics::System::DrawFullScreen(t);
	} End();
}