/*!
$Id: graphics.cpp 640 2009-12-24 07:03:53Z horino $
*/
#define GRAPHICS_CPP

#ifdef _DEBUG
#include <iostream>
using namespace std;
#endif

#include <sys/spu_initialize.h>
#include <cell/sysmodule.h>
#include <Cg/cgGL.h>
#if 0//def _DEBUG
// C:\usr\local\cell\host-win32\include\SCEShaderPerf.h͖
// Tv C:\usr\local\cell\samples\sdk\graphics\sceshaderperf\basic
#include <SCEShaderPerf.h> 
#endif

#include "graphics/graphics.h"
#include "graphics/shader/shader.h"
#include "graphics/texture.h"
#include "primitive/primitive.h"
#include "vector2.h"
using namespace indi;

_VIEWPORT graphics::System::viewport = {0, 0, 0, 0, 0.0f, 1.0f };
Collision::Frustum graphics::System::frustum;
Matrix44 graphics::System::viewportMatrix;
Matrix44 graphics::System::inverseViewportMatrix;
f32 graphics::System::aspectRatio = 0.0f;
graphics::System::SampleType graphics::System::sampleType = graphics::System::SAMPLE_2;
graphics::System::FrameRate graphics::System::frameRate = graphics::System::FRAME_RATE_60;

//< tXN[`p
graphics::shader::VertexProgram *graphics::System::fullscreen_vs = 0;
graphics::System::FullScreenPS *graphics::System::fullscreen_ps = 0;

static bool loadmodule(uint16_t m)
{
	bool ret = true;
	switch(cellSysmoduleLoadModule(m)){
	case CELL_OK:
	case CELL_SYSMODULE_ERROR_DUPLICATED: break;
	case CELL_SYSMODULE_ERROR_UNKNOWN: 
#ifdef _DEBUG
		cout << "unknown" << endl; 
#endif
	case CELL_SYSMODULE_ERROR_FATAL:
#ifdef _DEBUG
		cout << "fatal" << endl; 
#endif
		ret = false;
	}
	return ret;
}

static void callback(uint64_t status, uint64_t param, void *usrdata)
{
	switch(status){
	case CELL_SYSUTIL_REQUEST_EXITGAME: glFinish(); break;
	case CELL_SYSUTIL_DRAWING_BEGIN: 
	case CELL_SYSUTIL_DRAWING_END: break;
	}
}

/*!
\brief OtBNXfoCXȂǂ̏
\note Ƃ肠KɂĂ̂ŁAKvɉĕύXĂ
*/
void graphics::System::Initialize()
{
	//< SPU̎gpݒ
	sys_spu_initialize(6, 1);

	ASSERT(loadmodule(CELL_SYSMODULE_GCM_SYS));
	ASSERT(loadmodule(CELL_SYSMODULE_FS));
	ASSERT(loadmodule(CELL_SYSMODULE_USBD));
	ASSERT(loadmodule(CELL_SYSMODULE_IO));

	ASSERT(cellSysutilRegisterCallback(0, callback, NULL)==CELL_OK);

	//< rfIo͂̏҂
#ifdef _DEBUG
	cout << "[ VIDEO OUTPUT READY ] ... ";
#endif
	while(!isVideoReady()) ;
#ifdef _DEBUG
	cout << "OK" << endl;
#endif

	//< PSGL̏
#ifdef _DEBUG
	cout << "[ PSGL INIT ] ... ";
#endif
	PSGLinitOptions opt = {
		enable: PSGL_INIT_MAX_SPUS | PSGL_INIT_INITIALIZE_SPUS | PSGL_INIT_HOST_MEMORY_SIZE,
		maxSPUs: 1,
		initializeSPUs: false,
		persistentMemorySize: 0,           //< 
		transientMemorySize: 0,            //< 
		errorConsole: 0,                   //< stderrւ̃G[o
		fifoSize: 0,                       //< RSXւ̃f[^]L[TCY
	    hostMemorySize: 128 * 1024 * 1024, //< RSXgp邽߂Ɋ蓖ĂꂽieNX`A_AsNZobt@jTCY
	};
	psglInit(&opt);
#ifdef _DEBUG
	cout << "OK" << endl;
#endif

	//< foCX̍쐬
#ifdef _DEBUG
	cout << "[ CREATE PSGL DEVICE ] ... ";
#endif
	PSGLdevice *glDevice = psglCreateDeviceAuto(GL_ARGB_SCE, GL_DEPTH_COMPONENT24, sampleType);
	ASSERT(glDevice!=0);
	psglGetDeviceDimensions(glDevice, &viewport.Width, &viewport.Height);

	frustum.Create(viewport.X, viewport.Y, viewport.Width, viewport.Height, viewport.MinZ, viewport.MaxZ);

	{
		f32 w(static_cast<f32>(viewport.Width) * 0.5f);
		f32 h(static_cast<f32>(viewport.Height) * 0.5f);
		viewportMatrix = Matrix44(w, 0.0f, 0.0f, 0.0f,
			0.0f, -h, 0.0f, 0.0f,
			0.0f, 0.0f, viewport.MaxZ - viewport.MinZ, 0.0f, 
			static_cast<f32>(viewport.X) + w, static_cast<f32>(viewport.Y) + h, viewport.MinZ, 1.0f);
		inverseViewportMatrix = viewportMatrix.Inverse();
	}
	aspectRatio = psglGetDeviceAspectRatio(glDevice);
#ifdef _DEBUG
	cout << "OK" << endl;
#endif

#ifdef _DEBUG
	cout << "[ SCREEN SIZE " << viewport.Width << " x " << viewport.Height << " ]" << endl;
#endif

	//< ReLXg̍쐬
#ifdef _DEBUG
	cout << "[ CREATE PSGL CONTEXT ] ... ";
#endif
	PSGLcontext *glContext = psglCreateContext();
	ASSERT(glContext!=0);
	psglMakeCurrent(glContext, glDevice);
	psglResetCurrentContext();
#ifdef _DEBUG
	cout << "OK" << endl;
#endif

	psglResetCurrentContext();
  
	glViewport(viewport.X, viewport.Y, viewport.Width, viewport.Height);
	glScissor(viewport.X, viewport.Y, viewport.Width, viewport.Height);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_VSYNC_SCE);
	glEnable(GL_CULL_FACE);
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_TEXTURE_CUBE_MAP);
	glClearDepthf(1.0f);

#ifdef _DEBUG
	ASSERT(frameRate != FRAME_RATE_20); // TODO
	ASSERT(frameRate != FRAME_RATE_15); // TODO
#endif
	if(frameRate == FRAME_RATE_30)
		glEnable(GL_SKIP_FIRST_VSYNC_SCE);

#if 0//def _DEBUG
	SceSpResult InitRes = sceShaderPerfInit();
#endif
}

/*!
\brief XN[W  [hWϊ
\param [hW
\param [h}gNX
\param r[}gNX
\param vWFNV}gNX
\return XN[W
*/
Vector3 graphics::System::UnProject(const Vector3 &spos, const Matrix44 &world, const Matrix44 &view, const Matrix44 &projection)
{
	Vector4 tmp(Vector4(spos, 1.0f) * GetInverseViewportMatrix() * projection.Inverse() * view.Inverse() * world.Inverse());
	return Vector3(tmp) / tmp.W();
}

/*!
\brief XN[W  [hWϊ
\param [hW
\param [hW
\\param XN[Wo͐
\param [h}gNX
\param r[}gNX
\param vWFNV}gNX
\return XN[W
*/
Vector3 *graphics::System::UnProject(const Vector3 *spos, u32 n, Vector3 *dpos, const Matrix44 &world, const Matrix44 &view, const Matrix44 &projection)
{
	Matrix44 m(GetInverseViewportMatrix() * projection.Inverse() * view.Inverse() * world.Inverse());
	for(u32 i=0;i<n;++i){
		Vector4 tmp(Vector4(spos[i], 1.0f) * m);
		dpos[i] = Vector3(tmp) / tmp.W();
	}
	return dpos;
}

/*!
\brief [hW  XN[Wϊ
\param XN[W
\param [hr[vWFNV}gNX * r[|[g}gNX
\return [hW
\note
)
Vector3 screen(Project(wpos, WVP * graphics::System::GetViewportMatrix()));
*/
Vector3 graphics::System::Project(const Vector3 &wpos, const Matrix44 &wvp_vp)
{
	Vector4 tmp(Vector4(wpos, 1.0f) * wvp_vp);
	return Vector3(tmp) / tmp.W();
}

/*!
\brief (AABB)ʓɂ\傫̔䗦Ԃ܂(LODIȂǂɎg邩)
\param (AABB)SW
\param (AABB)a
\param [hr[vWFNV}gNX * r[|[g}gNX
\return ʓɂ䗦
)
   Vector2 ratio(ScreenRatio(AABB.Center(), AABB.Radius(), WVP * graphics::System::GetViewportMatrix()));
*/
Vector2 graphics::System::ScreenRatio(const Vector3 &center, const Vector3 &radius, const Matrix44 &wvp_vp)
{
	Vector3 mn(Vector3(LIM_MAX(f32), LIM_MAX(f32), LIM_MAX(f32)));
	Vector3 mx(Vector3(LIM_MIN(f32), LIM_MIN(f32), LIM_MIN(f32)));

	Vector3 tmp(Project(center + radius, wvp_vp));
	mn.Set(math::Min(mn.X(), tmp.X()), math::Min(mn.Y(), tmp.Y()), math::Min(mn.Z(), tmp.Z()));
	mx.Set(math::Max(mx.X(), tmp.X()), math::Max(mx.Y(), tmp.Y()), math::Max(mx.Z(), tmp.Z()));

	tmp = Project(center + Vector3(radius.X(), radius.Y(), -radius.Z()), wvp_vp);
	mn.Set(math::Min(mn.X(), tmp.X()), math::Min(mn.Y(), tmp.Y()), math::Min(mn.Z(), tmp.Z()));
	mx.Set(math::Max(mx.X(), tmp.X()), math::Max(mx.Y(), tmp.Y()), math::Max(mx.Z(), tmp.Z()));

	tmp = Project(center + Vector3(radius.X(), -radius.Y(), radius.Z()), wvp_vp);
	mn.Set(math::Min(mn.X(), tmp.X()), math::Min(mn.Y(), tmp.Y()), math::Min(mn.Z(), tmp.Z()));
	mx.Set(math::Max(mx.X(), tmp.X()), math::Max(mx.Y(), tmp.Y()), math::Max(mx.Z(), tmp.Z()));

	tmp = Project(center + Vector3(radius.X(), -radius.Y(), -radius.Z()), wvp_vp);
	mn.Set(math::Min(mn.X(), tmp.X()), math::Min(mn.Y(), tmp.Y()), math::Min(mn.Z(), tmp.Z()));
	mx.Set(math::Max(mx.X(), tmp.X()), math::Max(mx.Y(), tmp.Y()), math::Max(mx.Z(), tmp.Z()));

	tmp = Project(center + Vector3(-radius.X(), radius.Y(), radius.Z()), wvp_vp);
	mn.Set(math::Min(mn.X(), tmp.X()), math::Min(mn.Y(), tmp.Y()), math::Min(mn.Z(), tmp.Z()));
	mx.Set(math::Max(mx.X(), tmp.X()), math::Max(mx.Y(), tmp.Y()), math::Max(mx.Z(), tmp.Z()));

	tmp = Project(center + Vector3(-radius.X(), radius.Y(), -radius.Z()), wvp_vp);
	mn.Set(math::Min(mn.X(), tmp.X()), math::Min(mn.Y(), tmp.Y()), math::Min(mn.Z(), tmp.Z()));
	mx.Set(math::Max(mx.X(), tmp.X()), math::Max(mx.Y(), tmp.Y()), math::Max(mx.Z(), tmp.Z()));

	tmp = Project(center + Vector3(-radius.X(), -radius.Y(), radius.Z()), wvp_vp);
	mn.Set(math::Min(mn.X(), tmp.X()), math::Min(mn.Y(), tmp.Y()), math::Min(mn.Z(), tmp.Z()));
	mx.Set(math::Max(mx.X(), tmp.X()), math::Max(mx.Y(), tmp.Y()), math::Max(mx.Z(), tmp.Z()));

	tmp = Project(center - radius, wvp_vp);
	mn.Set(math::Min(mn.X(), tmp.X()), math::Min(mn.Y(), tmp.Y()), math::Min(mn.Z(), tmp.Z()));
	mx.Set(math::Max(mx.X(), tmp.X()), math::Max(mx.Y(), tmp.Y()), math::Max(mx.Z(), tmp.Z()));

	return Vector2(math::Abs(mx.X() - mn.X()) / static_cast<f32>(GetScreenWidth()), math::Abs(mx.Y() - mn.Y()) / static_cast<f32>(GetScreenHeight()));
}

/*!
\brief 
*/
void graphics::System::Finalize()
{
	if(fullscreen_vs) { delete fullscreen_vs; fullscreen_vs = 0; }
	if(fullscreen_ps) { delete fullscreen_ps; fullscreen_ps = 0; }
}

/*!
\breif rfIo̓foCX̏𒲂ׂ܂
\retval ĂꍇTRUE
\retval ĂȂꍇFALSE
*/
BOOL graphics::System::isVideoReady()
{
	CellVideoOutState vos;
	cellVideoOutGetState(CELL_VIDEO_OUT_PRIMARY, 0, &vos);
	return (vos.state == CELL_VIDEO_OUT_OUTPUT_STATE_ENABLED);
}

/*!
\brief w肳ꂽeNX`tXN[`悵܂
\param eNX`
*/
void graphics::System::DrawFullScreen(const TextureBase *t)
{
	static primitive::ScreenPlane screenPlane;
	if(fullscreen_vs->IsLoaded() && fullscreen_ps->IsLoaded()){
		graphics::shader::Program::AutoBegin auto_vs(*fullscreen_vs);
		graphics::shader::Program::AutoBegin auto_ps(*fullscreen_ps);
					
		{
			graphics::shader::Parameter::AutoBeginTexture auto_tex(fullscreen_ps->sampler, t);
				
			screenPlane.Begin();{
				screenPlane.Draw();
			} screenPlane.End();
		}
	}
}

/*!
\brief _[obt@̃Xbv
\param XbveNX`
\note XbveNX`𖾎IɎw肵ꍇ͂̃eNX`ɕ`悳ĂeփXbv܂
*/
void graphics::System::SwapBuffers(graphics::TextureBase *t)
{
	if(t) {
		Clear();
		DrawFullScreen(t);
	}
	psglSwap(); 
}

/*!
\brief tXN[`ɕKvȃVF[_pӂ܂
*/
void graphics::System::CreateFullScreenShader()
{
	fullscreen_vs = new graphics::shader::VertexProgram("fullscreen_vs"SH_EXT);
	fullscreen_ps = new FullScreenPS("fullscreen_ps"SH_EXT);
}