/*!
$Id: matrix44.cpp 632 2009-12-22 06:51:16Z horino $
*/
#define MATRIX44_CPP

#include "vector2.h"
#include "vector3.h"
#include "vector4.h"
#include "quaternion.h"
#include "matrix44.h"
#include "graphics/graphics.h"
using namespace indi;

Matrix44::Matrix44(const f32 *p) : _e(Vector4(p[0], p[1], p[2], p[3]),
								Vector4(p[4], p[5], p[6], p[7]),
								Vector4(p[8], p[9], p[10], p[11]),
								Vector4(p[12], p[13], p[14], p[15])) {}
Matrix44::Matrix44(f32 m00, f32 m01, f32 m02, f32 m03,
				   f32 m10, f32 m11, f32 m12, f32 m13,
				   f32 m20, f32 m21, f32 m22, f32 m23,
				   f32 m30, f32 m31, f32 m32, f32 m33) : _e(Vector4(m00, m01, m02, m03),
				   Vector4(m10, m11, m12, m13),
				   Vector4(m20, m21, m22, m23),
				   Vector4(m30, m31, m32, m33)) {}
Matrix44::Matrix44(const Quaternion &q) :_e(Aos::Matrix4::rotation(q)) {}

Matrix44 Matrix44::LookAt(const Vector3 &pos, const Vector3 &tag, const Vector3 &up){ return Matrix44(Aos::Matrix4::lookAt(Aos::Point3(pos), Aos::Point3(tag), up)); }
Matrix44 Matrix44::Rotation(const Vector3 &v3, f32 rad) { return Matrix44(Aos::Matrix4::rotation(rad, v3)); }

Matrix44 Matrix44::Scale(const Vector2 &s){ return Scale(Vector3(s.X(), s.Y(), 1.0f)); }
Matrix44 Matrix44::Scale(const Vector3 &s){ return Matrix44(Aos::Matrix4::scale(s)); }
Matrix44 Matrix44::Scale(f32 x, f32 y, f32 z) { return Scale(Vector3(x, y, z)); }
Matrix44 Matrix44::Translation(const Vector3 &t) { return Matrix44(Aos::Matrix4::translation(t)); }
Matrix44 Matrix44::Translation(f32 x, f32 y, f32 z) { return Translation(Vector3(x, y, z)); }
