/*!
$Id: quaternion.cpp 295 2009-08-20 05:16:37Z horino $
*/
#define QUATERNION_CPP

#include "vector3.h"
#include "vector4.h"
#include "quaternion.h"
#include "matrix44.h"
#include "utilities.h"

using namespace indi;

Quaternion::Quaternion(const Vector3 &v3) : _e(Aos::Quat(v3, 0.0f)) {}
Quaternion::Quaternion(const Matrix44 &m44) : _e(static_cast<Aos::Matrix3>(m44)) {}

/*!
  \brief ]ƊpxiWAjl쐬RXgN^
  \param ]iKĂKv܂j
  \param ]iWAj
*/
Quaternion::Quaternion(const Vector3 &axis, f32 rad) : _e(Aos::Quat::rotation(rad, axis)) {}

/*!
  \brief xNgfromxNgto֌悤ȉ]\l쐬RXgN^
  \param ]xNg
  \param ]xNg
  \note Zbg͐ł܂
*/
Quaternion::Quaternion(const Vector3 &from, const Vector3 &to)
{
	f32 c(from * to);	
	if(c > 0.999f)
		*this = Identity();
	else if(c < -0.999f){
		Vector3 t(from.Z(), 0.0f, -from.X());
		f32 len2(t.LengthSquared());
		if(len2 < LIM_EPSILON(f32))
			t.Set(0.0f, -from.Z(), from.Y()).Normalize();
		else 
			t /= math::Sqrt(len2);
		Set(t.X(), t.Y(), t.Z(), 0.0f);
	} else {
		Vector3 t(from ^ to);
		if(t.LengthSquared() > LIM_EPSILON(f32)){
			t.Adjust(math::Sqrt((0.5f * (1.0f - c))));
			Set(t.X(), t.Y(), t.Z(), math::Sqrt((0.5f * (1.0f + c))));
		} else 
			*this = Identity();
	}
}

Quaternion::Quaternion(const Vector4 &v4) : _e(Aos::Quat(v4)) {}

/*!
  \brief [Asb`A[iWAjl쐬RXgN^
  \param [iWAj
  \param sb`iWAj
  \param [iWAj
*/
Quaternion::Quaternion(f32 yaw, f32 pitch, f32 roll)
{	
	yaw *= 0.5f; pitch *= 0.5f; roll *= 0.5f;
	f32 ycos(math::Cos(yaw)), ysin(math::Sin(yaw));
	f32 pcos(math::Cos(pitch)), psin(math::Sin(pitch));
	f32 rcos(math::Cos(roll)), rsin(math::Sin(roll));
	Set(ycos * psin * rcos + ysin * pcos * rsin,
		-ycos * psin * rsin + ysin * pcos * rcos,
		-ysin * psin * rcos + ycos * pcos * rsin,
		ysin * psin * rsin + ycos * pcos * rcos);
}

/*!
  \brief xNglŉ]܂(l͐KĂKv܂)
  \param ]xNg
  \return ]̃xNg
*/
Vector3 Quaternion::Rotate(const Vector3 &v) const { return Vector3(Aos::rotate(*this, v)); }

/*!
  \brief l̉]Ɖ]pxiWAjԂ܂
  \param ]Ԃ܂
  \param ]pxiWAjԂ܂
*/
void Quaternion::ToAxisRadian(Vector3 &axis, f32 &rad) const
{
	rad = math::Acos(math::Saturate(W())) * 2.0f;
	axis = *this;
}
