/*!
$Id: main.cpp 566 2009-11-27 02:41:47Z horino $

u[

*/
#include "stdafx.h"

#define MAIN_CPP

#include "indi.h"
#include "graphics/graphics.h"
#include "file/file_manager.h"
#include "input/input.h"
#include "camera/debugcamera.h"
#include "mesh/scenes.h"
#include "mesh/scenes_texture.h"
#include "graphics/shader/shader.h"
#include "primitive/primitive.h"
#include "graphics/rendertarget.h"
#include "graphics/framebuffer.h"
using namespace indi;

#ifdef PS3
//#define ROOT "/dev_bdvd/PS3_GAME/USRDIR/"
#define ROOT "/app_home/samples/"
#else
#define ROOT "game:\\"
#endif

int main(int argc, char *argv[])
{
	//< 
	System::Initialize(ROOT"media"SEPARATOR);

	//< pbh͗p
	Input input;

	//< J
	DebugCamera camera(Vector3(0.0f, 15.0f, 200.0f), Vector3::Zero(), Vector3::Up(), Matrix44::Perspective(INDI_PI * 0.25f, graphics::System::GetAspectRatio(), 10.f, 1000.0f));

	//< eNX`fobO\p̃v[
	primitive::ScreenPlane plane;

	//< f
	mesh::ScenesTextureFile *texture(new mesh::ScenesTextureFile("duck.tex"));
	mesh::ScenesFile *model(new mesh::ScenesFile("duck.scn"));

	//< VF[_
	graphics::shader::VertexProgram *vs(new graphics::shader::VertexProgram("textured_vs"SH_EXT));
	graphics::shader::PixelProgram *ps(new graphics::shader::PixelProgram("textured_ps"SH_EXT));
	//< u[VF[_
	graphics::shader::VertexProgram *blur_vs(new graphics::shader::VertexProgram("blur_vs"SH_EXT));
	graphics::shader::PixelProgram *blurH_ps(new graphics::shader::PixelProgram("blurH_ps"SH_EXT)); //< u[
	graphics::shader::PixelProgram *blurV_ps(new graphics::shader::PixelProgram("blurV_ps"SH_EXT)); //< u[

	//< _[eNX`A_[^[QbgAt[obt@
	u32 width(graphics::System::GetScreenWidth()), height(graphics::System::GetScreenHeight());
	graphics::RenderTexture *color_map(new graphics::RenderTexture(width, height)); 
	graphics::RenderTarget *color_target(new graphics::RenderTarget(color_map));
	graphics::DepthTarget *depth_target(new graphics::DepthTarget(width, height));
	graphics::FrameBuffer *color_buffer(new graphics::FrameBuffer(color_target, depth_target));

	graphics::RenderTexture *blur_map(new graphics::RenderTexture(width, height));
	graphics::RenderTarget *blur_target(new graphics::RenderTarget(blur_map));
	graphics::FrameBuffer *blur_buffer(new graphics::FrameBuffer(blur_target));

	graphics::System::ClearColor(127, 127, 255);

	//< Q[[v
	while(1){
		//< 񓯊ǂݍ݂̏
		if(file::FileManager::Get())
			file::FileManager::Get()->Update();

		//< pbh
		input.Update();

		//< fobOJ
		if(input.Current(0).IsValid())
			camera.Control(input.Current(0));
		//< J̍XV
		camera.Update();		

		//< t[obt@ւ̕`(ȉ̕`͉ʂɂ͕\܂)
		{
			graphics::FrameBuffer::AutoBegin auto_fb(color_buffer); //< `挋ʂcolor_map

			graphics::System::Clear();

			//< VF[_ǂݍ݊ĂȂƂ`͂łȂ
			if(vs->IsLoaded() && ps->IsLoaded()){
				//< fƃeNX`ǂݍ܂ĂΕ`
				if(model->IsLoaded() && texture->IsLoaded()){
					graphics::shader::Program::AutoBegin auto_vs(*vs);
					graphics::shader::Program::AutoBegin auto_ps(*ps);

					//< {͖t[Ŏ擾̂͗ǂȂ(Tvł͉ǐd)
					graphics::shader::Parameter wvp(vs->GetParameter("WorldViewProjection"));
					graphics::shader::Parameter smp(ps->GetParameter("Sampler"));
					if(texture->GetTextureNum()) {
						wvp = Matrix44::Identity() * camera.ViewProjection();
						graphics::shader::Parameter::AutoBeginTexture auto_tex(smp, texture->GetTexture(0));

						model->Draw();
					}
				}
			}
		}

		if(blur_vs->IsLoaded() && blurH_ps->IsLoaded() && blurV_ps->IsLoaded() && !input.Current(0).A()){
			//< u[
			//< IWi摜(color_map)֐u[Ȃ猋ʂblur_map֏o͂܂
			{
				graphics::FrameBuffer::AutoBegin auto_fb(blur_buffer); //< `挋ʂblur_map
					
				graphics::System::Clear();
					
				graphics::shader::Program::AutoBegin auto_vs(*blur_vs);
				graphics::shader::Program::AutoBegin auto_ps(*blurH_ps);
				graphics::shader::Parameter smp(blurH_ps->GetParameter("Sampler"));
				graphics::shader::Parameter bsize(blurH_ps->GetParameter("BlurSize"));
				{
					graphics::shader::Parameter::AutoBeginTexture auto_tex(smp, color_map);
					bsize = 0.5f;

					plane.Begin();{
						plane.Draw();
					} plane.End();
				}
			}

			//< u[
			//< ōu[̉摜(blur_map)IWi摜ƂĐu[Ȃcolor_map֏o͂܂
			{
				graphics::FrameBuffer::AutoBegin auto_bf(color_buffer); //< `挋ʂcolor_map
					
				graphics::System::Clear();

				graphics::shader::Program::AutoBegin auto_vs(*blur_vs);
				graphics::shader::Program::AutoBegin auto_ps(*blurV_ps);
				graphics::shader::Parameter smp(blurV_ps->GetParameter("Sampler"));
				graphics::shader::Parameter bsize(blurV_ps->GetParameter("BlurSize"));
				{
					graphics::shader::Parameter::AutoBeginTexture auto_tex(smp, blur_map);
					bsize = 0.5f;

					plane.Begin();{
						plane.Draw();
					} plane.End();
				}
			}
		}

		graphics::System::Clear();

		//< u[̂ŏIcolor_maptXN[`
		//< ŏI摜(color_map)͊ɋ܂Ă̂ŁA킴킴obNobt@ւ̕`͍s킸ASwapBuffers()֓nĒڕ\OK
		graphics::System::SwapBuffers(color_map);
	}

	delete blur_buffer;
	delete blur_target;
	delete blur_map;

	delete color_buffer;
	delete depth_target;
	delete color_target;
	delete color_map;

	delete ps;
	delete vs;

	delete blurV_ps;
	delete blurH_ps;
	delete blur_vs;

	delete model;
	delete texture;

	System::Finalize();
}
