/*!
$Id: main.cpp 642 2009-12-28 05:16:12Z horino $

NXizjV~[VTv
*/
#include "stdafx.h"

#define MAIN_CPP

#include "indi.h"
#include "graphics/graphics.h"
#include "file/file_manager.h"
#include "input/input.h"
#include "camera/debugcamera.h"
#include "mesh/scenes.h"
#include "graphics/shader/shader.h"
#include "physics/physics.h"
#include "timer.h"
#include "debug_stop.h"
#include "primitive/primitive.h"
using namespace indi;

#ifdef PS3
//#define ROOT "/dev_bdvd/PS3_GAME/USRDIR/"
#define ROOT "/app_home/samples/"
#else
#define ROOT "game:\\"
#endif

int main(int argc, char *argv[])
{
	//< 
	System::Initialize(ROOT"media"SEPARATOR);

	//< pbh͗p
	Input input;

	//< J
	DebugCamera camera(Vector3(0.0f, 15.0f, 20.0f), Vector3::Zero(), Vector3::Up(), Matrix44::Perspective(INDI_PI * 0.25f, graphics::System::GetAspectRatio(), 1.0f, 500.0f));

	//< f
	mesh::ScenesFile *sphere(new mesh::ScenesFile("sphere.scn"));
	primitive::SegmentZ seg;

	//< VF[_
	graphics::shader::VertexProgram *vs(new graphics::shader::VertexProgram("color_vs"SH_EXT));
	graphics::shader::PixelProgram *ps(new graphics::shader::PixelProgram("color_ps"SH_EXT));

	graphics::System::ClearColor(127, 127, 255);

	//< _\
	primitive::ArrowXYZ origin;

#ifdef _DEBUG
	//< fobOXgbv
	DebugStop debug_stop;
#endif

	//< vZ[h
	physics::World::Create();
	u32 cloth_flag(0);
	//u32 cloth_flag((1 << physics::Cloth::NoShearSpring) | (1 << physics::Cloth::NoBendSpring));
	physics::Cloth *cloth(new physics::Cloth(1.0f, 8, 8, 1.0f, 1.0f, cloth_flag));
	//< Œ肷Q_
	cloth->GetParticle(0)->SetStatic(TRUE);
	cloth->GetParticle(cloth->GetWidth() - 1)->SetStatic(TRUE);
	physics::World::Get()->Add(cloth);

	//< Q[[v
	while(1){
		//< 񓯊ǂݍ݂̏
		if(file::FileManager::Get())
			file::FileManager::Get()->Update();

		//< pbh
		input.Update();

		//< fobOJ
		if(input.Current(0).IsValid())
			camera.Control(input.Current(0));

		//< J̍XV
		camera.Update();

#ifdef _DEBUG
		//< fobOXgbv̑
		debug_stop.Control(input.Previous(0), input.Current(0));
#endif

		//< žŒ_𒍎_̈ړɔAړ
		cloth->GetParticle(0)->SetPosition(camera.Target() - camera.Right() * 4.0f);
		cloth->GetParticle(cloth->GetWidth() - 1)->SetPosition(camera.Target() + camera.Right() * 4.0f);

		//< A{^ŕz̃Zbg
		if(input.Current(0).A()) 
			cloth->Reset();

		//< V~[V̍XV
		physics::World::Get()->Update(debug_stop.IsStop() ? 0 : Timer::GetElapsedSecond());

		graphics::System::BeginBackBuffer(); {
			graphics::System::Clear();

			if(vs->IsLoaded() && ps->IsLoaded()){
				graphics::shader::Program::AutoBegin auto_vs(*vs);
				graphics::shader::Program::AutoBegin auto_ps(*ps);

				graphics::shader::Parameter color(ps->GetParameter("Color"));
				graphics::shader::Parameter wvp(vs->GetParameter("WorldViewProjection"));

				//< Obh̕`
				color = color::Red;
				for(u32 i=0;i<10;++i) {
					wvp = Matrix44::TranslationX(static_cast<f32>(i) * 5.0f) * camera.ViewProjection();
					origin.Begin(); origin.Draw(); origin.End();
				}
				color = color::Green;
				for(u32 i=0;i<10;++i) {
					wvp = Matrix44::TranslationY(static_cast<f32>(i) * 5.0f) * camera.ViewProjection();
					origin.Begin(); origin.Draw(); origin.End();
				}
				color = color::Blue;
				for(u32 i=0;i<10;++i) {
					wvp = Matrix44::TranslationZ(static_cast<f32>(i) * 5.0f) * camera.ViewProjection();
					origin.Begin(); origin.Draw(); origin.End();
				}

				if(sphere->IsLoaded()){
					//< _̕`
					for(u32 i=0;i<cloth->GetParticleNum();++i){
						color = color::Green;
						if(cloth->GetParticle(i)->IsStatic())
							color = color::Yellow;
						const physics::Particle *p(cloth->GetParticle(i));
						wvp = Matrix44::Scale(0.1f) * Matrix44::Translation(p->GetPosition()) * camera.ViewProjection();
						sphere->Draw();
					}

					//< ΂˂̕`
					color = color::Red;
					for(u32 i=0;i<cloth->GetSpringNum();++i){
						const physics::Spring *s(cloth->GetSpring(i));
						if(s && s->GetA() && s->GetB()){
							Vector3 dir(s->GetA()->GetPosition() - s->GetB()->GetPosition());
							f32 len(dir.Length());
							wvp = Matrix44::Scale(len) * Matrix44(Quaternion(Vector3::UnitZ(), dir / len)) * Matrix44::Translation(s->GetB()->GetPosition()) * camera.ViewProjection();
							seg.Begin(); seg.Draw(); seg.End();
						}
					}
				}
			}
		} graphics::System::EndBackBuffer();
		graphics::System::SwapBuffers();

#ifdef _DEBUG
		if(!debug_stop.IsStop())
			++AnimationTimer::current;
#else
		++AnimationTimer::current;
#endif
	}

	delete cloth;

	physics::World::Delete();

	delete ps;
	delete vs;

	delete sphere;

	System::Finalize();
}
