/*!
$Id: main.cpp 652 2010-01-08 09:54:38Z horino $

RWTv
*/
#include "stdafx.h"

#define MAIN_CPP

#include "indi.h"
#include "graphics/graphics.h"
#include "file/file_manager.h"
#include "input/input.h"
#include "camera/debugcamera.h"
#include "graphics/shader/shader.h"
#include "primitive/primitive.h"
#include "colors.h"
#include "quaternion.h"
using namespace indi;

#ifdef PS3
//#define ROOT "/dev_bdvd/PS3_GAME/USRDIR/"
#define ROOT "/app_home/samples/"
#else
#define ROOT "game:\\"
#endif

int main(int argc, char *argv[])
{
	//< 
	System::Initialize(ROOT"media"SEPARATOR);

	//< pbh͗p
	Input input;

	//< J
	DebugCamera camera(Vector3(0.0f, 15.0f, 20.0f), Vector3::Zero(), Vector3::Up(), Matrix44::Perspective(INDI_PI * 0.25f, graphics::System::GetAspectRatio(), 1.0f, 500.0f));

	//< VF[_
	graphics::shader::VertexProgram *vs(new graphics::shader::VertexProgram("color_vs"SH_EXT));
	graphics::shader::PixelProgram *ps(new graphics::shader::PixelProgram("color_ps"SH_EXT));

	graphics::System::ClearColor(127, 127, 255);

	primitive::ArrowX axis_x;
	primitive::ArrowY axis_y;
	primitive::ArrowZ axis_z;

	primitive::Box box;
	primitive::WireBox w_box;

	primitive::Capsule capsule;
	primitive::WireCapsule w_capsule;

	primitive::Cylinder cylinder;
	primitive::WireCylinder w_cylinder;

	primitive::Sphere sphere;
	primitive::WireSphere w_sphere;

	primitive::PlaneZX plane;
	primitive::WirePlaneZX w_plane;

	primitive::SegmentZ segment;

	enum PRIM {
		Point,
		AABB,
		Sphere,
		Cylinder,
		Capsule,
		Segment,
		OBB,
		Plane,

		PrimNum,
	};
	u32 target0(Point);
	u32 target1(Point);
	Vector2 rot(Vector2::Zero());
	Vector3 pre_pos(camera.Target());

	//< Q[[v
	while(1){
		//< 񓯊ǂݍ݂̏
		if(file::FileManager::Get())
			file::FileManager::Get()->Update();

		//< pbh
		input.Update();

		//< fobOJ
		if(input.Current(0).IsValid())
			camera.Control(input.Current(0));
		//< J̍XV
		camera.Update();

		//< `{^ŃRW^[QbgO̕ύX
		if(!input.Previous(0).A() && input.Current(0).A()) { ++target0; target0 %= PrimNum; }
		//< a{^ŃRW^[QbgP̕ύX
		if(!input.Previous(0).B() && input.Current(0).B()) { ++target1; target1 %= PrimNum; }

		//< \L[Ń^[Qbg1̉]
		if(input.Current(0).Down()) { rot.SetX(rot.X() + 1.0f); while(rot.X() > 360.0f) rot.SetX(rot.X() - 360.0f); }
		if(input.Current(0).Up()) { rot.SetX(rot.X() - 1.0f); while(rot.X() < 0.0f) rot.SetX(rot.X() + 360.0f); }
		if(input.Current(0).Right()) { rot.SetY(rot.Y() + 1.0f); while(rot.Y() > 360.0f) rot.SetY(rot.Y() - 360.0f); }
		if(input.Current(0).Left()) { rot.SetY(rot.Y() - 1.0f); while(rot.Y() < 0.0f) rot.SetY(rot.Y() + 360.0f); }
		Quaternion rotation_q(Quaternion::RotationX(math::ToRadian(rot.X())) * Quaternion::RotationY(math::ToRadian(rot.Y())));
		Matrix44 rotation(rotation_q);
		Vector3 position(camera.Target());
		Matrix44 translation(Matrix44::Translation(position));
		Vector3 vel(position - pre_pos);

		//< ^[Qbg̃ftHg̕`F
		Vector4 color0(color::Green);
		Vector4 color1(color::Yellow);

		//< _󂯎p
		Vector3 point(Vector3::Zero());
		//< 󂯎p
		Vector3 sline(Vector3::Zero()), dline(Vector3::Zero());
		//< Փ˃p[^󂯎p
		f32 t0(-1.0f), t1(-1.0f);
		//< ŋߐړ_󂯎p
		Vector3 close0(Vector3::Zero()), close1(Vector3::Zero());
		f32 close_len(0.0f);

		//////////////////////////////////////////////////////////////////////////////////////////////
		//< RW̌oōs܂
		switch(target0){
		case Point:
			switch(target1){
		    case AABB:
				//< AABBƓ_Ƃ̏Փˌo
				if(Collision::Detection::AABBPoint(position, Vector3::Half(), Vector3::Zero()))
					color0 = color1 = color::White;				

				//< AABBƓ_Ƃ̍ŋߐړ_
				close0 = Vector3::Zero();
				close1 = Collision::ClosestPoint::AABBPoint(position, Vector3::Half(), Vector3::Zero());
				close_len = (close1 - close0).Length();
				break;
			case Plane:
				{
					Vector3 n(rotation_q.Rotate(Vector3::UnitY()));
					f32 d(n * position);
					//< _ƕʂƂ̍ŋߐړ_
					close0 = Vector3::Zero();
					close1 = Collision::ClosestPoint::PlanePoint(n, d, Vector3::Zero());
					close_len = (close1 - close0).Length();
				}
				break;
			case Segment:
				{
					Vector3 s0(position);
					Vector3 s1(rotation_q.Rotate(Vector3::UnitZ()) + position);
					//< _ƐƂ̍ŋߐړ_
					close0 = Vector3::Zero();
					close1 = Collision::ClosestPoint::PointSegment(Vector3::Zero(), s0, s1);
					close_len = (close1 - close0).Length();
				}
				break;
			case OBB:
				{
					Vector3 axis_x(rotation_q.Rotate(Vector3::UnitX()));
					Vector3 axis_y(rotation_q.Rotate(Vector3::UnitY()));
					Vector3 axis_z(rotation_q.Rotate(Vector3::UnitZ()));
					//< _OBBƂ̍ŋߐړ_
					close0 = Vector3::Zero();
					close1 = Collision::ClosestPoint::OBBPoint(position, Vector3::Half(), axis_x, axis_y, axis_z, Vector3::Zero());
					close_len = (close1 - close0).Length();
				}
				break;
			}
			break;
		case AABB:
			switch(target1){
		    case Point:
				//< AABBƓ_Ƃ̏Փˌo
				if(Collision::Detection::AABBPoint(Vector3::Zero(), Vector3::Half(), position))
					color0 = color1 = color::White;				

				//< AABBƓ_Ƃ̍ŋߐړ_
				close0 = Collision::ClosestPoint::AABBPoint(Vector3::Zero(), Vector3::Half(), position);
				close1 = position;
				close_len = (close1 - close0).Length();
				break;
		    case AABB:
				//< ĂȂAABBm̏Փˌo
//				if(Collision::Detection::AABBAABB(Vector3::Zero(), Vector3::Half(), position, Vector3::Half()))
//					color0 = color1 = color::White;
				//< ĂAABBm̏Փˌo
//				if(Collision::Detection::AABBAABB(Vector3::Zero(), Vector3::Half(), Vector3::Zero(), position, Vector3::Half(), vel))
//					color0 = color1 = color::White;
				//< ĂAABBm̏Փˌo(Փ˃p[^߂)
				if(Collision::Detection::AABBAABB(Vector3::Zero(), Vector3::Half(), Vector3::Zero(), position, Vector3::Half(), vel, t0))
					color0 = color1 = color::White;				
				break;
			case Sphere:
				//< AABBƋ̏Փˌo
//				if(Collision::Detection::AABBSphere(Vector3::Zero(), Vector3::Half(), position, 0.5f))
//					color0 = color1 = color::White;				
				//< AABBƓĂ鋅̏Փˌo
				if(Collision::Detection::AABBSphere(Vector3::Zero(), Vector3::Half(), position, 0.5f, vel))
					color0 = color1 = color::White;				
				break;
			case Plane:
				{
					Vector3 n(rotation_q.Rotate(Vector3::UnitY()));
					f32 d(n * position);
					//< yʂAABBƂ̏Փˌo
//					if(Collision::Detection::AABBPlane(Vector3::Zero(), Vector3::Half(), n, d))
//						color0 = color1 = color::White;				
					//< \bhʂAABBƂ̏Փˌo
//					if(Collision::Detection::AABBSolidPlane(Vector3::Zero(), Vector3::Half(), n, d))
//						color0 = color1 = color::White;				
					//< yʂƓĂAABBƂ̏Փˌo
//					if(Collision::Detection::AABBPlane(Vector3::Zero(), Vector3::Half(), Vector3::Zero(), n, d))
//						color0 = color1 = color::White;				
					//< \bhʂƓĂAABBƂ̏Փˌo
//					if(Collision::Detection::AABBSolidPlane(Vector3::Zero(), Vector3::Half(), Vector3::Zero(), n, d))
//						color0 = color1 = color::White;				
					//< yʂƓĂAABBƂ̏Փˌo(Փ˃p[^߂)
					if(Collision::Detection::AABBPlane(Vector3::Zero(), Vector3::Half(), Vector3::Zero(), n, d, t0))
						color0 = color1 = color::White;				
					//< \bhʂƓĂAABBƂ̏Փˌo(Փ˃p[^߂)
//					if(Collision::Detection::AABBSolidPlane(Vector3::Zero(), Vector3::Half(), Vector3::Zero(), n, d, t0))
//						color0 = color1 = color::White;				
				}
				break;
			}
			break;
		case Sphere:
			switch(target1){
		    case AABB:
				//< AABBƋ̏Փˌo
//				if(Collision::Detection::AABBSphere(position, Vector3::Half(), Vector3::Zero(), 0.5f))
//					color0 = color1 = color::White;				
				//< AABBƓĂ鋅̏Փˌo
				if(Collision::Detection::AABBSphere(position, Vector3::Half(), Vector3::Zero(), 0.5f, Vector3::Zero()))
					color0 = color1 = color::White;				
				break;
		    case Sphere:
				//< ĂȂ̏Փˌo
				if(Collision::Detection::SphereSphere(Vector3::Zero(), 0.5f, position, 0.5f))
					color0 = color1 = color::White;
				//< Ă鋅̏Փˌo
//				if(Collision::Detection::SphereSphere(Vector3::Zero(), 0.5f, Vector3::Zero(), position, 0.5f, vel))
//					color0 = color1 = color::White;
				//< Ă鋅̏Փˌo(Փ˃p[^߂)
//				if(Collision::Detection::SphereSphere(Vector3::Zero(), 0.5f, Vector3::Zero(), position, 0.5f, vel, t0))
//					color0 = color1 = color::White;
				break;
			case Capsule:
				{
					Vector3 c0(rotation_q.Rotate(Vector3::UnitY() * 0.5f) + position);
					Vector3 c1(rotation_q.Rotate(-Vector3::UnitY() * 0.5f) + position);
					//< JvZƋ̏Փˌo
					if(Collision::Detection::CapsuleSphere(c0, c1, 0.5f, Vector3::Zero(), 0.5f))
						color0 = color1 = color::White;
				}
				break;
			case Plane:
				{
					Vector3 n(rotation_q.Rotate(Vector3::UnitY()));
					f32 d(n * position);
					//< yʂƋƂ̏Փˌo
//					if(Collision::Detection::PlaneSphere(n, d, Vector3::Zero(), 0.5f))
//						color0 = color1 = color::White;
					//< \bhʂƋƂ̏Փˌo
//					if(Collision::Detection::SolidPlaneSphere(n, d, Vector3::Zero(), 0.5f))
//						color0 = color1 = color::White;
					//< yʂƓĂ鋅Ƃ̏Փˌo
//					if(Collision::Detection::PlaneSphere(n, d, Vector3::Zero(), 0.5f, Vector3::Zero()))
//						color0 = color1 = color::White;
					//< yʂƓĂ鋅Ƃ̏Փˌo(Փ˃p[^߂)
					if(Collision::Detection::PlaneSphere(n, d, Vector3::Zero(), 0.5f, Vector3::Zero(), t0))
						color0 = color1 = color::White;
				}
				break;
			case OBB:
				{
					Vector3 axis_x(rotation_q.Rotate(Vector3::UnitX()));
					Vector3 axis_y(rotation_q.Rotate(Vector3::UnitY()));
					Vector3 axis_z(rotation_q.Rotate(Vector3::UnitZ()));
					//< OBBƋƂ̏Փˌo
					if(Collision::Detection::OBBSphere(position, Vector3::Half(), axis_x, axis_y, axis_z, Vector3::Zero(), 0.5f))
						color0 = color1 = color::White;
				}
				break;
			case Segment:
				{
					Vector3 s0(position);
					Vector3 s1(rotation_q.Rotate(Vector3::UnitZ()) + position);
					//< ƋƂ̏Փˌo  (ł͂ȂČȂ̂Œ!!)
//					if(Collision::Detection::RaySphere(s0, s1 - s0, Vector3::Zero(), 0.5f))
//						color0 = color1 = color::White;
					//< ƋƂ̏Փˌo(Փ˃p[^߂)  (ł͂ȂČȂ̂Œ!!)
//					if(Collision::Detection::RaySphere(s0, s1 - s0, Vector3::Zero(), 0.5f, t0))
//						color0 = color1 = color::White;
					//< ƋƂ̏Փˌo
					if(Collision::Detection::SegmentSphere(s0, s1, Vector3::Zero(), 0.5f))
						color0 = color1 = color::White;
					//< ƋƂ̏Փˌo(Փ˃p[^߂)
//					if(Collision::Detection::SegmentSphere(s0, s1, Vector3::Zero(), 0.5f, t0))
//						color0 = color1 = color::White;
				}
				break;
			}
			break;
		case Cylinder:
			switch(target1){
		    case Segment:
				{
					Vector3 s0(position);
					Vector3 s1(rotation_q.Rotate(Vector3::UnitZ()) + position);
					//< V_Ɛ̏Փˌo
//					if(Collision::Detection::CylinderSegment(Vector3::UnitY() * 0.5f, -Vector3::UnitY() * 0.5f, 0.5f, s0, s1))
//						color0 = color1 = color::White;
					//< V_Ɛ̏Փˌo(Փ˃p[^߂)
					if(Collision::Detection::CylinderSegment(Vector3::UnitY() * 0.5f, -Vector3::UnitY() * 0.5f, 0.5f, s0, s1, t0))
						color0 = color1 = color::White;
				}
				break;
			}
			break;
		case Capsule:
			switch(target1){
		    case Sphere:
				//< JvZƋ̏Փˌo
				if(Collision::Detection::CapsuleSphere(Vector3::UnitY() * 0.5f, -Vector3::UnitY() * 0.5f, 0.5f, position, 0.5f))
					color0 = color1 = color::White;
				break;
		    case Capsule:
				{
					Vector3 c0(rotation_q.Rotate(Vector3::UnitY() * 0.5f) + position);
					Vector3 c1(rotation_q.Rotate(-Vector3::UnitY() * 0.5f) + position);
					//< JvZm̏Փˌo
					if(Collision::Detection::CapsuleCapsule(Vector3::UnitY() * 0.5f, -Vector3::UnitY() * 0.5f, 0.5f, c0, c1, 0.5f))
						color0 = color1 = color::White;
				}
				break;
			case Segment:
				{
					Vector3 s0(position);
					Vector3 s1(rotation_q.Rotate(Vector3::UnitZ()) + position);
					//< JvZƐ̏Փˌo
					if(Collision::Detection::CapsuleSegment(Vector3::UnitY() * 0.5f, -Vector3::UnitY() * 0.5f, 0.5f, s0, s1))
						color0 = color1 = color::White;
					//< JvZƐ̏Փˌo(Փ˃p[^߂)
//					if(Collision::Detection::CapsuleSegment(Vector3::UnitY() * 0.5f, -Vector3::UnitY() * 0.5f, 0.5f, s0, s1, t0))
//						color0 = color1 = color::White;
				}
				break;
			}
			break;
		case Segment:
			switch(target1){
		    case Sphere:
				//< ƋƂ̏Փˌo  (ł͂ȂČȂ̂Œ!!)
//				if(Collision::Detection::RaySphere(Vector3::Zero(), Vector3::UnitZ(), position, 0.5f))
//					color0 = color1 = color::White;
				//< ƋƂ̏Փˌo(Փ˃p[^߂)  (ł͂ȂČȂ̂Œ!!)
//				if(Collision::Detection::RaySphere(Vector3::Zero(), Vector3::UnitZ(), position, 0.5f, t0))
//					color0 = color1 = color::White;
				//< ƋƂ̏Փˌo
				if(Collision::Detection::SegmentSphere(Vector3::Zero(), Vector3::UnitZ(), position, 0.5f))
					color0 = color1 = color::White;
				//< ƋƂ̏Փˌo(Փ˃p[^߂)
//				if(Collision::Detection::SegmentSphere(Vector3::Zero(), Vector3::UnitZ(), position, 0.5f, t0))
//					color0 = color1 = color::White;
				break;
		    case Point:
				//< _ƐƂ̍ŋߐړ_
				close0 = Collision::ClosestPoint::PointSegment(position, Vector3::Zero(), Vector3::UnitZ());
				close1 = position;
				close_len = (close1 - close0).Length();
				break;
		    case Capsule:
				{
					Vector3 c0(rotation_q.Rotate(Vector3::UnitY() * 0.5f) + position);
					Vector3 c1(rotation_q.Rotate(-Vector3::UnitY() * 0.5f) + position);
					//< JvZƐ̏Փˌo
					if(Collision::Detection::CapsuleSegment(c0, c1, 0.5f, Vector3::Zero(), Vector3::UnitZ()))
						color0 = color1 = color::White;
					//< JvZƐ̏Փˌo(Փ˃p[^߂)
//					if(Collision::Detection::CapsuleSegment(c0, c1, 0.5f, Vector3::Zero(), Vector3::UnitZ(), t0))
//						color0 = color1 = color::White;
				}
				break;
			case Plane:
				{
					Vector3 n(rotation_q.Rotate(Vector3::UnitY()));
					f32 d(n * position);
					//< ʂƐƂ̏Փ˂o
					if(Collision::Detection::PlaneSegment(n, d, Vector3::Zero(), Vector3::UnitZ()))
						color0 = color1 = color::White;
				}
				break;
			case Segment:
				//< m̍ŋߐړ_
				{
					Vector3 s0(position);
					Vector3 s1(rotation_q.Rotate(Vector3::UnitZ()) + position);
					Collision::ClosestPoint::SegmentSegment(Vector3::Zero(), Vector3::UnitZ(), s0, s1, close0, close1);
					close_len = (close1 - close0).Length();
				}
				break;
			case Cylinder:
				{
					Vector3 c0(rotation_q.Rotate(Vector3::UnitY() * 0.5f) + position);
					Vector3 c1(rotation_q.Rotate(-Vector3::UnitY() * 0.5f) + position);
					//< V_Ɛ̏Փˌo
//					if(Collision::Detection::CylinderSegment(c0, c1, 0.5f, Vector3::Zero(), Vector3::UnitZ()))
//						color0 = color1 = color::White;
					//< V_Ɛ̏Փˌo(Փ˃p[^߂)
					if(Collision::Detection::CylinderSegment(c0, c1, 0.5f, Vector3::Zero(), Vector3::UnitZ(), t0))
						color0 = color1 = color::White;
				}
				break;
			}
			break;
		case OBB:
			switch(target1){
		    case Sphere:
				//< OBBƋƂ̏Փˌo
				if(Collision::Detection::OBBSphere(Vector3::Zero(), Vector3::Half(), Vector3::UnitX(), Vector3::UnitY(), Vector3::UnitZ(), position, 0.5f))
					color0 = color1 = color::White;
				break;
		    case Plane:
				{
					Vector3 n(rotation_q.Rotate(Vector3::UnitY()));
					f32 d(n * position);
					//< ʂOBBƂ̏Փ˂o
//					if(Collision::Detection::OBBPlane(Vector3::Zero(), Vector3::Half(), Vector3::UnitX(), Vector3::UnitY(), Vector3::UnitZ(), n, d))
//						color0 = color1 = color::White;
					//< ʂƓĂOBBƂ̏Փ˂o
//					if(Collision::Detection::OBBPlane(Vector3::Zero(), Vector3::Half(), Vector3::UnitX(), Vector3::UnitY(), Vector3::Zero(), Vector3::UnitZ(), n, d))
//						color0 = color1 = color::White;
					//< ʂƓĂOBBƂ̏Փ˂o(Փ˃p[^߂)
					if(Collision::Detection::OBBPlane(Vector3::Zero(), Vector3::Half(), Vector3::UnitX(), Vector3::UnitY(), Vector3::UnitZ(), Vector3::Zero(), n, d, t0))
						color0 = color1 = color::White;
				}
				break;
			case OBB:
				{
					Vector3 axis_x(rotation_q.Rotate(Vector3::UnitX()));
					Vector3 axis_y(rotation_q.Rotate(Vector3::UnitY()));
					Vector3 axis_z(rotation_q.Rotate(Vector3::UnitZ()));
					//< OBBm̏Փ˂o
					if(Collision::Detection::OBBOBB(Vector3::Zero(), Vector3::Half(), Vector3::UnitX(), Vector3::UnitY(), Vector3::UnitZ(), position, Vector3::Half(), axis_x, axis_y, axis_z))
						color0 = color1 = color::White;
				}
				break;
			}
			break;
		case Plane:
			switch(target1){
		    case Point:
				//< _ƕʂƂ̍ŋߐړ_
				close0 = Collision::ClosestPoint::PlanePoint(Vector3::UnitY(), 0.0f, position);
				close1 = position;
				close_len = (close1 - close0).Length();
				break;
		    case AABB:
					//< yʂAABBƂ̏Փˌo
//					if(Collision::Detection::AABBPlane(position, Vector3::Half(), Vector3::UnitY(), 0.0f))
//						color0 = color1 = color::White;				
					//< \bhʂAABBƂ̏Փˌo
//					if(Collision::Detection::AABBSolidPlane(position, Vector3::Half(), Vector3::UnitY(), 0.0f))
//						color0 = color1 = color::White;				
					//< yʂƓĂAABBƂ̏Փˌo
//					if(Collision::Detection::AABBPlane(position, Vector3::Half(), Vector3::Zero(), Vector3::UnitY(), 0.0f))
//						color0 = color1 = color::White;				
					//< \bhʂƓĂAABBƂ̏Փˌo
//					if(Collision::Detection::AABBSolidPlane(position, Vector3::Half(), Vector3::Zero(), Vector3::UnitY(), 0.0f))
//						color0 = color1 = color::White;				
					//< yʂƓĂAABBƂ̏Փˌo(Փ˃p[^߂)
					if(Collision::Detection::AABBPlane(position, Vector3::Half(), Vector3::Zero(), Vector3::UnitY(), 0.0f, t0))
						color0 = color1 = color::White;				
					//< \bhʂƓĂAABBƂ̏Փˌo(Փ˃p[^߂)
//					if(Collision::Detection::AABBSolidPlane(position, Vector3::Half(), Vector3::Zero(), Vector3::UnitY(), 0.0f, t0))
//						color0 = color1 = color::White;				
					break;
		    case Sphere:
				//< yʂƋƂ̏Փˌo
//				if(Collision::Detection::PlaneSphere(Vector3::UnitY(), 0.0f, position, 0.5f))
//					color0 = color1 = color::White;
				//< \bhʂƋƂ̏Փˌo
//				if(Collision::Detection::SolidPlaneSphere(Vector3::UnitY(), 0.0f, position, 0.5f))
//					color0 = color1 = color::White;
				//< yʂƓĂ鋅Ƃ̏Փˌo
//				if(Collision::Detection::PlaneSphere(Vector3::UnitY(), 0.0f, position, 0.5f, vel))
//					color0 = color1 = color::White;
				//< yʂƓĂ鋅Ƃ̏Փˌo(Փ˃p[^߂)
				if(Collision::Detection::PlaneSphere(Vector3::UnitY(), 0.0f, position, 0.5f, vel, t0))
					color0 = color1 = color::White;
				break;
			case Plane:
				//< ʂƕʂƂ̏Փ˂o
//				if(Collision::Detection::PlanePlane(Vector3::UnitY(), 0.0f, rotation_q.Rotate(Vector3::UnitY()), rotation_q.Rotate(Vector3::UnitY()) * position))
//					color0 = color1 = color::White;
				//< ʂƕʂƂ̏Փ˂o(߂)
				if(Collision::Detection::PlanePlane(Vector3::UnitY(), 0.0f, rotation_q.Rotate(Vector3::UnitY()), rotation_q.Rotate(Vector3::UnitY()) * position, sline, dline))
					color0 = color1 = color::White;
				break;
			case OBB:
				{
					Vector3 axis_x(rotation_q.Rotate(Vector3::UnitX()));
					Vector3 axis_y(rotation_q.Rotate(Vector3::UnitY()));
					Vector3 axis_z(rotation_q.Rotate(Vector3::UnitZ()));
					//< ʂOBBƂ̏Փ˂o
//					if(Collision::Detection::OBBPlane(position, Vector3::Half(), axis_x, axis_y, axis_z, Vector3::UnitY(), 0.0f))
//						color0 = color1 = color::White;
					//< ʂƓĂOBBƂ̏Փ˂o
//					if(Collision::Detection::OBBPlane(position, Vector3::Half(), axis_x, axis_y, axis_z, vel, Vector3::UnitY(), 0.0f))
//						color0 = color1 = color::White;
					//< ʂƓĂOBBƂ̏Փ˂o(Փ˃p[^߂)
					if(Collision::Detection::OBBPlane(position, Vector3::Half(), axis_x, axis_y, axis_z, vel, Vector3::UnitY(), 0.0f, t0))
						color0 = color1 = color::White;
				}
				break;
			case Segment:
				{
					Vector3 s0(position);
					Vector3 s1(rotation_q.Rotate(Vector3::UnitZ()) + position);
					//< ʂƐƂ̏Փ˂o
					if(Collision::Detection::PlaneSegment(Vector3::UnitY(), 0.0f, s0, s1))
						color0 = color1 = color::White;
				}
				break;
			}
			break;
		}
		//< ܂ŁARW̌o
		//////////////////////////////////////////////////////////////////////////////////////////////

		graphics::System::BeginBackBuffer(); {
			graphics::System::Clear();

			//< VF[_ǂݍ݊ĂȂƂ`͂łȂ
			if(vs->IsLoaded() && ps->IsLoaded()){
				graphics::shader::Program::AutoBegin auto_vs(*vs);
				graphics::shader::Program::AutoBegin auto_ps(*ps);

				graphics::shader::Parameter wvp(vs->GetParameter("WorldViewProjection"));
				graphics::shader::Parameter color(ps->GetParameter("Color"));

#if 1
				//< _XYZ̕`
				wvp = Matrix44::ScaleX(10.0f) * camera.ViewProjection();
				color = color::Red;
				axis_x.Begin(); axis_x.Draw(); axis_x.End();
				wvp = Matrix44::ScaleY(10.0f) * camera.ViewProjection();
				color = color::Green;
				axis_y.Begin(); axis_y.Draw(); axis_y.End();
				wvp = Matrix44::ScaleZ(10.0f) * camera.ViewProjection();
				color = color::Blue;
				axis_z.Begin(); axis_z.Draw(); axis_z.End();
#endif

				//< ^[QbgO(_֔zu)̕`
				wvp = Matrix44::Identity() * camera.ViewProjection();
				color = color0;
				switch(target0){
                case Point:
					wvp = Matrix44::Scale(0.05f) * camera.ViewProjection();
					box.Begin(); box.Draw(); box.End();
					color = color::Black;
					w_box.Begin(); w_box.Draw(); w_box.End();
					break;
					break;
                case AABB:
                case OBB:
					box.Begin(); box.Draw(); box.End();
					color = color::Black;
					w_box.Begin(); w_box.Draw(); w_box.End();
					break;
                case Sphere:
					sphere.Begin(); sphere.Draw(); sphere.End();
					color = color::Black;
					w_sphere.Begin(); w_sphere.Draw(); w_sphere.End();
					break;
                case Cylinder:
					cylinder.Begin(); cylinder.Draw(); cylinder.End();
					color = color::Black;
					w_cylinder.Begin(); w_cylinder.Draw(); w_cylinder.End();
					break;
                case Capsule:
					capsule.Begin(); capsule.Draw(); capsule.End();
					color = color::Black;
					w_capsule.Begin(); w_capsule.Draw(); w_capsule.End();
					break;
                case Segment:
					segment.Begin(); segment.Draw(); segment.End();
					color = color::Black;
					wvp = Matrix44::Scale(0.05f) * camera.ViewProjection();
					box.Begin(); box.Draw(); box.End();
					wvp = Matrix44::Scale(0.05f) * Matrix44::Translation(Vector3::UnitZ()) * camera.ViewProjection();
					box.Begin(); box.Draw(); box.End();
					break;
				case Plane:
					plane.Begin(); plane.Draw(); plane.End();
					color = color::Black;
					w_plane.Begin(); w_plane.Draw(); w_plane.End();
					break;
				}

				//< ^[Qbg1i_ʒu֔zuj̕`
				color = color1;
				switch(target1){
                case Point:
					wvp = Matrix44::Scale(0.05f) * translation * camera.ViewProjection(); //< _͉]łȂiĂӖȂj
					box.Begin(); box.Draw(); box.End();
					color = color::Black;
					w_box.Begin(); w_box.Draw(); w_box.End();
					break;
					break;
                case AABB:
					wvp = translation * camera.ViewProjection(); //< AABB͉]łȂ
					box.Begin(); box.Draw(); box.End();
					color = color::Black;
					w_box.Begin(); w_box.Draw(); w_box.End();
					break;
                case Sphere:
					wvp = rotation * translation * camera.ViewProjection(); //< ͉]ĂӖȂǂ܂
					sphere.Begin(); sphere.Draw(); sphere.End();
					color = color::Black;
					w_sphere.Begin(); w_sphere.Draw(); w_sphere.End();
					break;
                case Cylinder:
					wvp = rotation * translation * camera.ViewProjection();
					cylinder.Begin(); cylinder.Draw(); cylinder.End();
					color = color::Black;
					w_cylinder.Begin(); w_cylinder.Draw(); w_cylinder.End();
					break;
                case Capsule:
					wvp = rotation * translation * camera.ViewProjection();
					capsule.Begin(); capsule.Draw(); capsule.End();
					color = color::Black;
					w_capsule.Begin(); w_capsule.Draw(); w_capsule.End();
					break;
                case Segment:
					wvp = rotation * translation * camera.ViewProjection();
					segment.Begin(); segment.Draw(); segment.End();
					color = color::Black;
					wvp = Matrix44::Scale(0.05f) * translation * camera.ViewProjection();
					box.Begin(); box.Draw(); box.End();
					wvp = Matrix44::Scale(0.05f) * Matrix44::Translation(Vector3::UnitZ()) * rotation * translation * camera.ViewProjection();
					box.Begin(); box.Draw(); box.End();
					break;
                case OBB:
					wvp = rotation * translation * camera.ViewProjection();
					box.Begin(); box.Draw(); box.End();
					color = color::Black;
					w_box.Begin(); w_box.Draw(); w_box.End();
					break;
				case Plane:
					wvp = rotation * translation * camera.ViewProjection();
					plane.Begin(); plane.Draw(); plane.End();
					color = color::Black;
					w_plane.Begin(); w_plane.Draw(); w_plane.End();
					break;
				}

				//< \
				if(dline != Vector3::Zero()){
					color = color::Red;
					Matrix44 tr(Matrix44::Translation(sline));
					Matrix44 rot(Matrix44(Quaternion(Vector3::UnitZ(), dline.Normalize())));
					wvp = rot * tr * camera.ViewProjection();
					segment.Begin(); segment.Draw(); segment.End();
					color = color::Black;
					wvp = Matrix44::Scale(0.05f) * tr * camera.ViewProjection();
					box.Begin(); box.Draw(); box.End();
					wvp = Matrix44::Scale(0.05f) * Matrix44::Translation(Vector3::UnitZ()) * rot * tr * camera.ViewProjection();
					box.Begin(); box.Draw(); box.End();
				}

				//< ŋߐړ_\
				if(close_len > LIM_EPSILON(f32)) {
					color = color::Red;
					Matrix44 tr(Matrix44::Translation(close0));
					Matrix44 rot(Matrix44(Quaternion(Vector3::UnitZ(), (close1 - close0) / close_len)));
					wvp = rot * Matrix44::Scale(close_len) * tr * camera.ViewProjection();
					segment.Begin(); segment.Draw(); segment.End();
					wvp = Matrix44::Scale(0.05f) * tr * camera.ViewProjection();
					box.Begin(); box.Draw(); box.End();
					wvp = Matrix44::Scale(0.05f) * Matrix44::Translation(Vector3::UnitZ() * close_len) * rot * tr * camera.ViewProjection();
					box.Begin(); box.Draw(); box.End();
				}
			}

			pre_pos = position;
		} graphics::System::EndBackBuffer();
		graphics::System::SwapBuffers();
	}

	delete ps;
	delete vs;

	System::Finalize();
}
