uniform float4x4 WorldViewProjection;
uniform float4x4 World;
uniform float3 LightPosition;
uniform float3 CameraPosition;

struct VS_IN
{
	float4 Position : POSITION;
	float4 Normal : NORMAL;
	float2 Texcoord : TEXCOORD0;
};

struct VS_OUT
{
	float4 Position : POSITION;
	float3 Normal : TEXCOORD0;
	float2 Texcoord : TEXCOORD1;

	float3 LightDirection : TEXCOORD2;
	float3 CameraDirection : TEXCOORD3;
};

VS_OUT main(VS_IN In)
{
	VS_OUT Out;

	Out.Position = mul(WorldViewProjection, In.Position);
	Out.Normal = mul((float3x3)World, In.Normal).xyz;
	Out.Texcoord = float2(In.Texcoord.x, 1.0f - In.Texcoord.y);

	float3 pos = mul(World, In.Position).xyz;
	Out.LightDirection = -(pos - LightPosition);
	Out.CameraDirection = pos - CameraPosition;

	return Out;
}
