uniform float4x4 InverseViewProjection;
uniform float4x4 LightViewProjection;
uniform sampler2D NormalSampler;
uniform sampler2D DepthSampler;
uniform sampler2D PreviousSampler;
uniform sampler2D ShadowMapSampler;
uniform float3 LightPosition;
uniform float3 LightDirection;

struct PS_IN
{
	float4 Position : POSITION;
	float2 Texcoord : TEXCOORD0;
};

struct PS_OUT
{
	float4 Color : COLOR;
};

PS_OUT main (PS_IN In)
{       
	PS_OUT Out = (PS_OUT)0;

	float3 normal = normalize(tex2D(NormalSampler, In.Texcoord).rgb * 2.0f - 1.0f);
	float4 screen = float4(In.Texcoord.x * 2.0f - 1.0f, -(In.Texcoord.y * 2.0f - 1.0f), tex2D(DepthSampler, In.Texcoord).r, 1.0f);
	float4 world = mul(InverseViewProjection, screen);
	world /= world.w;
	float4 light_screen = mul(LightViewProjection, world);
	light_screen /= light_screen.w;
	float2 light_sample = float2(light_screen.x * 0.5f + 0.5f, -light_screen.y * 0.5f + 0.5f);
	float3 light_dir = normalize(world.xyz - LightPosition);
	float dot_val = max(dot(light_dir, LightDirection), 0.0f);
	float shade = 0.0f;
	if(dot_val >= cos(3.14f * 0.125f) && tex2D(ShadowMapSampler, light_sample).r > screen.z - 0.01f)
//	if(tex2D(ShadowMapSampler, light_sample).r > screen.z - 0.01f)
//	if(dot_val >= cos(3.14f * 0.125f))
		shade = dot(normal, -light_dir) * pow(dot_val, 2.0f); 

	Out.Color.rgb = tex2D(PreviousSampler, In.Texcoord).rgb + shade;

	return Out;
}


