//< L[t[ԂƂȂ߂炩A͏dBȂƃL[t[ȂꍇɃKNKNA͌yB
#define USE_SLERP

uniform float4x4 World;
uniform float4x4 ViewProjection;
uniform sampler2D AnimationSampler;
uniform float2 AnimationSize;
uniform float AnimationIndex;
#ifdef USE_SLERP
uniform float AnimationIndexMax;
uniform float AnimationIndexRatio;
#endif

#define PI 3.1415f
#define EPSILON 0.00001f

struct VS_IN
{
	float4 Position : POSITION;
	float4 Normal : NORMAL;
	float2 Texcoord : TEXCOORD0;
	float4 Indices : BLENDINDICES;
	float4 Weights : BLENDWEIGHT;
};

struct VS_OUT
{
	float4 Position : POSITION;
	float2 Texcoord : TEXCOORD0;
//	float3 Normal : TEXCOORD1;
};

float4x4 animation_map_transform(float4 q0, float3 t0,
				 float4 q1, float3 t1,
				 float4 q2, float3 t2,
				 float4 q3, float3 t3,
				 float4 weights)
{
	float ww = q0.w * q0.w - 0.5f;
	float xy = q0.x * q0.y;
	float xz = q0.x * q0.z;
	float yz = q0.y * q0.z;
	float xw = q0.w * q0.x;
	float yw = q0.w * q0.y;
	float zw = q0.w * q0.z;
	float3 r10 = float3(ww, xy, xz) + float3(q0.x * q0.x, zw, -yw);
	float3 r11 = float3(xy, ww, yz) + float3(-zw, q0.y * q0.y, xw);
	float3 r12 = float3(xz, yz, ww) + float3(yw, -xw, q0.z * q0.z );
	
	ww = q1.w * q1.w - 0.5f;
	xy = q1.x * q1.y;
	xz = q1.x * q1.z;
	yz = q1.y * q1.z;
	xw = q1.w * q1.x;
	yw = q1.w * q1.y;
	zw = q1.w * q1.z;
	float3 r20 = float3(ww, xy, xz) + float3(q1.x * q1.x, zw, -yw);
	float3 r21 = float3(xy, ww, yz) + float3(-zw, q1.y * q1.y, xw);
	float3 r22 = float3(xz, yz, ww) + float3(yw, -xw, q1.z * q1.z);
	
	ww = q2.w * q2.w - 0.5f;
	xy = q2.x * q2.y;
	xz = q2.x * q2.z;
	yz = q2.y * q2.z;
	xw = q2.w * q2.x;
	yw = q2.w * q2.y;
	zw = q2.w * q2.z;
	float3 r30 = float3(ww, xy, xz) + float3(q2.x * q2.x, zw, -yw);
	float3 r31 = float3(xy, ww, yz) + float3(-zw, q2.y * q2.y, xw);
	float3 r32 = float3(xz, yz, ww) + float3(yw, -xw, q2.z * q2.z);
	
	ww = q3.w * q3.w - 0.5f;
	xy = q3.x * q3.y;
	xz = q3.x * q3.z;
	yz = q3.y * q3.z;
	xw = q3.w * q3.x;
	yw = q3.w * q3.y;
	zw = q3.w * q3.z;
	float3 r40 = float3(ww, xy, xz) + float3(q3.x * q3.x, zw, -yw);
	float3 r41 = float3(xy, ww, yz) + float3(-zw, q3.y * q3.y, xw);
	float3 r42 = float3(xz, yz, ww) + float3(yw, -xw, q3.z * q3.z);
	
	float4 w2 = 2.0f * weights;	
	return float4x4(r10 * w2.x + r20 * w2.y + r30 * w2.z + r40 * w2.w, 0.0f,
			r11 * w2.x + r21 * w2.y + r31 * w2.z + r41 * w2.w, 0.0f,
			r12 * w2.x + r22 * w2.y + r32 * w2.z + r42 * w2.w, 0.0f, 
			t0 * weights.x + t1 * weights.y + t2 * weights.z + t3 * weights.w, 1.0f);	
}

#ifdef USE_SLERP
float4 slerp(float4 q0, float4 q1, float t)
{
	float q01 = dot(q0, q1);
	if(abs(1.0f - q01 * q01) < EPSILON)
		return q0;

	float ph = acos(q01);
	float inv_s = 1.0f / sin(ph);
	float t0 = sin(ph * (1.0f - t)) * inv_s;  
	float t1 = sin(ph * t) * inv_s;  

	if (q01 < 0.0f && ph > PI * 0.5) 
		return normalize(q0 * t0 - q1 * t1);
	else 
		return normalize(q0 * t0 + q1 * t1);
}
float4x4 AnimationMapTransform(float4 boneIndices, float4 weights)
{
	float2 uv0 = 1.0f / AnimationSize;
	float2 uv1 = uv0;
	uv0.y *= (0.5f + AnimationIndex);
	uv1.y *= (0.5f + min(AnimationIndex + 1.0f, AnimationIndexMax));

	float4 tmp = 0.5f + boneIndices * 2.0f;
	float4 coord00 = float4(tmp.x * uv0.x, uv0.y, 0.0f, 1.0f);
	float4 coord01 = float4(tmp.y * uv0.x, uv0.y, 0.0f, 1.0f);
	float4 coord02 = float4(tmp.z * uv0.x, uv0.y, 0.0f, 1.0f);
	float4 coord03 = float4(tmp.w * uv0.x, uv0.y, 0.0f, 1.0f);

	float4 coord10 = float4(tmp.x * uv1.x, uv1.y, 0.0f, 1.0f);
	float4 coord11 = float4(tmp.y * uv1.x, uv1.y, 0.0f, 1.0f);
	float4 coord12 = float4(tmp.z * uv1.x, uv1.y, 0.0f, 1.0f);
	float4 coord13 = float4(tmp.w * uv1.x, uv1.y, 0.0f, 1.0f);

	float4 q0 = slerp(tex2Dlod(AnimationSampler, coord00), tex2Dlod(AnimationSampler, coord10), AnimationIndexRatio);
	float4 q1 = slerp(tex2Dlod(AnimationSampler, coord01), tex2Dlod(AnimationSampler, coord11), AnimationIndexRatio);
	float4 q2 = slerp(tex2Dlod(AnimationSampler, coord02), tex2Dlod(AnimationSampler, coord12), AnimationIndexRatio);
	float4 q3 = slerp(tex2Dlod(AnimationSampler, coord03), tex2Dlod(AnimationSampler, coord13), AnimationIndexRatio);

	tmp += 1.0f;
	coord00 = float4(tmp.x * uv0.x, uv0.y, 0.0f, 1.0f);
	coord01 = float4(tmp.y * uv0.x, uv0.y, 0.0f, 1.0f);
	coord02 = float4(tmp.z * uv0.x, uv0.y, 0.0f, 1.0f);
	coord03 = float4(tmp.w * uv0.x, uv0.y, 0.0f, 1.0f);

	coord10 = float4(tmp.x * uv1.x, uv1.y, 0.0f, 1.0f);
	coord11 = float4(tmp.y * uv1.x, uv1.y, 0.0f, 1.0f);
	coord12 = float4(tmp.z * uv1.x, uv1.y, 0.0f, 1.0f);
	coord13 = float4(tmp.w * uv1.x, uv1.y, 0.0f, 1.0f);

	float4 t0 = lerp(tex2Dlod(AnimationSampler, coord00), tex2Dlod(AnimationSampler, coord10), AnimationIndexRatio);
	float4 t1 = lerp(tex2Dlod(AnimationSampler, coord01), tex2Dlod(AnimationSampler, coord11), AnimationIndexRatio);
	float4 t2 = lerp(tex2Dlod(AnimationSampler, coord02), tex2Dlod(AnimationSampler, coord12), AnimationIndexRatio);
	float4 t3 = lerp(tex2Dlod(AnimationSampler, coord03), tex2Dlod(AnimationSampler, coord13), AnimationIndexRatio);

	return animation_map_transform(q0, t0.xyz,
				       q1, t1.xyz,
				       q2, t2.xyz,
				       q3, t3.xyz,
				       weights);
}
#else
float4x4 AnimationMapTransform(float4 boneIndices, float4 weights)
{
	float2 uv = 1.0f / AnimationSize;
	uv.y *= (0.5f + AnimationIndex);

	float4 tmp = 0.5f + boneIndices * 2.0f;
	float4 coord0 = float4(tmp.x * uv.x, uv.y, 0.0f, 1.0f);
	float4 coord1 = float4(tmp.y * uv.x, uv.y, 0.0f, 1.0f);
	float4 coord2 = float4(tmp.z * uv.x, uv.y, 0.0f, 1.0f);
	float4 coord3 = float4(tmp.w * uv.x, uv.y, 0.0f, 1.0f);

	float4 q0 = tex2Dlod(AnimationSampler, coord0);
	float4 q1 = tex2Dlod(AnimationSampler, coord1);
	float4 q2 = tex2Dlod(AnimationSampler, coord2);
	float4 q3 = tex2Dlod(AnimationSampler, coord3);

	tmp += 1.0f;
	coord0 = float4(tmp.x * uv.x, uv.y, 0.0f, 1.0f);
	coord1 = float4(tmp.y * uv.x, uv.y, 0.0f, 1.0f);
	coord2 = float4(tmp.z * uv.x, uv.y, 0.0f, 1.0f);
	coord3 = float4(tmp.w * uv.x, uv.y, 0.0f, 1.0f);

	float4 t0 = tex2Dlod(AnimationSampler, coord0);
	float4 t1 = tex2Dlod(AnimationSampler, coord1);
	float4 t2 = tex2Dlod(AnimationSampler, coord2);
	float4 t3 = tex2Dlod(AnimationSampler, coord3);

	return animation_map_transform(q0, t0.xyz,
				       q1, t1.xyz,
				       q2, t2.xyz,
				       q3, t3.xyz,
				       weights);
}
#endif

VS_OUT main(VS_IN In)
{
	VS_OUT Out;

	float4x4 skin = mul(World, transpose(AnimationMapTransform(In.Indices, In.Weights)));
	Out.Position = mul(ViewProjection, mul(skin, In.Position));
	//Out.Normal = mul((float3x3)skin, In.Normal).xyz;

	Out.Texcoord = float2(In.Texcoord.x, 1.0f - In.Texcoord.y);

	return Out;
}
