/*!
$Id: framebuffer.h 541 2009-11-20 10:40:29Z horino $
*/
#ifndef FRAMEBUFFER_H
#define FRAMEBUFFER_H

#include <vector>
#include <iostream>
using namespace std;

#include "itl/vector.h"
#include "itl/algorithm.h"
#include "indi.h"
#include "graphics/graphics.h"
#include "graphics/rendertarget.h"
#include "graphics/texture.h"

namespace indi
{
	namespace graphics
	{
		class RenderTargetBase;

		/*!
		\brief t[obt@NX
		*/
		class FrameBuffer
		{
		public:
			FrameBuffer(RenderTargetBase *rt0 = 0, RenderTargetBase *rt1 = 0, RenderTargetBase *rt2 = 0, RenderTargetBase *rt3 = 0, RenderTargetBase *rt4 = 0);
			~FrameBuffer();

			void Add(u32 i, RenderTargetBase *rt);
			void Clear(u8 r = 0, u8 g = 0, u8 b = 0, u8 a = 255);

		private:
			struct RenderTargetSemantics
			{
				RenderTargetSemantics() { semantics = 0; target = 0; }
				RenderTargetSemantics(u32 s, RenderTargetBase *t) { semantics = s; target = t; }

				u32 semantics;
				RenderTargetBase *target;
			};
			struct SurfaceSemantics
			{
				SurfaceSemantics() { semantics = 0; surface = 0; }
				SurfaceSemantics(u32 s, D3DSurface *sur) { semantics = s; surface = sur; }

				u32 semantics;
				D3DSurface *surface;
			};

		public:
			void Begin();
			void End();

			void FullScreenRender(const RenderTexture *t);

			/*!
			\brief XR[v𔲂ƂɎIEnd()R[֗NX
			*/
			class AutoBegin
			{
			public:
				AutoBegin(FrameBuffer &fb) : _fb(fb) { _fb.Begin(); }
				AutoBegin(FrameBuffer *fb) : _fb(*fb) { _fb.Begin(); }
				~AutoBegin(){ _fb.End(); }
			private:
				FrameBuffer &_fb;
			};

#ifdef PREDICATED_TILING
#ifdef _DEBUG
			static void CheckCommandBufferSpace();
#endif
			static const s32 _nrects = 15;
			itl::Vector<D3DRECT, _nrects> _rects;
#endif
			static const s32 _ntarget = 5;
			itl::Vector<SurfaceSemantics, _ntarget> _backups;
			itl::Vector<RenderTargetSemantics, _ntarget> _rendertargets;
		};
	};
};

#endif // FRAMEBUFFER_H