/*!
$Id: graphics.h 640 2009-12-24 07:03:53Z horino $
*/
#ifndef GRAPHICS_H
#define GRAPHICS_H

#define PREDICATED_TILING //< vfBP[g^COgꍇ

#include "indi.h"
#include "graphics/shader/shader.h"
#include "graphics/vertexbuffer.h"
#include "collision/collision.h"

namespace indi
{
	namespace graphics
	{
		class TextureBase;

#ifdef PREDICATED_TILING
		class RenderTexture;
		class RenderTarget;
		class DepthTarget;
		class FrameBuffer;
#endif

		class System		
		{
			//< GetDevice()ɃANZX𓾂Ȃl
			friend class indi::graphics::shader::Program;
			friend class indi::graphics::shader::System;
			friend class indi::graphics::shader::Parameter;

		private:
			System(){}

		public:
			static void Initialize();
			static void Finalize();

			static u32 GetScreenWidth() { return viewport.Width; }
			static u32 GetScreenHeight() { return viewport.Height; }
			static Collision::Frustum &GetFrustum() { return frustum; }
			static const Matrix44 &GetViewportMatrix() { return viewportMatrix; }
			static const Matrix44 &GetInverseViewportMatrix() { return inverseViewportMatrix; }
			static D3DVIEWPORT9 GetViewport() { return viewport; }
			static f32 GetAspectRatio() { return aspectRatio; }
			static Vector3 UnProject(const Vector3 &spos, const Matrix44 &world, const Matrix44 &view, const Matrix44 &projection);
			static Vector3 *UnProject(const Vector3 *spos, u32 n, Vector3 *dpos, const Matrix44 &world, const Matrix44 &view, const Matrix44 &projection);
			static Vector3 Project(const Vector3 &wpos, const Matrix44 &wvp_vp);
			static Vector2 ScreenRatio(const Vector3 &center, const Vector3 &radius, const Matrix44 &wvp_vp);

		private:
			static u32 _clearColor;
		public:
			enum ClearTarget {
#define CLEAR_TARGET(id, d3d) id = d3d,
#include "graphics.h"
#undef CLEAR_TARGET
			};

			static D3DVECTOR4 GetClearColor() { 
				f32 inv(1.0f / 255.0f);
				D3DVECTOR4 v = { D3DCOLOR_GETRED(_clearColor) *inv, D3DCOLOR_GETGREEN(_clearColor) *inv, D3DCOLOR_GETBLUE(_clearColor) *inv, D3DCOLOR_GETALPHA(_clearColor) *inv};
				return v; 
			}

			static void ClearColor(u8 r, u8 g, u8 b, u8 a = 255) { _clearColor = D3DCOLOR_RGBA(r, g, b, a); }
			static void Clear(u32 m = COLOR | DEPTH | STENCIL) { GetDevice()->Clear(0, NULL, m, _clearColor, 1.0f, 0); }

			static void CreateFullScreenShader();
			static void DrawFullScreen(const TextureBase *t);

			static void BeginBackBuffer();
			static TextureBase *EndBackBuffer();
			static void SwapBuffers(TextureBase *t = 0);
			static void DepthWrite(BOOL b) { GetDevice()->SetRenderState(D3DRS_ZWRITEENABLE, b); }
			class AutoBeginBackBuffer
			{
			public:
				AutoBeginBackBuffer() { BeginBackBuffer(); }
				~AutoBeginBackBuffer(){ EndBackBuffer(); }
			};

			enum SampleType {
#define SAMPLE_TYPE(id, val) id = val,
#include "graphics.h"
#undef SAMPLE_TYPE
			};
			static SampleType GetSampleType() { return sampleType; }

			enum FrameRate {
#define FRAME_RATE(id, val) id = val,
#include "graphics.h"
#undef FRAME_RATE
			};
			static FrameRate GetFrameRate() { return frameRate; }
		public: 
			static D3DDevice *GetDevice() { return d3dDevice; }

		private:
			static D3DDevice *d3dDevice;
			static D3DVIEWPORT9 viewport;
			static Collision::Frustum frustum;
			static Matrix44 viewportMatrix;
			static Matrix44 inverseViewportMatrix;
			static f32 aspectRatio;
			static SampleType sampleType;
			static FrameRate frameRate;
			static shader::VertexProgram *fullscreen_vs;
			class FullScreenPS : public shader::PixelProgram
			{
			public:
				FullScreenPS(const char *path) : shader::PixelProgram(path) {}
				virtual ~FullScreenPS() {}
#ifdef USE_REGISTER
				void OnLoaded() { shader::PixelProgram::OnLoaded(); sampler = GetParameter(0); }
#else
				void OnLoaded() { shader::PixelProgram::OnLoaded(); sampler = GetParameter("Sampler"); }
#endif
				shader::Parameter sampler;
			};
			static FullScreenPS *fullscreen_ps;
			static shader::Parameter fullscreen_sampler;

#ifdef PREDICATED_TILING
			static D3DSurface *renderTarget;
			static D3DTexture *frontBuffer;
#endif

#ifdef PREDICATED_TILING
		private:
			static graphics::RenderTexture *bf_color_map;
			static graphics::RenderTarget *bf_color_target;
			static graphics::DepthTarget *bf_depth_target;
			static graphics::FrameBuffer *back_buffer;
#endif

		public:
			class Blend
			{
			public:
				enum BlendTarget {
#define BLEND_TARGET(id, d3d) id = d3d,
#include "graphics.h"
#undef BLEND_TARGET
				};

				static void Begin() { graphics::System::GetDevice()->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE); } 
				static void End() { graphics::System::GetDevice()->SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE); } 

				static void Set(BlendTarget src, BlendTarget dst) {
					graphics::System::GetDevice()->SetRenderState(D3DRS_SRCBLEND, src);
					graphics::System::GetDevice()->SetRenderState(D3DRS_DESTBLEND, dst);
				}

				/*!
				\brief XR[v𔲂ƂɎIEnd()R[֗NX
				*/
				class AutoBegin
				{
				public:
					AutoBegin() { Begin(); }
					~AutoBegin(){ End(); }
				};
			};
		};
	};
};

#endif // GRAPHICS_H

#ifdef CLEAR_TARGET
CLEAR_TARGET(COLOR, D3DCLEAR_TARGET)
CLEAR_TARGET(DEPTH, D3DCLEAR_ZBUFFER)
CLEAR_TARGET(STENCIL, D3DCLEAR_STENCIL)
#endif

#ifdef BLEND_TARGET
BLEND_TARGET(ZERO, D3DBLEND_ZERO)
BLEND_TARGET(ONE, D3DBLEND_ONE)
BLEND_TARGET(SRC_COLOR, D3DBLEND_SRCCOLOR)
BLEND_TARGET(INV_SRC_COLOR, D3DBLEND_INVSRCCOLOR)
BLEND_TARGET(SRC_ALPHA, D3DBLEND_SRCALPHA)
BLEND_TARGET(INV_SRC_ALPHA, D3DBLEND_INVSRCALPHA)
BLEND_TARGET(DST_ALPHA, D3DBLEND_DESTALPHA)
BLEND_TARGET(INV_DST_ALPHA, D3DBLEND_INVDESTALPHA)
#endif

#ifdef FRAME_RATE
FRAME_RATE(FRAME_RATE_60, D3DPRESENT_INTERVAL_ONE)
FRAME_RATE(FRAME_RATE_30, D3DPRESENT_INTERVAL_TWO)
FRAME_RATE(FRAME_RATE_20, D3DPRESENT_INTERVAL_THREE)
FRAME_RATE(FRAME_RATE_15, D3DPRESENT_INTERVAL_FOUR)
FRAME_RATE(FRAME_RATE_IMMEDIATE, D3DPRESENT_INTERVAL_IMMEDIATE)
#endif

#ifdef SAMPLE_TYPE
SAMPLE_TYPE(SAMPLE_NONE, D3DMULTISAMPLE_NONE)
SAMPLE_TYPE(SAMPLE_2, D3DMULTISAMPLE_2_SAMPLES)
SAMPLE_TYPE(SAMPLE_4, D3DMULTISAMPLE_4_SAMPLES)
#endif