/*!
$Id: rendertarget.h 536 2009-11-20 02:57:07Z horino $
*/
#ifndef RENDERTARGET_H
#define RENDERTARGET_H

#include <iostream>
using namespace std;

#include "graphics/graphics.h"
#include "graphics/texture.h"
#include "indi.h"

namespace indi
{
	namespace graphics
	{
		/*!
		\brief _^[Qbgx[XNX
		*/
		class RenderTargetBase 
		{
		public:
			RenderTargetBase() : _rendertarget(0) {}
			virtual ~RenderTargetBase(){ if(_rendertarget) { _rendertarget->Release(); _rendertarget = 0; } }

			D3DSurface *GetRenderTarget() { return _rendertarget; }

			virtual void Bind(RenderTexture *t) = 0;
			virtual void Create(u32 w, u32 h) = 0;
			virtual BOOL IsRenderTarget() const = 0;
			virtual BOOL IsDepthTarget() const = 0;
			virtual u32 GetWidth() const = 0;
			virtual u32 GetHeight() const = 0;

		protected:
			D3DSurface *_rendertarget;
		};

		/*!
		\brief _[^[QbgNX
		*/
		class RenderTarget : public RenderTargetBase
		{
		public:
			RenderTarget(RenderTexture *r) : RenderTargetBase(), _resolve(r) { CreateRenderTarget(r->GetWidth(), r->GetHeight()); }
			virtual ~RenderTarget() {}

			RenderTexture *GetResolveTexture() { return _resolve; }

			//< RXgN^ł͎g܂
			virtual void Bind(RenderTexture *r) { CreateRenderTarget(r->GetWidth(), r->GetHeight()); }
			virtual void Create(u32 w, u32 h) { CreateRenderTarget(w, h); }
			virtual BOOL IsRenderTarget() const { return TRUE; }
			virtual BOOL IsDepthTarget() const { return FALSE; }
			virtual u32 GetWidth() const { 
#ifdef _DEBUG
				ASSERT(_resolve);
#endif
				return _resolve->GetWidth(); 
			}
			virtual u32 GetHeight() const { 
#ifdef _DEBUG
				ASSERT(_resolve);
#endif
				return _resolve->GetHeight(); 
			}

		private:
			void CreateRenderTarget(u32 w, u32 h);
			RenderTexture *_resolve;
		};

		/*!
		\brief fvX_[^[QbgNX
		*/
		class DepthTarget : public RenderTargetBase
		{
		public:
			DepthTarget(u32 w, u32 h) : RenderTargetBase(), _width(w), _height(h) { CreateDepthTarget(w, h); }
			virtual ~DepthTarget() {}

			//< RXgN^ł͎g܂
			virtual void Bind(RenderTexture *r) { CreateDepthTarget(r->GetWidth(), r->GetHeight()); }
			virtual void Create(u32 w, u32 h) { CreateDepthTarget(w, h); }
			virtual BOOL IsRenderTarget() const { return FALSE; }
			virtual BOOL IsDepthTarget() const { return TRUE; }
			virtual u32 GetWidth() const { return _width; }
			virtual u32 GetHeight() const { return _height; }

		private:
			void CreateDepthTarget(u32 w, u32 h);
			u32 _width, _height;
		};
	};
};

#endif // RENDERTARGET_H