/*!
$Id: shader.h 624 2009-12-17 07:37:52Z horino $
*/
#ifndef SHADER_H
#define SHADER_H

#include "indi.h"
#include "itl/bitset.h"

#include "file/file.h"

//#define USE_REGISTER // PS3łƂ̌݊lāAgp֎~

namespace indi
{
	class Vector2;
	class Vector3;
	class Vector4;
	class Matrix44;

	namespace graphics
	{
		class TextureBase;

		namespace shader
		{
			/*!
			\brief VF[_VXeǗNX
			*/
			class System
			{
			private:
				System(){}

			public:
				static void Initialize(){}
				static void Finalize(){}

				/*!
				\brief XR[v𔲂ƂɎIɃt@CiCY֗NX
				*/
				class AutoInit 
				{
				public:
					AutoInit(){ Initialize(); }
					~AutoInit(){ Finalize(); }
				};
			};

			/*!
			\brief VF[_ɓnp[^ǗNX
			*/
			class Parameter
			{
			public:
#ifdef USE_REGISTER
				enum {
					VertexShader,
				};
				Parameter() : _parameter(0xffffffff), _is_pixel(FALSE) {}
				Parameter(u32 i, BOOL is_p) : _parameter(i), _is_pixel(is_p){}
#else
				Parameter() : _handle(0), _table(0) {}
				Parameter(D3DXHANDLE h, ID3DXConstantTable *t) : _handle(h), _table(t) {}
#endif
				~Parameter() {
#ifndef USE_REGISTER
					_handle = 0; /* ReleaseIȂ̂͂H */
#endif
				}

				void operator=(f32 lhs);
				void operator=(const Vector2 &lhs);
				void operator=(const Vector3 &lhs);
				void operator=(const Vector4 &lhs);
				void operator=(const Matrix44 &lhs);
				void operator=(const graphics::TextureBase &lhs) { (*this) = &lhs; }
				void operator=(const graphics::TextureBase *lhs);
 
				void Disable();

				/*!
				\brief XR[v𔲂ƂɎIDisable()R[֗NX
				*/
				class AutoBeginTexture
				{
				public:
					AutoBeginTexture(Parameter &p, const TextureBase &t) : _parameter(p) { _parameter = t; }
					AutoBeginTexture(Parameter &p, const TextureBase *t) : _parameter(p) { _parameter = t; }
#if 0
					AutoBeginTexture(Parameter &p, D3DTexture &t) : _parameter(p) { _parameter = t; }
					AutoBeginTexture(Parameter &p, D3DTexture *t) : _parameter(p) { _parameter = t; }
#endif
					~AutoBeginTexture(){ _parameter.Disable(); }
				private:
					Parameter &_parameter;
				};

#ifdef USE_REGISTER
				BOOL IsPixel() { return _is_pixel; }
				BOOL IsVertex() { return !_is_pixel; }
#endif

			private:
#ifdef USE_REGISTER
				u32 _parameter;
				BOOL _is_pixel;
#else
				D3DXHANDLE _handle;
				ID3DXConstantTable *_table;
#endif
			};

			/*!
			\brief VF[_vOǗNX
			*/
			class Program 
			{
			public:
#ifdef USE_REGISTER
				Program() {}
#else
				Program() : _table(0) {}
#endif
				virtual ~Program() { 
#ifndef USE_REGISTER
					if(_table) { _table->Release(); _table = 0; } 
#endif
				}

				virtual void Begin() = 0;
				void End(){}

#ifdef USE_REGISTER
				virtual Parameter GetParameter(u32 i) = 0;
#else
				virtual Parameter GetParameter(const char *name) = 0;
#endif
				virtual BOOL IsVertexProgram() const = 0;
				virtual BOOL IsPixelProgram() const = 0;

				/*!
				\brief XR[v𔲂ƂɎIEnd()R[֗NX
				*/
				class AutoBegin
				{
				public:
					AutoBegin(Program &p) : _program(p){ _program.Begin(); }
					~AutoBegin(){ _program.End(); }
				private:
					Program &_program;
				};

			protected:
#ifndef USE_REGISTER
				ID3DXConstantTable *_table;
#endif
			};

			/*!
			\brief VF[_vO(t@C񓯊ǂݍ)
			*/
			class ProgramFile : public Program, public file::File
			{
			public:
				ProgramFile() : Program(), file::File() {}
				ProgramFile(const char *path) : Program(), file::File(path) {}
				virtual ~ProgramFile() {}

#ifdef USE_REGISTER
				Parameter GetParameter(u32 i) { return Parameter(i, IsPixelProgram()); }
#else
				Parameter GetParameter(const char *name);
#endif
			};	

			/*!
			\brief o[ebNXVF[_
			*/
			class VertexProgram : public ProgramFile
			{
			public:
				VertexProgram() : ProgramFile(), _program(0) {}
				VertexProgram(const char *path) : ProgramFile(path), _program(0) {}
				VertexProgram(IDirect3DVertexShader9 *v) : ProgramFile(), _program(v) {}
				virtual ~VertexProgram() {}

				operator IDirect3DVertexShader9*() { return _program; }
				void operator=(IDirect3DVertexShader9* lhs) { _program = lhs; }

				void Begin();

				virtual void OnLoaded();

				BOOL IsVertexProgram() const { return TRUE; }
				BOOL IsPixelProgram() const { return  FALSE; }

			private:
				IDirect3DVertexShader9 *_program;
			};

			/*!
			\brief sNZVF[_
			*/
			class PixelProgram : public ProgramFile
			{
			public:
				PixelProgram() : ProgramFile(), _program(0) {}
				PixelProgram(const char *path) : ProgramFile(path), _program(0) {}
				PixelProgram(IDirect3DPixelShader9 *p) : ProgramFile(), _program(p) {}
				virtual ~PixelProgram() {}

				operator IDirect3DPixelShader9*() { return _program; }
				void operator=(IDirect3DPixelShader9* lhs) { _program = lhs; }

				void Begin();

				virtual void OnLoaded();

				BOOL IsVertexProgram() const { return FALSE; }
				BOOL IsPixelProgram() const { return  TRUE; }
			private:
				IDirect3DPixelShader9 *_program;
			};
		}
	}
}

#endif // SHADER_H