/*!
$Id: vertexbuffer.h 534 2009-11-19 11:21:24Z horino $
*/
#ifndef VERTEXBUFFER_H
#define VERTEXBUFFER_H

#include "indi.h"
#include "utilities.h"
#include "graphics/graphics.h"
#include "graphics/vertexbuffer_specialization.h"

namespace indi
{
	namespace graphics
	{
		enum PrimitiveType {
#define PRIMTYPE(id, d3d) id = d3d,
#include "vertexbuffer.h"
#undef PRIMTYPE
		};

		/*!
		\brief o[ebNXobt@x[XNX
		*/
		class VertexBufferBase 
		{
		public:
			VertexBufferBase() {}
			virtual ~VertexBufferBase() {}

			virtual void Begin() = 0;
			virtual void Draw() = 0;
			virtual void End() = 0;

			virtual void SetIndexWeight(shader::Program *prog, const char *index_name, const char *weight_name) = 0;

			/*!
			\brief XR[v𔲂ƂɎIEnd()R[֗NX
			*/
			class AutoBegin
			{
			public:
				AutoBegin(VertexBufferBase &vb) : _vb(vb){ _vb.Begin(); }
				~AutoBegin(){ _vb.End(); }
			private:
				VertexBufferBase &_vb;
			};
		};

		/*!
		\brief o[ebNXobt@NX
		\note ev[gNXA^̒`CreateDeclaration()֐̃ev[g邱Ƃ\łB
		({"graphics/vertexbuffer_specialization.h"̌^g܂j
		*/
		template<typename T>
		class VertexBuffer : public VertexBufferBase
		{
		public:
			VertexBuffer(PrimitiveType p=TRIANGLES) : VertexBufferBase(), _primtype(p)/*, _vertices(0)*/, _nvertices(0) {}
			VertexBuffer(PrimitiveType p, const T *data, u32 n) : VertexBufferBase(), _primtype(p)/*, _vertices(0)*/, _nvertices(n) { 
				CreateDeclaration();
				CreateVertexBuffer(data, _nvertices);
			};
			~VertexBuffer(){}

			void SetPrimitiveType(PrimitiveType p) { _primtype = p; }
			void CreateDeclaration();

			/*!
			\brief o[ebNXobt@쐬܂
			\param f[^ւ̃|C^
			\param f[^̌
			*/
			void CreateVertexBuffer(const T *data, u32 n) {
				if(!FAILED(graphics::System::GetDevice()->CreateVertexBuffer(n * sizeof(T), D3DUSAGE_WRITEONLY, NULL, D3DPOOL_MANAGED, &_vertexbuffer, NULL))) {
					T *v;
					if(!FAILED(_vertexbuffer->Lock(0, 0, (void **)&v, 0))){
						Memcpy(v, data, _nvertices * sizeof(T));
						_vertexbuffer->Unlock();
					}
				}
			}

			/*!
			\brief o[ebNXobt@̃CfbNXAEGCgZ}eBNXVF[_֓n܂
			\param VF[_vO
			\param VF[_̃EGCg
			\param VF[_̃CfbNX
			*/
			void SetIndexWeight(shader::Program *prog, const char *index_name, const char *weight_name){}

			/*!
			\brief `̑O
			*/
			void Begin() {
				graphics::System::GetDevice()->SetVertexDeclaration(_declaration);
				graphics::System::GetDevice()->SetStreamSource(0, _vertexbuffer, 0, sizeof(T));
			}

			/*!
			\brief `
			*/
			void Draw() { 
				u32 nprim(0);
				switch(_primtype){
		        case POINTS:         ASSERT(0); break; //< Ή
				case LINES:          nprim = (_nvertices >> 1); break;
				case LINE_STRIP:     ASSERT(0); break; //< Ή
				case TRIANGLES:      nprim = _nvertices / 3; break;
				case TRIANGLE_STRIP: nprim = _nvertices - 2; break;
				case TRIANGLE_FAN:   ASSERT(0); break; //< Ή
				}
				graphics::System::GetDevice()->DrawPrimitive(static_cast<D3DPRIMITIVETYPE>(_primtype), 0, nprim); 
			}

			/*!
			\brief `̌㏈
			*/
			void End(){}

			/*!
			\brief XR[v𔲂ƂɎIEnd()R[֗NX
			*/
			class AutoBegin
			{
			public:
				AutoBegin(VertexBuffer &vb) : _vb(vb){ _vb.Begin(); }
				~AutoBegin(){ _vb.End(); }
			private:
				VertexBuffer &_vb;
			};

		private:
			PrimitiveType _primtype;
			D3DVertexDeclaration *_declaration;
			D3DVertexBuffer *_vertexbuffer;
			u32 _nvertices;
		};
	};
};

#endif // VERTEXBUFFER_H

#ifdef PRIMTYPE
PRIMTYPE(POINTS, D3DPT_POINTLIST)
PRIMTYPE(LINES, D3DPT_LINELIST)
PRIMTYPE(LINE_STRIP, D3DPT_LINESTRIP)
PRIMTYPE(TRIANGLES, D3DPT_TRIANGLELIST)
PRIMTYPE(TRIANGLE_STRIP, D3DPT_TRIANGLESTRIP)
PRIMTYPE(TRIANGLE_FAN, D3DPT_TRIANGLEFAN)
//PRIMTYPE(RECTANGLES, D3DPT_RECTLIST)
//PRIMTYPE(QUADS, D3DPT_QUADLIST)
#endif