/*!
$Id: input.h 293 2009-08-19 09:41:20Z horino $
*/
#ifndef INPUT_H
#define INPUT_H

#ifdef _DEBUG
#include <iostream>
using namespace std;
#endif
#include <D3dx9math.h>

#include "indi.h"
#include "utilities.h"
#include "itl/bitset.h"
//#include "itl/itl.h"
#include "vector2.h"

namespace indi
{
	/*!
	\brief pbh͂܂AƂ肠ƍƂɂȂȂ̂
	\note ̂ւ͌ŒNɔC
	*/
	class Input
	{
	public:
		class State
		{
		public:
			enum Kind {
#define PADASSIGN(id, xinput) id, 
#include "input.h"
#undef PADASSIGN
				KindNum
			};

			/*!
			\breif AiOXeBbNȂǂ̗Vє͈͓H
			*/
			enum {
				VALID,
				LS_DEADZONE,
				RS_DEADZONE,
				LT_DEADZONE,
				RT_DEADZONE,
			};
			State() : _flag(0) { indi::Memzero(&_state, sizeof(_state)); }

			BOOL Update(u32 i);
			BOOL IsValid() const { return _flag.Test(VALID); }

			u8 LeftTrigger() const { return _flag.Test(LT_DEADZONE) ? 0 : _state.Gamepad.bLeftTrigger; }
			u8 RightTrigger() const { return _flag.Test(RT_DEADZONE) ? 0 : _state.Gamepad.bRightTrigger; }
			f32 LeftTriggerf() const { return _flag.Test(LT_DEADZONE) ? 0.0f : static_cast<f32>(LeftTrigger()) / 0xff; }
			f32 RightTriggerf() const { return _flag.Test(RT_DEADZONE) ? 0.0f : static_cast<f32>(RightTrigger()) / 0xff; }

			s16 LeftStickX() const { return _flag.Test(LS_DEADZONE) ? 0 : _state.Gamepad.sThumbLX; }
			s16 LeftStickY() const { return _flag.Test(LS_DEADZONE) ? 0 : _state.Gamepad.sThumbLY; }
			f32 LeftStickXf() const { return _flag.Test(LS_DEADZONE) ? 0.0f : static_cast<f32>(LeftStickX()) / 32767.0f; }
			f32 LeftStickYf() const { return _flag.Test(LS_DEADZONE) ? 0.0f : static_cast<f32>(LeftStickY()) / 32767.0f; }
			Vector2 LeftStick() const { return _flag.Test(LS_DEADZONE) ? Vector2::Zero() : Vector2(LeftStickXf(), LeftStickYf()); }

			s16 RightStickX() const { return _flag.Test(RS_DEADZONE) ? 0 : _state.Gamepad.sThumbRX; }
			s16 RightStickY() const { return _flag.Test(RS_DEADZONE) ? 0 : _state.Gamepad.sThumbRY; }
			f32 RightStickXf() const { return _flag.Test(RS_DEADZONE) ? 0.0f : static_cast<f32>(RightStickX()) / 32767.0f; }
			f32 RightStickYf() const { return _flag.Test(RS_DEADZONE) ? 0.0f : static_cast<f32>(RightStickY()) / 32767.0f; }
			Vector2 RightStick() const { return _flag.Test(RS_DEADZONE) ? Vector2::Zero() : Vector2(RightStickXf(), RightStickYf()); }

			BOOL Button(u32 k) const { return Button(static_cast<Kind>(k)); }
			BOOL Button(Kind k) const { return Xinput(k) & _state.Gamepad.wButtons; }

			BOOL Up() const { return Button(D_UP); }
			BOOL Down() const { return Button(D_DOWN); }
			BOOL Left() const { return Button(D_LEFT); }
			BOOL Right() const { return Button(D_RIGHT); }

			BOOL Start() const { return Button(START); }
			BOOL Back() const { return Button(BACK_SELECT); }
			BOOL Select() const { return Back(); }

			BOOL LeftStickPush() const { return Button(L_STICK_PUSH); }
			BOOL RightStickPush() const { return Button(R_STICK_PUSH); }

			BOOL LeftShoulder() const { return Button(L_SHOULDER); }
			BOOL RightShoulder() const { return Button(R_SHOULDER); }

			BOOL A() const { return Button(A_CROSS); }
			BOOL Cross() const { return A(); }

			BOOL B() const { return Button(B_CIRCLE); }
			BOOL Circle() const { return B(); }

			BOOL X() const { return Button(X_SQUARE); }
			BOOL Square() const { return X(); }

			BOOL Y() const { return Button(Y_TRIANGLE); }
			BOOL Triangle() const { return Y(); }

			BOOL Guide() const { return Button(GUIDE); }

//			friend BOOL operator==(const State &lhs, const State &rhs){ return lhs._state.dwPacketNumber == rhs._state.dwPacketNumber;	}
//			friend BOOL operator!=(const State &lhs, const State &rhs){ return lhs._state.dwPacketNumber != rhs._state.dwPacketNumber ; }

#ifdef _DEBUG
			static char *KindString(Kind k);
			static char *KindString(u32 k) { return KindString(static_cast<Kind>(k)); }
#endif

		private:
			static u32 Xinput(Kind k);
			static BOOL IsDeadzone(s16 x, s16 y, s16 dz) { return Vector2(static_cast<f32>(x), static_cast<f32>(y)).Length() < static_cast<f32>(dz); }

		private:
			XINPUT_STATE _state;
			itl::Bitset _flag;
		};

	public:
		Input() {}

		void Update();

		const State &Previous(u32 i) const { ASSERT(i >= 0 && i < 4); return _state[i][1]; }
		const State &Current(u32 i) const { ASSERT(i >= 0 && i < 4); return _state[i][0]; }

	private:
		static const u32 _padMax = 4; //< pbh͂S܂ŁH
		static const u32 _historyMax = 2; //< iƂ肠jOt[̓͂܂ŗoĂ
		State _state[_padMax][_historyMax];
	};
};

#ifdef _DEBUG
static ostream &operator<<(ostream &lhs, const indi::Input::State &rhs) 
{
	for(indi::u32 i = 0;i < indi::Input::State::KindNum; ++i)
		if(rhs.Button(i)) cout << indi::Input::State::KindString(i) << endl;
	if(rhs.LeftStick() != indi::Vector2::Zero()) cout << "LS : " << rhs.LeftStick() << endl;
	if(rhs.RightStick() != indi::Vector2::Zero()) cout << "RS : " << rhs.RightStick() << endl;
	if(rhs.LeftTriggerf() != 0.0f) cout << "LT : " << rhs.LeftTriggerf() << endl;
	if(rhs.RightTriggerf() != 0.0f) cout << "RT : " << rhs.RightTriggerf() << endl;
	return lhs; 
}
#endif

#endif // INPUT_H

/*!
\brief {^̎ނenumXinputƂ̑Ή\
*/
#ifdef PADASSIGN
PADASSIGN(D_UP,         XINPUT_GAMEPAD_DPAD_UP)
PADASSIGN(D_DOWN,       XINPUT_GAMEPAD_DPAD_DOWN)
PADASSIGN(D_LEFT,       XINPUT_GAMEPAD_DPAD_LEFT)
PADASSIGN(D_RIGHT,      XINPUT_GAMEPAD_DPAD_RIGHT)
PADASSIGN(START,        XINPUT_GAMEPAD_START)
PADASSIGN(BACK_SELECT,  XINPUT_GAMEPAD_BACK)
PADASSIGN(L_STICK_PUSH, XINPUT_GAMEPAD_LEFT_THUMB)
PADASSIGN(R_STICK_PUSH, XINPUT_GAMEPAD_RIGHT_THUMB)
PADASSIGN(L_SHOULDER,   XINPUT_GAMEPAD_LEFT_SHOULDER)
PADASSIGN(R_SHOULDER,   XINPUT_GAMEPAD_RIGHT_SHOULDER)
PADASSIGN(A_CROSS,      XINPUT_GAMEPAD_A)
PADASSIGN(B_CIRCLE,     XINPUT_GAMEPAD_B)
PADASSIGN(X_SQUARE,     XINPUT_GAMEPAD_X)
PADASSIGN(Y_TRIANGLE,   XINPUT_GAMEPAD_Y)
PADASSIGN(GUIDE,        XINPUT_GAMEPAD_BIGBUTTON)
#endif