/*!
$Id: framebuffer.cpp 546 2009-11-25 03:21:03Z horino $
*/
#include "stdafx.h"

#define RENDERTARGET_CPP

#ifdef _DEBUG
#include <iostream>
using namespace std;
#endif

#include "graphics/framebuffer.h"
#include "graphics/tiling.h"
using namespace indi;

/*!
\brief FrameBuffer̃RXgN^
\param Z}eBNX0̃_^[Qbg
\param Z}eBNX1̃_^[Qbg
\param Z}eBNX2̃_^[Qbg
\param Z}eBNX3̃_^[Qbg
\param Z}eBNX4̃_^[Qbg
*/
graphics::FrameBuffer::FrameBuffer(RenderTargetBase *rt0, RenderTargetBase *rt1, RenderTargetBase *rt2, RenderTargetBase *rt3, RenderTargetBase *rt4) 
{ 
#ifdef PREDICATED_TILING
	graphics::Tiling::GetCurrentAddress() = 0;
#endif
	if(rt0) Add(0, rt0);
	if(rt1) Add(1, rt1); 
	if(rt2) Add(2, rt2);
	if(rt3) Add(3, rt3);
	if(rt4) Add(4, rt4);
}

/*!
\brief FrameBuffer̃fXgN^
*/
graphics::FrameBuffer::~FrameBuffer() 
{ 
	_backups.clear(); 
	_rendertargets.clear();

#ifdef PREDICATED_TILING
	_rects.clear(); 
#endif
}

/*!
\brief t[obt@փ_[^[Qbgǉ܂
\param Z}eBNXԍ
\param _[^[Qbg
*/
void graphics::FrameBuffer::Add(u32 i, RenderTargetBase *rt)
{ 
	_rendertargets.push_back(RenderTargetSemantics(i, rt)); 

#ifdef PREDICATED_TILING
	//< _[^[Qbg̃TCY𓾂܂(Z}eBbNOƂ)
	if(i == 0){
		D3DRECT rect[_nrects];
		u32 n(graphics::Tiling::CreateTilingRects(rect, _rendertargets[i].target->GetWidth(), _rendertargets[i].target->GetHeight()));
		for(u32 j=0;j<n;++j)
			_rects.push_back(rect[j]);
	}
#endif
}

/*!
\brief t[obt@w̐FŃNA܂
\param R
\param G
\param B
\param A
*/
void graphics::FrameBuffer::Clear(u8 r, u8 g, u8 b, u8 a)
{ 
	Begin();
	graphics::System::ClearColor(r, g, b, a);
	graphics::System::Clear();
	End();
}

/*!
\brief _[^[Qbgւ̕`̊JnOɃR[܂
*/
void graphics::FrameBuffer::Begin()
{
#ifdef _DEBUG
	ASSERT(!_rendertargets.empty()); //< ̃t[obt@ł
#endif

	for(itl::Vector<RenderTargetSemantics, _ntarget>::iterator it = _rendertargets.begin();it != _rendertargets.end();++it){
		D3DSurface *p;
		if(it->target->IsRenderTarget()){
			if(static_cast<RenderTarget *>(it->target)->GetResolveTexture()){
				//< Aׂ_[^[QbgoĂ
				graphics::System::GetDevice()->GetRenderTarget(it->semantics, &p);
				_backups.push_back(SurfaceSemantics(it->semantics, p));
				//< _[^[Qbg̕ύX
				graphics::System::GetDevice()->SetRenderTarget(it->semantics, it->target->GetRenderTarget());
			}
		} else if(it->target->IsDepthTarget()){
			//< AׂfvX^[QbgoĂ
			graphics::System::GetDevice()->GetDepthStencilSurface(&p);
			_backups.push_back(SurfaceSemantics(0xffffffff, p));
			graphics::System::GetDevice()->SetDepthStencilSurface(it->target->GetRenderTarget());
		}
	}

#ifdef PREDICATED_TILING
	//< vfBP[g^CȌ
	graphics::System::GetDevice()->BeginTiling(D3DTILING_SKIP_FIRST_TILE_CLEAR, _rects.size(), &_rects[0], NULL, 1.0f, 0);
	graphics::System::GetDevice()->SetPredication(D3DPRED_TILE(0)); {
		for(itl::Vector<RenderTargetSemantics, _ntarget>::iterator it=_rendertargets.begin();it!=_rendertargets.end();++it){
			u32 target(0);
			if(it->target->IsRenderTarget()){
				switch(it->semantics){
			    case 0: target = D3DCLEAR_TARGET0; break;
				case 1: target = D3DCLEAR_TARGET1; break;
				case 2: target = D3DCLEAR_TARGET2; break;
				case 3: target = D3DCLEAR_TARGET3; break;
				}
			} else if(it->target->IsDepthTarget())
				target = D3DCLEAR_ZBUFFER | D3DCLEAR_STENCIL;
			graphics::System::GetDevice()->ClearF(target, &_rects[0], NULL, 1.0f, 0);
		}
	} graphics::System::GetDevice()->SetPredication(0);

	graphics::System::GetDevice()->BeginZPass(0);
#endif
}

/*!
\brief _[^[Qbgւ̕`̏IɃR[܂
*/
void graphics::FrameBuffer::End()
{
#ifdef PREDICATED_TILING
#ifdef _DEBUG
	CheckCommandBufferSpace();
#endif

	graphics::System::GetDevice()->EndZPass();

	D3DVECTOR4 cc(graphics::System::GetClearColor());
	for(u32 i=0;i<_rects.size();++i){
		graphics::System::GetDevice()->SetPredication(D3DPRED_TILE(i)); {
			for(itl::Vector<RenderTargetSemantics, _ntarget>::iterator it = _rendertargets.begin();it != _rendertargets.end();++it) {
				if(it->target->IsRenderTarget()){
					graphics::RenderTexture *res_tex(static_cast<RenderTarget *>(it->target)->GetResolveTexture());
					//< ]ueNX`w肳Ăꍇ̓]u
					if(res_tex) {
						graphics::System::GetDevice()->Resolve((D3DRESOLVE_RENDERTARGET0 + it->semantics) | D3DRESOLVE_CLEARRENDERTARGET, 
							&_rects[i],
							res_tex->GetTexture(), 
							reinterpret_cast<D3DPOINT *>(&_rects[i]),
							0, 0, &cc, 1.0f, 0, NULL);
					}
				} 
			}
		} graphics::System::GetDevice()->SetPredication(0);
	}
	graphics::System::GetDevice()->EndTiling(0, NULL, NULL, NULL, 1.0f, 0, NULL);
#else
	for(itl::Vector<RenderTargetSemantics, _ntarget>::iterator it = _rendertargets.begin();it != _rendertargets.end();++it) {
		if(it->target->IsRenderTarget()){
			graphics::RenderTexture *res_tex(static_cast<RenderTarget *>(it->target)->GetResolveTexture());
			if(res_tex) {
				graphics::System::GetDevice()->Resolve((D3DRESOLVE_RENDERTARGET0 + it->semantics) | D3DRESOLVE_CLEARRENDERTARGET,
					NULL,
					it->target->_resolve->GetTexture(), 
					NULL, 0, 0, &cc, 1.0f, 0, NULL);
			}
		}
	}
#endif

	//< _[^[Qbg̕A
	for(itl::Vector<SurfaceSemantics, _ntarget>::iterator it1 = _backups.begin();it1 != _backups.end();++it1){
		if(it1->semantics == 0xffffffff)
			graphics::System::GetDevice()->SetDepthStencilSurface(it1->surface);  
		else 
			graphics::System::GetDevice()->SetRenderTarget(it1->semantics, it1->surface);  

		if(it1->surface) 
			it1->surface->Release(); 
	}
	_backups.clear();
}

#ifdef _DEBUG
void graphics::FrameBuffer::CheckCommandBufferSpace()
{
	u64 used, remain;
	graphics::System::GetDevice()->QueryBufferSpace(&used, &remain);
	u64 total(used + remain);
	u32 per(remain * 100 / total);
	//< c10%
	if(per < 10)
		cout << "COMMAND BUFFER : " << remain << " / " << total << "(" << per << "%)" << endl;
}
#endif

/*!
\brief ̃t[obt@pāAw肳ꂽ_eNX`tXN[`悵܂
\param _eNX`
\note eNX`̃Rs[ȂǂɎg܂
*/
void graphics::FrameBuffer::FullScreenRender(const RenderTexture *t)
{
	Begin();{
		graphics::System::DrawFullScreen(t);
	} End();
}