/*!
$Id: graphics.cpp 640 2009-12-24 07:03:53Z horino $
*/
#include "stdafx.h"

#define GRAPHICS_CPP

#include <D3dx9math.h>
#ifdef _DEBUG
#include <iostream>
using namespace std;
#endif

#include "utilities.h"
#include "graphics/graphics.h"
#include "graphics/shader/shader.h"
#include "primitive/primitive.h"
#include "vector2.h"
#include "vector3.h"
#ifdef PREDICATED_TILING
#include "graphics/framebuffer.h"
#endif

using namespace indi;

D3DDevice *graphics::System::d3dDevice = NULL;
u32 graphics::System::_clearColor = 0;
D3DVIEWPORT9 graphics::System::viewport = {0, 0, 0, 0, 0.0f, 1.0f };
Collision::Frustum graphics::System::frustum;
Matrix44 graphics::System::viewportMatrix;
Matrix44 graphics::System::inverseViewportMatrix;
f32 graphics::System::aspectRatio = 0.0f;

//< TvO
#ifdef PREDICATED_TILING
graphics::System::SampleType graphics::System::sampleType = graphics::System::SAMPLE_2;
#else
graphics::System::SampleType graphics::System::sampleType = graphics::System::SAMPLE_NONE; //< vfBP[g^COgȂƂǂł傤
#endif

//< t[[g
graphics::System::FrameRate graphics::System::frameRate = graphics::System::FRAME_RATE_60;

//< ŏI]up
#ifdef PREDICATED_TILING
D3DSurface *graphics::System::renderTarget = 0;
D3DTexture *graphics::System::frontBuffer = 0;
#endif

#ifdef PREDICATED_TILING
graphics::RenderTexture *graphics::System::bf_color_map = 0;
graphics::RenderTarget *graphics::System::bf_color_target = 0;
graphics::DepthTarget *graphics::System::bf_depth_target = 0;
graphics::FrameBuffer *graphics::System::back_buffer = 0;
#endif

//< tXN[`p
graphics::shader::VertexProgram *graphics::System::fullscreen_vs = 0;
graphics::System::FullScreenPS *graphics::System::fullscreen_ps = 0;

/*!
\brief OtBNXfoCXȂǂ̏
\note Ƃ肠KɂĂ̂ŁAKvɉĕύXĂ
*/
void graphics::System::Initialize()
{
	Direct3D *d3d = Direct3DCreate9(D3D_SDK_VERSION);
	ASSERT(d3d);

	XVIDEO_MODE vm;
	XGetVideoMode(&vm);

	//vm.fIsHiDef
	viewport.Width = vm.dwDisplayWidth;
	viewport.Height = vm.dwDisplayHeight;
	aspectRatio = vm.fIsWideScreen ? (16.0f / 9.0f) : (4.0f / 3.0f);

#ifdef _DEBUG
	cout << "[ SCREEN SIZE " << viewport.Width << " x " << viewport.Height << " ]" << endl;
#endif

	frustum.Create(viewport.X, viewport.Y, viewport.Width, viewport.Height, viewport.MinZ, viewport.MaxZ);

	{
		f32 w(static_cast<f32>(viewport.Width) * 0.5f);
		f32 h(static_cast<f32>(viewport.Height) * 0.5f);
		viewportMatrix = Matrix44(w, 0.0f, 0.0f, 0.0f,
			0.0f, -h, 0.0f, 0.0f,
			0.0f, 0.0f, viewport.MaxZ - viewport.MinZ, 0.0f, 
			static_cast<f32>(viewport.X) + w, static_cast<f32>(viewport.Y) + h, viewport.MinZ, 1.0f);
		inverseViewportMatrix = viewportMatrix.Inverse();
	}

	D3DPRESENT_PARAMETERS pp;
	ZeroMemory(&pp, sizeof(pp));
	pp.BackBufferWidth        = viewport.Width;
	pp.BackBufferHeight       = viewport.Height;
	pp.MultiSampleType        = static_cast<D3DMULTISAMPLE_TYPE>(sampleType);
    pp.SwapEffect             = D3DSWAPEFFECT_DISCARD;
	pp.PresentationInterval   = frameRate; 

#ifdef PREDICATED_TILING
	pp.BackBufferCount = 0;
	pp.BackBufferFormat = (D3DFORMAT)MAKESRGBFMT(D3DFMT_LE_A8R8G8B8);
	pp.FrontBufferFormat = (D3DFORMAT)MAKESRGBFMT(D3DFMT_LE_A8R8G8B8);
	pp.DisableAutoBackBuffer = TRUE;
	pp.DisableAutoFrontBuffer = TRUE;
	pp.EnableAutoDepthStencil = FALSE;
	ASSERT(!FAILED(d3d->CreateDevice(0, D3DDEVTYPE_HAL, NULL, D3DCREATE_BUFFER_2_FRAMES, &pp, &d3dDevice)));
#else
	pp.BackBufferCount = 1;
	pp.BackBufferFormat =  (D3DFORMAT)MAKESRGBFMT(D3DFMT_A8R8G8B8);
	pp.FrontBufferFormat = (D3DFORMAT)MAKESRGBFMT(D3DFMT_LE_X8R8G8B8);
	pp.DisableAutoBackBuffer = FALSE;
	pp.DisableAutoFrontBuffer = FALSE;
	pp.EnableAutoDepthStencil = TRUE;
	pp.AutoDepthStencilFormat = D3DFMT_D24S8;
	ASSERT(!FAILED(d3d->CreateDevice(0, D3DDEVTYPE_HAL, NULL, D3DCREATE_HARDWARE_VERTEXPROCESSING, &pp, &d3dDevice)));
#endif

	D3DRING_BUFFER_PARAMETERS ring = { 0 };
	//ring.SecondarySize = 3 * 1024 * 1024;
	d3dDevice->SetRingBufferParameters(&ring);

#ifdef PREDICATED_TILING
	//< ]ueNX`
	d3dDevice->CreateTexture(
		pp.BackBufferWidth,
		pp.BackBufferHeight,
		1,
		0,
		(D3DFORMAT)MAKESRGBFMT(D3DFMT_LE_X8R8G8B8),
		D3DPOOL_DEFAULT,
		&frontBuffer, 
		NULL);
	//< O_[^[Qbg
	D3DSURFACE_PARAMETERS p = { 0 };
	d3dDevice->CreateRenderTarget(
		pp.BackBufferWidth,
		pp.BackBufferHeight,
		(D3DFORMAT)MAKESRGBFMT(D3DFMT_A8R8G8B8), 
		static_cast<D3DMULTISAMPLE_TYPE>(sampleType),
		0, 
		FALSE,
		&renderTarget,
		&p);
	d3dDevice->SetRenderTarget(0, renderTarget);

	//< obNobt@
	bf_color_map = new graphics::RenderTexture(pp.BackBufferWidth, pp.BackBufferHeight);
	bf_color_target = new graphics::RenderTarget(bf_color_map);
	bf_depth_target = new graphics::DepthTarget(pp.BackBufferWidth, pp.BackBufferHeight);
	back_buffer = new graphics::FrameBuffer(bf_color_target, bf_depth_target);

#endif
	d3dDevice->SetRenderState(D3DRS_CULLMODE, D3DCULL_CW); //< JOfkɍ킹
	//d3dDevice->SetRenderState(D3DRS_ZFUNC, D3DCMP_LESSEQUAL);
}

/*!
\brief 
*/
void graphics::System::Finalize()
{
#ifdef PREDICATED_TILING
	if(renderTarget) { renderTarget->Release(); renderTarget = 0; }
	if(frontBuffer) { frontBuffer->Release(); frontBuffer = 0; }
#endif

#ifdef PREDICATED_TILING
	if(bf_color_map) { delete bf_color_map; bf_color_map = 0; }
	if(bf_color_target) { delete bf_color_target; bf_color_target = 0; }
	if(bf_depth_target) { delete bf_depth_target; bf_depth_target = 0; }
	if(back_buffer) { delete back_buffer; back_buffer = 0; }
#endif

	if(fullscreen_vs) { delete fullscreen_vs; fullscreen_vs = 0; }
	if(fullscreen_ps) { delete fullscreen_ps; fullscreen_ps = 0; }
}

#ifdef PREDICATED_TILING
void graphics::System::BeginBackBuffer() 
{
#ifdef PREDICATED_TILING
	back_buffer->Begin();
#endif
}

graphics::TextureBase *graphics::System::EndBackBuffer() 
{
#ifdef PREDICATED_TILING
	back_buffer->End();
	return bf_color_map;
#else
	return 0;
#endif
}
#endif

/*!
\brief XN[W  [hWϊ
\param [hW
\param [h}gNX
\param r[}gNX
\param vWFNV}gNX
\return XN[W
*/
Vector3 graphics::System::UnProject(const Vector3 &spos, const Matrix44 &world, const Matrix44 &view, const Matrix44 &projection)
{
	D3DXVECTOR3 v;
	return Vector3(*D3DXVec3Unproject(&v, spos, &GetViewport(), projection, view, world));
}

/*!
\brief XN[W  [hWϊ
\param [hW
\param [hW
\\param XN[Wo͐
\param [h}gNX
\param r[}gNX
\param vWFNV}gNX
\return XN[W
*/
Vector3 *graphics::System::UnProject(const Vector3 *spos, u32 n, Vector3 *dpos, const Matrix44 &world, const Matrix44 &view, const Matrix44 &projection)
{
	D3DXVec3UnprojectArray(static_cast<D3DXVECTOR3 *>(*dpos), sizeof(dpos[0]), static_cast<const D3DXVECTOR3 *>(*spos), sizeof(spos[0]), &GetViewport(), projection, view, world, n);
	return dpos;
}

/*!
\brief [hW  XN[Wϊ
\param XN[W
\param [hr[vWFNV}gNX * r[|[g}gNX
\return [hW
\note
)
Vector3 screen(Project(wpos, WVP * graphics::System::GetViewportMatrix()));
*/
Vector3 graphics::System::Project(const Vector3 &wpos, const Matrix44 &wvp_vp)
{
	Vector4 tmp(Vector4(wpos, 1.0f) * wvp_vp);
	return Vector3(tmp) / tmp.W();
}

/*!
\brief (AABB)ʓɂ\傫̔䗦Ԃ܂(LODIȂǂɎg邩)
\param (AABB)SW
\param (AABB)a
\param [hr[vWFNV}gNX * r[|[g}gNX
\return ʓɂ䗦
)
   Vector2 ratio(ScreenRatio(AABB.Center(), AABB.Radius(), WVP * graphics::System::GetViewportMatrix()));
*/
Vector2 graphics::System::ScreenRatio(const Vector3 &center, const Vector3 &radius, const Matrix44 &wvp_vp)
{
	Vector3 mn(Vector3(LIM_MAX(f32), LIM_MAX(f32), LIM_MAX(f32)));
	Vector3 mx(Vector3(LIM_MIN(f32), LIM_MIN(f32), LIM_MIN(f32)));

	Vector3 tmp(Project(center + radius, wvp_vp));
	mn.Set(math::Min(mn.X(), tmp.X()), math::Min(mn.Y(), tmp.Y()), math::Min(mn.Z(), tmp.Z()));
	mx.Set(math::Max(mx.X(), tmp.X()), math::Max(mx.Y(), tmp.Y()), math::Max(mx.Z(), tmp.Z()));

	tmp = Project(center + Vector3(radius.X(), radius.Y(), -radius.Z()), wvp_vp);
	mn.Set(math::Min(mn.X(), tmp.X()), math::Min(mn.Y(), tmp.Y()), math::Min(mn.Z(), tmp.Z()));
	mx.Set(math::Max(mx.X(), tmp.X()), math::Max(mx.Y(), tmp.Y()), math::Max(mx.Z(), tmp.Z()));

	tmp = Project(center + Vector3(radius.X(), -radius.Y(), radius.Z()), wvp_vp);
	mn.Set(math::Min(mn.X(), tmp.X()), math::Min(mn.Y(), tmp.Y()), math::Min(mn.Z(), tmp.Z()));
	mx.Set(math::Max(mx.X(), tmp.X()), math::Max(mx.Y(), tmp.Y()), math::Max(mx.Z(), tmp.Z()));

	tmp = Project(center + Vector3(radius.X(), -radius.Y(), -radius.Z()), wvp_vp);
	mn.Set(math::Min(mn.X(), tmp.X()), math::Min(mn.Y(), tmp.Y()), math::Min(mn.Z(), tmp.Z()));
	mx.Set(math::Max(mx.X(), tmp.X()), math::Max(mx.Y(), tmp.Y()), math::Max(mx.Z(), tmp.Z()));

	tmp = Project(center + Vector3(-radius.X(), radius.Y(), radius.Z()), wvp_vp);
	mn.Set(math::Min(mn.X(), tmp.X()), math::Min(mn.Y(), tmp.Y()), math::Min(mn.Z(), tmp.Z()));
	mx.Set(math::Max(mx.X(), tmp.X()), math::Max(mx.Y(), tmp.Y()), math::Max(mx.Z(), tmp.Z()));

	tmp = Project(center + Vector3(-radius.X(), radius.Y(), -radius.Z()), wvp_vp);
	mn.Set(math::Min(mn.X(), tmp.X()), math::Min(mn.Y(), tmp.Y()), math::Min(mn.Z(), tmp.Z()));
	mx.Set(math::Max(mx.X(), tmp.X()), math::Max(mx.Y(), tmp.Y()), math::Max(mx.Z(), tmp.Z()));

	tmp = Project(center + Vector3(-radius.X(), -radius.Y(), radius.Z()), wvp_vp);
	mn.Set(math::Min(mn.X(), tmp.X()), math::Min(mn.Y(), tmp.Y()), math::Min(mn.Z(), tmp.Z()));
	mx.Set(math::Max(mx.X(), tmp.X()), math::Max(mx.Y(), tmp.Y()), math::Max(mx.Z(), tmp.Z()));

	tmp = Project(center - radius, wvp_vp);
	mn.Set(math::Min(mn.X(), tmp.X()), math::Min(mn.Y(), tmp.Y()), math::Min(mn.Z(), tmp.Z()));
	mx.Set(math::Max(mx.X(), tmp.X()), math::Max(mx.Y(), tmp.Y()), math::Max(mx.Z(), tmp.Z()));

	return Vector2(math::Abs(mx.X() - mn.X()) / static_cast<f32>(GetScreenWidth()), math::Abs(mx.Y() - mn.Y()) / static_cast<f32>(GetScreenHeight()));
}

/*!
\brief w肳ꂽeNX`tXN[`悵܂
\param eNX`
*/
void graphics::System::DrawFullScreen(const graphics::TextureBase *t)
{
	static primitive::ScreenPlane screenPlane;
	if(fullscreen_vs->IsLoaded() && fullscreen_ps->IsLoaded()){
		graphics::shader::Program::AutoBegin auto_vs(*fullscreen_vs);
		graphics::shader::Program::AutoBegin auto_ps(*fullscreen_ps);

		{
			graphics::shader::Parameter::AutoBeginTexture auto_tex(fullscreen_ps->sampler, t);
				
			screenPlane.Begin();{
				screenPlane.Draw();
			} screenPlane.End();
		}
	}
}

/*!
\brief _[obt@̃Xbv
\param XbveNX`
\note XbveNX`𖾎IɎw肵ꍇ͂̃eNX`ɕ`悳ĂeփXbv܂
\note vfBP[g^COgĂꍇ(#ifdef PREDICATED_TILING)́A̎_renderTarget֕`悳ĂefrontBufferփ]uɕ\܂
*/
void graphics::System::SwapBuffers(TextureBase *t) 
{ 
#ifdef PREDICATED_TILING
	if(t) 
		DrawFullScreen(t);
	else
		DrawFullScreen(bf_color_map);

	GetDevice()->SynchronizeToPresentationInterval();
	GetDevice()->Resolve(D3DRESOLVE_RENDERTARGET0,
		NULL,
		frontBuffer,
		NULL,
		0,
		0,
		NULL,
		1.0f, 
		0,
		NULL);
	GetDevice()->Swap(frontBuffer, NULL);
#else
	if(t) 
		DrawFullScreen(t);
	GetDevice()->Present(NULL, NULL, NULL, NULL); 
#endif
}

/*!
\brief tXN[`ɕKvȃVF[_pӂ܂
*/
void graphics::System::CreateFullScreenShader()
{
	fullscreen_vs = new graphics::shader::VertexProgram("fullscreen_vs"SH_EXT);
	fullscreen_ps = new FullScreenPS("fullscreen_ps"SH_EXT);
}
