/*!
$Id: rendertarget.cpp 545 2009-11-24 08:32:09Z horino $
*/
#include "stdafx.h"

#define RENDERTARGET_CPP

#ifdef _DEBUG
#include <iostream>
using namespace std;
#endif

#include "utilities.h"

#include "graphics/rendertarget.h"
#include "graphics/framebuffer.h"
#include "graphics/tiling.h"
using namespace indi;

/*!
\brief _^[Qbg쐬܂
*/
void graphics::RenderTarget::CreateRenderTarget(u32 w, u32 h)
{
#ifdef PREDICATED_TILING
	D3DRECT rect[graphics::FrameBuffer::_nrects];
	u32 n(graphics::Tiling::CreateTilingRects(rect, w, h));   

#ifdef _DEBUG
	for(u32 i=0;i<n;++i)
		cout << "[" << i << "] : " << rect[i].x1 << ", " << rect[i].y1 << ", " << rect[i].x2 << ", " << rect[i].y2 << endl;
#endif

	U32Vector2 tile_size(graphics::Tiling::GetTileMax(rect, n));
#ifdef _DEBUG
	cout << "Tile : " << tile_size.x << " x " << tile_size.y;
#endif

	graphics::Tiling::AdjustTileSize(tile_size, n > 1);

#ifdef _DEBUG
	cout << "(" << tile_size.x << " x " << tile_size.y << ")" << endl << endl;
#endif

	D3DSURFACE_PARAMETERS p = { 0 };
	//< _^[Qbg쐬
	if(_resolve){
		p.Base = graphics::Tiling::GetCurrentAddress() / GPU_EDRAM_TILE_SIZE;
		ASSERT(S_OK == graphics::System::GetDevice()->CreateRenderTarget(
			tile_size.x,
			tile_size.y,
			static_cast<D3DFORMAT>(_resolve->Format()),
			static_cast<D3DMULTISAMPLE_TYPE>(graphics::System::GetSampleType()), 
			0, 
			FALSE,
			&_rendertarget, 
			&p));
		graphics::Tiling::GetCurrentAddress() += _rendertarget->Size;
	}
#else
	if(_resolve){
		ASSERT(S_OK == graphics::System::GetDevice()->CreateRenderTarget(
			w,
			h, 
			static_cast<D3DFORMAT>(resolve->Format()),
			static_cast<D3DMULTISAMPLE_TYPE>(graphics::System::GetSampleType()), 
			0, 
			FALSE,
			&_rendertarget, 
			NULL));
	}
#endif
}

/*!
\brief fvX^[Qbg쐬܂
*/
void graphics::DepthTarget::CreateDepthTarget(u32 w, u32 h)
{
#ifdef PREDICATED_TILING
	D3DRECT rect[graphics::FrameBuffer::_nrects];
	u32 n(graphics::Tiling::CreateTilingRects(rect, w, h));   
	U32Vector2 tile_size(graphics::Tiling::GetTileMax(rect, n));
	graphics::Tiling::AdjustTileSize(tile_size, n > 1);

	D3DSURFACE_PARAMETERS p = { 0 };
	p.Base = graphics::Tiling::GetCurrentAddress() / GPU_EDRAM_TILE_SIZE;
	ASSERT(S_OK == graphics::System::GetDevice()->CreateDepthStencilSurface(
		tile_size.x,
		tile_size.y,
		D3DFMT_D24S8,		
		static_cast<D3DMULTISAMPLE_TYPE>(graphics::System::GetSampleType()),
		0, 
		FALSE,
		&_rendertarget, 
		&p));
	graphics::Tiling::GetCurrentAddress() += _rendertarget->Size;
#else
	if(is_depth){
		ASSERT(S_OK == graphics::System::GetDevice()->CreateDepthStencilSurface(
			w,
			h, 
			D3DFMT_D24S8,		
			static_cast<D3DMULTISAMPLE_TYPE>(graphics::System::GetSampleType()),
			0, 
			FALSE,
			&static_cast<DepthTarget *>(this)->_rendertarget, 
			NULL));
	} 
#endif
}