/*!
$Id: shader.cpp 624 2009-12-17 07:37:52Z horino $
*/
#include "stdafx.h"

#define SHADER_CPP

#ifdef _DEBUG
#include <iostream>
using namespace std;
#endif

#include "graphics/graphics.h"
#include "graphics/texture.h"
#include "graphics/shader/shader.h"
#include "vector2.h"
#include "vector3.h"
#include "vector4.h"
#include "matrix44.h"
using namespace indi;

#ifndef USE_REGISTER
/*!
\brief p[^p[^擾
\param p[^
*/
graphics::shader::Parameter graphics::shader::ProgramFile::GetParameter(const char *name)
{
	ASSERT(_table);
	return Parameter(_table->GetConstantByName(NULL, name), _table);
}
#endif

/*!
\brief VF[_p[^f32Zbg܂
\param f32
*/
void graphics::shader::Parameter::operator=(f32 lhs) 
{
#ifdef USE_REGISTER
	ASSERT(_parameter != 0xffffffff);
	if(IsPixel())
		graphics::System::GetDevice()->SetPixelShaderConstantF(_parameter, Vector4(lhs, 0.0f, 0.0f, 0.0f), 1); 
	else
		graphics::System::GetDevice()->SetVertexShaderConstantF(_parameter, Vector4(lhs, 0.0f, 0.0f, 0.0f), 1); 
#else
#ifdef _DEBUG
	NAN_CHECK(lhs);
	if(_table && _handle) 
		ASSERT(D3D_OK == _table->SetFloat(graphics::System::GetDevice(), _handle, lhs));
	else 
		cout << "Cannot Set Shader Parameter" << __FILE__ << " : " << __LINE__ << endl;
#else
	_table->SetFloat(graphics::System::GetDevice(), _handle, lhs);
#endif
#endif
}

/*!
\brief VF[_p[^ɂQxNgZbg܂
\param const Vector2 &
*/
void graphics::shader::Parameter::operator=(const Vector2 &lhs) 
{
#ifdef USE_REGISTER
	ASSERT(_parameter != 0xffffffff);
	if(IsPixel())
		graphics::System::GetDevice()->SetPixelShaderConstantF(_parameter, Vector4(lhs, 0.0f, 0.0f), 1); 
	else
		graphics::System::GetDevice()->SetVertexShaderConstantF(_parameter, Vector4(lhs, 0.0f, 0.0f), 1); 
#else
#ifdef _DEBUG
	lhs.NanCheck();
	if(_table && _handle) 
		ASSERT(D3D_OK == _table->SetVector(graphics::System::GetDevice(), _handle, Vector4(lhs, 0.0f, 0.0f)));
	else 
		cout << "Cannot Set Shader Parameter" << __FILE__ << " : " << __LINE__ << endl;
#else
	_table->SetVector(graphics::System::GetDevice(), _handle, Vector4(lhs, 0.0f, 0.0f));
#endif
#endif
}

/*!
\brief VF[_p[^ɂRxNgZbg܂
\param const Vector3 &
*/
void graphics::shader::Parameter::operator=(const Vector3 &lhs) 
{
#ifdef USE_REGISTER
	ASSERT(_parameter != 0xffffffff);
	if(IsPixel())
		graphics::System::GetDevice()->SetPixelShaderConstantF(_parameter, Vector4(lhs, 0.0f), 1); 
	else
		graphics::System::GetDevice()->SetVertexShaderConstantF(_parameter, Vector4(lhs, 0.0f), 1); 
#else
#ifdef _DEBUG
	lhs.NanCheck();
	if(_table && _handle) 
		ASSERT(D3D_OK == _table->SetVector(graphics::System::GetDevice(), _handle, Vector4(lhs, 0.0f)));
	else 
		cout << "Cannot Set Shader Parameter" << __FILE__ << " : " << __LINE__ << endl;
#else
	_table->SetVector(graphics::System::GetDevice(), _handle, Vector4(lhs, 0.0f));
#endif
#endif
}

/*!
\brief VF[_p[^ɂSxNgZbg܂
\param const Vector4 &
*/
void graphics::shader::Parameter::operator=(const Vector4 &lhs) 
{
#ifdef USE_REGISTER
	ASSERT(_parameter != 0xffffffff);
	if(IsPixel())
		graphics::System::GetDevice()->SetPixelShaderConstantF(_parameter, lhs, 1); 
	else
		graphics::System::GetDevice()->SetVertexShaderConstantF(_parameter, lhs, 1); 
#else
#ifdef _DEBUG
	lhs.NanCheck();
	if(_table && _handle) 
		ASSERT(D3D_OK == _table->SetVector(graphics::System::GetDevice(), _handle, lhs));
	else 
		cout << "Cannot Set Shader Parameter" << __FILE__ << " : " << __LINE__ << endl;
#else
	_table->SetVector(graphics::System::GetDevice(), _handle, lhs);
#endif
#endif
}

/*!
\brief VF[_p[^Ƀ}gNXZbg܂
\param const Matrix44 &
*/
void graphics::shader::Parameter::operator=(const Matrix44 &lhs) 
{
#ifdef USE_REGISTER
	ASSERT(_parameter != 0xffffffff);
	if(IsPixel())
		graphics::System::GetDevice()->SetPixelShaderConstantF(_parameter, lhs, 4); 
	else
		graphics::System::GetDevice()->SetVertexShaderConstantF(_parameter, lhs, 4); 
#else
#ifdef _DEBUG
	lhs.NanCheck();
	if(_table && _handle) 
		ASSERT(D3D_OK == _table->SetMatrixTranspose(graphics::System::GetDevice(), _handle, lhs));
	else 
		cout << "Cannot Set Shader Parameter" << __FILE__ << " : " << __LINE__ << endl;
#else
	_table->SetMatrixTranspose(graphics::System::GetDevice(), _handle, lhs);
#endif
#endif
}

/*!
\brief VF[_p[^ɃeNX`Zbg܂
\param const graphics::TextureBase *
*/
void graphics::shader::Parameter::operator=(const graphics::TextureBase *lhs) 
{ 
	D3DBaseTexture *tex(const_cast<D3DBaseTexture *>(lhs->GetTexture())); 
#ifdef USE_REGISTER
	ASSERT(_parameter!=0xffffffff);
	graphics::System::GetDevice()->SetTexture(_parameter, tex); 
	if(tex->GetLevelCount() > 1) 
		graphics::System::GetDevice()->SetSamplerState(_parameter, D3DSAMP_MIPFILTER, D3DTEXF_LINEAR);
	else
		graphics::System::GetDevice()->SetSamplerState(_parameter, D3DSAMP_MIPFILTER, D3DTEXF_NONE);   
#else
	if(_table && _handle) {
		u32 smp(_table->GetSamplerIndex(_handle));
#ifdef _DEBUG
		ASSERT(smp != 0xffffffff);
#endif
		//< Aj[VeNX`̏ꍇ̐ݒ
		if(lhs->IsAnimationTexture()){
			graphics::System::GetDevice()->SetTexture(D3DVERTEXTEXTURESAMPLER0 + smp, tex); 
			graphics::System::GetDevice()->SetSamplerState(smp, D3DSAMP_MIPFILTER, D3DTEXF_NONE);   
			graphics::System::GetDevice()->SetSamplerState(smp, D3DSAMP_MAGFILTER, D3DTEXF_POINT);   
			graphics::System::GetDevice()->SetSamplerState(smp, D3DSAMP_MINFILTER, D3DTEXF_POINT);   
			graphics::System::GetDevice()->SetSamplerState(smp, D3DSAMP_ADDRESSU, D3DTADDRESS_CLAMP);   
			graphics::System::GetDevice()->SetSamplerState(smp, D3DSAMP_ADDRESSV, D3DTADDRESS_CLAMP);   
			graphics::System::GetDevice()->SetSamplerState(smp, D3DSAMP_TRILINEARTHRESHOLD, D3DTRILINEAR_THREEEIGHTHS);
		} else {
			graphics::System::GetDevice()->SetTexture(smp, tex); 
			if(tex->GetLevelCount() > 1) 
				graphics::System::GetDevice()->SetSamplerState(smp, D3DSAMP_MIPFILTER, D3DTEXF_LINEAR);
			else
				graphics::System::GetDevice()->SetSamplerState(smp, D3DSAMP_MIPFILTER, D3DTEXF_NONE);   
		}
	} 
#ifdef _DEBUG
	else 
		cout << "Cannot Set Shader Parameter" << __FILE__ << " : " << __LINE__ << endl;
#endif
#endif
}

/*!
\brief eNX`̃fBZ[u
*/
void graphics::shader::Parameter::Disable()
{
#ifdef USE_REGISTER
	ASSERT(_parameter!=0xffffffff);
	graphics::System::GetDevice()->SetTexture(_parameter, 0); 
#else
#ifdef _DEBUG
	if(_table && _handle) 
		ASSERT(S_OK == graphics::System::GetDevice()->SetTexture(_table->GetSamplerIndex(_handle), 0)); 
	else 
		cout << "Cannot Set Shader Parameter" << __FILE__ << " : " << __LINE__ << endl;
#else
	graphics::System::GetDevice()->SetTexture(_table->GetSamplerIndex(_handle), 0);
#endif
#endif
} 

/*!
\brief o[ebNXVF[_vOǂݍ܂ꂽƂ̃R[obN
*/
void graphics::shader::VertexProgram::OnLoaded() 
{
	IDirect3DVertexShader9 *v;
	ASSERT(S_OK == graphics::System::GetDevice()->CreateVertexShader(reinterpret_cast<const u64 *>(GetData()), &v));
	_program = v;

#ifndef USE_REGISTER
	ASSERT(D3D_OK ==D3DXGetShaderConstantTable(reinterpret_cast<const DWORD *>(GetData()), &_table));	
#endif
}

/*!
\brief o[ebNXVF[_̊Jn
*/
void graphics::shader::VertexProgram::Begin() 
{
#ifndef USE_REGISTER
	ASSERT(_table); //< IsLoaded()`FbNĂ܂?
	ASSERT(D3D_OK == _table->SetDefaults(graphics::System::GetDevice()));
#endif
	ASSERT(S_OK == graphics::System::GetDevice()->SetVertexShader(_program)); 
}

/*!
\brief sNZVF[_vOǂݍ܂ꂽƂ̃R[obN
*/
void graphics::shader::PixelProgram::OnLoaded() 
{
	IDirect3DPixelShader9 *p; 
	ASSERT(S_OK == graphics::System::GetDevice()->CreatePixelShader(reinterpret_cast<const u64 *>(GetData()), &p));
	_program = p;

#ifndef USE_REGISTER
	ASSERT(D3D_OK == D3DXGetShaderConstantTable(reinterpret_cast<const DWORD *>(GetData()), &_table));
#endif
}

/*!
\brief sNZVF[_̊Jn
*/
void graphics::shader::PixelProgram::Begin() 
{
#ifndef USE_REGISTER
	ASSERT(_table);
	ASSERT(D3D_OK == _table->SetDefaults(graphics::System::GetDevice()));
#endif
	ASSERT(S_OK == graphics::System::GetDevice()->SetPixelShader(_program)); 
}
