/*!
$Id: quaternion.cpp 295 2009-08-20 05:16:37Z horino $
*/
#include "stdafx.h"

#define QUATERNION_CPP

#include "vector3.h"
#include "vector4.h"
#include "quaternion.h"
#include "matrix44.h"
#include "utilities.h"

using namespace indi;

Quaternion::Quaternion(const Vector3 &v3) : _e(v3.X(), v3.Y(), v3.Z(), 0.0f) {}
Quaternion::Quaternion(const Matrix44 &m44) : _e(*D3DXQuaternionRotationMatrix(&_e, m44)) {}

/*!
  \brief ]ƊpxiWAjl쐬RXgN^
  \param ]iKĂKv܂j
  \param ]iWAj
*/
Quaternion::Quaternion(const Vector3 &axis, f32 rad) : _e(*D3DXQuaternionRotationAxis(&_e, axis, rad)) {}

/*!
  \brief xNgfromxNgto֌悤ȉ]\l쐬RXgN^
  \param ]xNg
  \param ]xNg
  \note Zbg͐ł܂
*/
Quaternion::Quaternion(const Vector3 &from, const Vector3 &to)
{
	f32 c(from * to);	
	if(c > 0.999f)
		*this = Identity();
	else if(c < -0.999f){
		Vector3 t(from.Z(), 0.0f, -from.X());
		f32 len2(t.LengthSquared());
		if(len2 < LIM_EPSILON(f32))
			t.Set(0.0f, -from.Z(), from.Y()).Normalize();
		else 
			t /= math::Sqrt(len2);
		Set(t.X(), t.Y(), t.Z(), 0.0f);
	} else {
		Vector3 t(from ^ to);
		if(t.LengthSquared() > LIM_EPSILON(f32)){
			t.Adjust(math::Sqrt((0.5f * (1.0f - c))));
			Set(t.X(), t.Y(), t.Z(), math::Sqrt((0.5f * (1.0f + c))));
		} else 
			*this = Identity();
	}
}

Quaternion::Quaternion(const Vector4 &v4) : _e(v4.X(), v4.Y(), v4.Z(), v4.W()) {}

/*!
  \brief xNglŉ]܂(l͐KĂKv܂)
  \param ]xNg
  \return ]̃xNg
*/
Vector3 Quaternion::Rotate(const Vector3 &v) const { return Vector3(~(*this) * v * (*this)); }

void Quaternion::ToAxisRadian(Vector3 &axis, f32 &rad) const { D3DXQuaternionToAxisAngle(&_e, axis, &rad); }
Quaternion Quaternion::RotationX(f32 rad) { return Quaternion(Vector3::UnitX(), rad); }
Quaternion Quaternion::RotationY(f32 rad) { return Quaternion(Vector3::UnitY(), rad); }
Quaternion Quaternion::RotationZ(f32 rad) { return Quaternion(Vector3::UnitZ(), rad); }
